/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "../server-error.h"
#include "../client-interface.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct finite_automata_struc *finite_automata;
struct stack_automata_struc *stack_automata;
struct regular_grammar_struc *regular_grammar;
struct contextfree_grammar_struc *contextfree_grammar;

char *q0_id;
char *q1_id;
char *q2_id;
char *q3_id;
char *q4_id;
char *q5_id;
char *q6_id;
char *q7_id;
char *s0_id;
char *s1_id;
char *ss0_id;
char *ss1_id;
char *nts0_id;
char *nts1_id;
char *nts2_id;
char *nts3_id;
char *nts4_id;
char *ts0_id;
char *ts1_id;
char *ts2_id;
char *ts3_id;
char *ts4_id;

char *
get_id(char *str, char *after)
{
  char *result, *iter_s;
  int skip = 1;
  result = calloc(100, 1);
  strcpy(result, "");
  for (iter_s = str; *iter_s != 0; ++iter_s) {
    if (*iter_s == ' ') { 
      if (!skip) break;
      if (!strcmp(result, after)) skip = 0;
      strcpy(result, "");
      continue;
    }
    if (*iter_s != '(') strncat(result, iter_s, 1);
    if (*iter_s == ')') strcpy(result, "");
  }
  return result;
}

void
vypis_finite (struct finite_automata_struc *automata)
{
  printf("%s\n", finite_automata_get_look(automata));
}

void
vypis_stack(struct stack_automata_struc *automata)
{
  printf("%s\n", stack_automata_get_look(automata));
}

void
vypis_regular (struct regular_grammar_struc *grammar)
{
  printf("%s\n", regular_grammar_get_look(grammar));
}

void
vypis_contextfree (struct contextfree_grammar_struc *grammar)
{
  printf("%s\n", contextfree_grammar_get_look(grammar));
}

void
veta_finite (struct finite_automata_struc *automata)
{
  char *sentence, *dests, *c_state, *c_sentence;
  sentence = calloc(100, 1);
  strcpy(sentence, s1_id); strcat(sentence, " "); strcat(sentence, s0_id);
  strcat(sentence, " "); strcat(sentence, s1_id); strcat(sentence, " ");
  strcat(sentence, s0_id);
  finite_automata_set_configuration(automata, q0_id, sentence);
  dests = finite_automata_get_transition_destinations(automata);
  finite_automata_get_configuration(automata, &c_state, &c_sentence);
  printf ("Veta: %s ", c_sentence);
  if (finite_automata_check_sentence(automata, sentence))
    printf("... yes\n"); 
  else printf("... no\n");
  printf ("Stav: %s, Veta: %s\n", c_state, c_sentence);
  while (strlen(get_id(dests, "state"))) {
    finite_automata_do_transition(automata, get_id(dests, "state"));
    free(dests);
    dests = finite_automata_get_transition_destinations(automata);
    free(c_state); free(c_sentence);
    finite_automata_get_configuration(automata, &c_state, &c_sentence);
    printf ("Stav: %s, Veta: %s\n", c_state, c_sentence);
  }
  free(sentence); free(dests); free(c_state); free(c_sentence);
}

void
veta_stack (struct stack_automata_struc *automata)
{
  char *sentence, *dests, *c_state, *c_sentence, *c_stack, *from_stacks;
  sentence = calloc(100, 1);
  strcpy(sentence, s0_id); strcat(sentence, " "); strcat(sentence, s0_id);
  strcat(sentence, " "); strcat(sentence, s1_id); strcat(sentence, " ");
  strcat(sentence, s1_id);
  stack_automata_set_configuration(automata, q0_id, sentence, ss0_id);
  from_stacks = stack_automata_get_from_stacks(automata);
  dests = 
  stack_automata_get_transition_destinations
  (automata, get_id(from_stacks, "stack_symbol"));
  stack_automata_get_configuration(automata, &c_state, &c_sentence, &c_stack);
  printf ("Veta: %s ", c_sentence);
  if (stack_automata_check_sentence(automata, sentence)) printf("... yes\n"); 
  else printf("... no\n");
  printf ("Stav: %s, Veta: %s, Zasobnik: %s\n", c_state, c_sentence, c_stack);
  while (strlen(get_id(dests, "state"))) {
    // some cheat
    if (strlen(get_id(dests, "stack_symbol"))) {
      strcpy(c_stack, get_id(dests, "stack_symbol")); strcat(c_stack, " ");
      strcat(c_stack, ss1_id);
    }
    else { free(c_stack); c_stack = strdup(""); }
    stack_automata_do_transition
      (automata, get_id(dests, "state"), get_id(from_stacks, "stack_symbol"),
       c_stack);
    free(from_stacks);
    from_stacks = stack_automata_get_from_stacks(automata);
    free(dests);
    dests = 
    stack_automata_get_transition_destinations
    (automata, get_id(from_stacks, "stack_symbol"));
    free(c_state); free(c_sentence); free(c_stack);
    stack_automata_get_configuration
      (automata, &c_state, &c_sentence, &c_stack);
    printf
      ("Stav: %s, Veta: %s, Zasobnik: %s\n", c_state, c_sentence, c_stack);
  }
  free(sentence); free(dests); free(c_state); free(c_sentence); free(c_stack);
  free(from_stacks);
}

void
demo ()
{
  int choice;
  char *stack;
  printf("\n");
  printf("1 - finite deterministic reduced automata\n2 - finite nondeterministic automata\n3 - finite deterministic nonreduced automata \n4 - stack nondeterministic top stack working automata\n5 - regular grammar\n6 - context-free grammar with redundant symbols\n7 - context-free grammar with inaccessible symbols\n8 - context-free grammar with e-rules and simple rules\n9 - left recursive context-free grammar\n10 or 11 - context-free grammar for conversion to Chomsky or Greibach normal form\n12 or 13 - context-free grammar for conversion to the stack automata (top-down or bottom-up)\n14 - stack automata for conversion to the contextfree grammar\n\nYour choice: ");
  scanf("%i", &choice);
  if ((choice > 0) && (choice < 4))
    finite_automata = formal_new_finite_automata();
  else if ((choice == 4) || (choice == 14)) 
    stack_automata = formal_new_stack_automata();
  else if (choice == 5) regular_grammar = formal_new_regular_grammar();
  else if ((choice >= 6) && (choice <= 13)) 
    contextfree_grammar = formal_new_contextfree_grammar();
  if (choice == 1) {
    q0_id = get_id(finite_automata_add_state(finite_automata, "q0"), "state");
    q1_id = get_id(finite_automata_add_state(finite_automata, "q1"), "state");
    q2_id = get_id(finite_automata_add_state(finite_automata, "q2"), "state");
    q3_id = get_id(finite_automata_add_state(finite_automata, "q3"), "state");
    finite_automata_set_start_state(finite_automata, q0_id);
    finite_automata_set_end_state(finite_automata, q3_id);
    s0_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "0"), "entry_symbol");
    s1_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "1"), "entry_symbol");
    finite_automata_add_transition(finite_automata, q0_id, s0_id, q2_id);
    finite_automata_add_transition(finite_automata, q0_id, s1_id, q1_id);
    finite_automata_add_transition(finite_automata, q1_id, s0_id, q3_id);
    finite_automata_add_transition(finite_automata, q1_id, s1_id, q0_id);
    finite_automata_add_transition(finite_automata, q2_id, s0_id, q0_id);
    finite_automata_add_transition(finite_automata, q2_id, s1_id, q3_id);
    finite_automata_add_transition(finite_automata, q3_id, s0_id, q1_id);
    finite_automata_add_transition(finite_automata, q3_id, s1_id, q2_id);
  }
  else if (choice == 2) {
    q0_id = get_id(finite_automata_add_state(finite_automata, "q0"), "state");
    q1_id = get_id(finite_automata_add_state(finite_automata, "q1"), "state");
    q2_id = get_id(finite_automata_add_state(finite_automata, "q2"), "state");
    q3_id = get_id(finite_automata_add_state(finite_automata, "q3"), "state");
    finite_automata_set_start_state(finite_automata, q0_id);
    finite_automata_set_end_state(finite_automata, q3_id);
    s0_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "0"), "entry_symbol");
    s1_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "1"), "entry_symbol");
    finite_automata_add_transition(finite_automata, q0_id, s0_id, q0_id);
    finite_automata_add_transition(finite_automata, q0_id, s0_id, q1_id);
    finite_automata_add_transition(finite_automata, q0_id, s1_id, q0_id);
    finite_automata_add_transition(finite_automata, q0_id, s1_id, q2_id);
    finite_automata_add_transition(finite_automata, q1_id, s0_id, q1_id);
    finite_automata_add_transition(finite_automata, q1_id, s0_id, q3_id);
    finite_automata_add_transition(finite_automata, q1_id, s1_id, q1_id);
    finite_automata_add_transition(finite_automata, q2_id, s0_id, q2_id);
    finite_automata_add_transition(finite_automata, q2_id, s1_id, q2_id);
    finite_automata_add_transition(finite_automata, q2_id, s1_id, q3_id);
  }
  else if (choice == 3) {
    q0_id = get_id(finite_automata_add_state(finite_automata, "q0"), "state");
    q1_id = get_id(finite_automata_add_state(finite_automata, "q1"), "state");
    q2_id = get_id(finite_automata_add_state(finite_automata, "q2"), "state");
    q3_id = get_id(finite_automata_add_state(finite_automata, "q3"), "state");
    q4_id = get_id(finite_automata_add_state(finite_automata, "q4"), "state");
    q5_id = get_id(finite_automata_add_state(finite_automata, "q5"), "state");
    q6_id = get_id(finite_automata_add_state(finite_automata, "q6"), "state");
    q7_id = get_id(finite_automata_add_state(finite_automata, "q7"), "state");
    finite_automata_set_start_state(finite_automata, q0_id);
    finite_automata_set_end_state(finite_automata, q0_id);
    finite_automata_set_end_state(finite_automata, q4_id);
    s0_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "0"), "entry_symbol");
    s1_id = 
      get_id
      (finite_automata_add_entry_symbol(finite_automata, "1"), "entry_symbol");
    finite_automata_add_transition(finite_automata, q0_id, s0_id, q4_id);
    finite_automata_add_transition(finite_automata, q0_id, s1_id, q3_id);
    finite_automata_add_transition(finite_automata, q1_id, s0_id, q2_id);
    finite_automata_add_transition(finite_automata, q1_id, s1_id, q3_id);
    finite_automata_add_transition(finite_automata, q2_id, s0_id, q2_id);
    finite_automata_add_transition(finite_automata, q2_id, s1_id, q0_id);
    finite_automata_add_transition(finite_automata, q3_id, s0_id, q5_id);
    finite_automata_add_transition(finite_automata, q3_id, s1_id, q2_id);
    finite_automata_add_transition(finite_automata, q4_id, s0_id, q4_id);
    finite_automata_add_transition(finite_automata, q4_id, s1_id, q5_id);
    finite_automata_add_transition(finite_automata, q5_id, s0_id, q3_id);
    finite_automata_add_transition(finite_automata, q5_id, s1_id, q7_id);
    finite_automata_add_transition(finite_automata, q6_id, s0_id, q7_id);
    finite_automata_add_transition(finite_automata, q6_id, s1_id, q4_id);
    finite_automata_add_transition(finite_automata, q7_id, s0_id, q7_id);
    finite_automata_add_transition(finite_automata, q7_id, s1_id, q4_id);
  }
  else if (choice == 4) {
    q0_id = get_id(stack_automata_add_state(stack_automata, "q0"), "state");
    q1_id = get_id(stack_automata_add_state(stack_automata, "q1"), "state");
    q2_id = get_id(stack_automata_add_state(stack_automata, "q2"), "state");
    stack_automata_set_start_state(stack_automata, q0_id);
    stack_automata_set_end_state(stack_automata, q0_id);
    s0_id = 
    get_id
    (stack_automata_add_entry_symbol(stack_automata, "0"), "entry_symbol");
    s1_id = 
    get_id
    (stack_automata_add_entry_symbol(stack_automata, "1"), "entry_symbol");
    ss0_id = 
    get_id
    (stack_automata_add_stack_symbol(stack_automata, "Z"), "stack_symbol");
    ss1_id = 
    get_id
    (stack_automata_add_stack_symbol(stack_automata, "0"), "stack_symbol");
    stack_automata_set_start_stack_symbol(stack_automata, ss0_id);
    stack = calloc(100, 1);
    strcpy(stack, ss0_id); strcat(stack, " "); strcat(stack, ss1_id);
    stack_automata_add_transition
    (stack_automata, q0_id, s0_id, q1_id, ss0_id, stack);
    strcpy(stack, ss1_id); strcat(stack, " "); strcat(stack, ss1_id);
    stack_automata_add_transition
    (stack_automata, q1_id, s0_id, q1_id, ss1_id, stack);
    stack_automata_add_transition
    (stack_automata, q1_id, s1_id, q2_id, ss1_id, "");
    stack_automata_add_transition
    (stack_automata, q2_id, s1_id, q2_id, ss1_id, "");
    stack_automata_add_transition
    (stack_automata, q2_id, "0", q0_id, ss0_id, "");
    free(stack);
  }
  else if (choice == 5) {
    nts0_id = 
      get_id(regular_grammar_add_nonterminal_symbol
	     (regular_grammar, "C"), "nonterminal_symbol");
    nts1_id = 
      get_id(regular_grammar_add_nonterminal_symbol
	     (regular_grammar, "D"), "nonterminal_symbol");
    regular_grammar_set_start_symbol(regular_grammar, nts0_id);
    ts0_id = 
    get_id(regular_grammar_add_terminal_symbol
	   (regular_grammar, "+"), "terminal_symbol");
    ts1_id = 
    get_id(regular_grammar_add_terminal_symbol
	   (regular_grammar, "-"), "terminal_symbol");
    ts2_id = 
    get_id(regular_grammar_add_terminal_symbol
	   (regular_grammar, "0"), "terminal_symbol");
    ts3_id = 
    get_id(regular_grammar_add_terminal_symbol
	   (regular_grammar, "1"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts0_id, stack);
    strcpy(stack, ts1_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts0_id, stack);
    regular_grammar_add_rule(regular_grammar, nts0_id, ts2_id);
    regular_grammar_add_rule(regular_grammar, nts0_id, ts3_id);
    strcpy(stack, ts2_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts0_id, stack);
    strcpy(stack, ts3_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts0_id, stack);
    regular_grammar_add_rule(regular_grammar, nts1_id, ts2_id);
    regular_grammar_add_rule(regular_grammar, nts1_id, ts3_id);
    strcpy(stack, ts2_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts1_id, stack);
    strcpy(stack, ts3_id); strcat(stack, " "); strcat(stack, nts1_id);
    regular_grammar_add_rule(regular_grammar, nts1_id, stack);
    free(stack);
  }
  else if (choice == 6) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "A"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "B"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "C"), "nonterminal_symbol");
    nts3_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "D"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "b"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "c"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts1_id);
    strcat(stack, " "); strcat(stack, ts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, nts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts2_id);
    strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    strcpy(stack, nts3_id); strcat(stack, " "); strcat(stack, nts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, ts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, ts0_id);
    strcpy(stack, nts3_id); strcat(stack, " "); strcat(stack, nts1_id);
    strcat(stack, " "); strcat(stack, nts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, "");
    free(stack);
  }
  else if (choice == 7) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "A"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "B"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "C"), "nonterminal_symbol");
    nts3_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "D"), "nonterminal_symbol");
    nts4_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "E"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "b"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "c"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts4_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts3_id); strcat(stack, " "); strcat(stack, nts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, ts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, nts4_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, ts1_id);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, stack);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, stack);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts0_id);
    strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts4_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts4_id, ts1_id);
    free(stack);
  }
  else if (choice == 8) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "S"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "A"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "B"), "nonterminal_symbol");
    nts3_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "C"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "b"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "c"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts0_id);
    strcat(stack, " "); strcat(stack, nts2_id); strcat(stack, " "); 
    strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, ts0_id); strcat(stack, " "); strcat(stack, nts3_id);
    strcat(stack, " "); strcat(stack, ts0_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, nts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, nts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, ts0_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, "");
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, ts1_id);
    strcpy(stack, ts2_id); strcat(stack, " "); strcat(stack, nts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, nts0_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts3_id, "");
    free(stack);
  }
  else if (choice == 9) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "A"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "B"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "C"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "b"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "c"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts0_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, ts2_id);
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, nts0_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    strcpy(stack, ts1_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    strcpy(stack, nts0_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    free(stack);
  }
  else if ((choice == 10) || (choice == 11)) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "E"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "T"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "F"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "+"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "*"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "("), "terminal_symbol");
    ts3_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, ")"), "terminal_symbol");
    ts4_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, nts0_id); strcat(stack, " "); strcat(stack, ts0_id);
    strcat(stack, " "); strcat(stack, nts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, nts1_id);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, ts1_id);
    strcat(stack, " "); strcat(stack, nts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, nts2_id);
    strcpy(stack, ts2_id); strcat(stack, " "); strcat(stack, nts0_id);
    strcat(stack, " "); strcat(stack, ts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, ts4_id);
    free(stack);
  }
  else if ((choice == 12) || (choice == 13)) {
    nts0_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "R"), "nonterminal_symbol");
    nts1_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "K"), "nonterminal_symbol");
    nts2_id = 
      get_id(contextfree_grammar_add_nonterminal_symbol
	     (contextfree_grammar, "I"), "nonterminal_symbol");
    contextfree_grammar_set_start_symbol(contextfree_grammar, nts0_id);
    ts0_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "+"), "terminal_symbol");
    ts1_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "*"), "terminal_symbol");
    ts2_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "("), "terminal_symbol");
    ts3_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, ")"), "terminal_symbol");
    ts4_id = 
    get_id(contextfree_grammar_add_terminal_symbol
	   (contextfree_grammar, "a"), "terminal_symbol");
    stack = calloc(100, 1);
    strcpy(stack, nts0_id); strcat(stack, " "); strcat(stack, ts0_id);
    strcat(stack, " "); strcat(stack, nts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts0_id, nts1_id);
    strcpy(stack, nts1_id); strcat(stack, " "); strcat(stack, nts2_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts1_id, nts2_id);
    strcpy(stack, nts2_id); strcat(stack, " "); strcat(stack, ts1_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    strcpy(stack, ts2_id); strcat(stack, " "); strcat(stack, nts0_id);
    strcat(stack, " "); strcat(stack, ts3_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, stack);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, ts4_id);
    contextfree_grammar_add_rule(contextfree_grammar, nts2_id, "");
    free(stack);  
  }
  else if (choice == 14) {
    q0_id = get_id(stack_automata_add_state(stack_automata, "q0"), "state");
    q1_id = get_id(stack_automata_add_state(stack_automata, "q1"), "state");
    stack_automata_set_start_state(stack_automata, q0_id);
    s0_id = 
    get_id
    (stack_automata_add_entry_symbol(stack_automata, "0"), "entry_symbol");
    s1_id = 
    get_id
    (stack_automata_add_entry_symbol(stack_automata, "1"), "entry_symbol");
    ss0_id = 
    get_id
    (stack_automata_add_stack_symbol(stack_automata, "Z0"), "stack_symbol");
    ss1_id = 
    get_id
    (stack_automata_add_stack_symbol(stack_automata, "X"), "stack_symbol");
    stack_automata_set_start_stack_symbol(stack_automata, ss0_id);
    stack = calloc(100, 1);
    strcpy(stack, ss0_id); strcat(stack, " "); strcat(stack, ss1_id);
    stack_automata_add_transition
    (stack_automata, q0_id, s0_id, q0_id, ss0_id, stack);
    strcpy(stack, ss1_id); strcat(stack, " "); strcat(stack, ss1_id);
    stack_automata_add_transition
    (stack_automata, q0_id, s0_id, q0_id, ss1_id, stack);
    stack_automata_add_transition
    (stack_automata, q0_id, s1_id, q1_id, ss1_id, "");
    stack_automata_add_transition
    (stack_automata, q1_id, s1_id, q1_id, ss1_id, "");
    stack_automata_add_transition
    (stack_automata, q1_id, "0", q1_id, ss1_id, "");
    stack_automata_add_transition
    (stack_automata, q1_id, "0", q1_id, ss0_id, "");
    free(stack);
  }
  if ((choice > 0) && (choice < 4)) {
    vypis_finite(finite_automata);
    veta_finite(finite_automata);
    printf("Nedosazitelne stavy: %s\n",
	   finite_automata_get_inaccessible_states(finite_automata));
    printf ("Deterministic ... ");
    if (finite_automata_is_deterministic(finite_automata)) {
      printf ("yes -> equivalent reduced finite automata:\n"); 
      finite_automata_to_reduced(finite_automata);
    }
    else {
      printf ("no -> equivalent deterministic finite automata:\n");
      finite_automata_to_deterministic(finite_automata);
      /*
      vypis_finite(finite_automata);
      finite_automata_undo(finite_automata);
      vypis_finite(finite_automata);
      finite_automata_redo(finite_automata);
      */
    }
    vypis_finite(finite_automata);
    veta_finite(finite_automata);
    free(finite_automata);
  }
  else if (choice == 4) {
    vypis_stack(stack_automata);
    veta_stack(stack_automata);
    printf ("Deterministic ... ");
    if (stack_automata_is_deterministic(stack_automata)) printf ("yes\n");
    else printf("no\n");
    printf ("Top stack working ... ");
    if (stack_automata_is_top_stack_working(stack_automata)) printf ("yes\n"); 
    else printf("no\n");
    printf ("Empty stack checking ... ");
    if (stack_automata_is_empty_stack_checking(stack_automata)) {
      printf ("yes -> equivalent end states checking stack automata:\n"); 
      stack_automata_to_end_states_checking(stack_automata);
    }
    else {
      printf ("no -> equivalent empty stack checking stack automata:\n");
      stack_automata_to_empty_stack_checking(stack_automata);
    }
    vypis_stack(stack_automata);
    veta_stack(stack_automata);
    free(stack_automata);
  }
  else if (choice == 5) {
    vypis_regular(regular_grammar);
    printf ("Classification ... %s\n", 
	    regular_grammar_get_classification(regular_grammar));
    printf("Equivalent finite automata:\n");
    finite_automata = regular_grammar_to_finite_automata(regular_grammar);
    vypis_finite(finite_automata);
    printf("Back to regular grammar:\n");
    regular_grammar = finite_automata_to_regular_grammar(finite_automata);
    vypis_regular(regular_grammar);
    free(finite_automata);
    free(regular_grammar);
  }
  else if ((choice >= 6) && (choice <= 13)) {
    vypis_contextfree(contextfree_grammar);
    printf ("Classification ... %s\n", 
	    contextfree_grammar_get_classification(contextfree_grammar));
  }
  if ((choice == 6) || (choice == 7)) {
    printf("Redundant symbols: %s\n",
	   contextfree_grammar_get_redundant_symbols(contextfree_grammar));
    printf("Inaccessible symbols: %s\n",
	   contextfree_grammar_get_inaccessible_symbols(contextfree_grammar));
    printf("Equivalent grammar without redundant symbols:\n");
    contextfree_grammar_remove_redundant_symbols(contextfree_grammar);
    vypis_contextfree(contextfree_grammar);
    free(contextfree_grammar);
  }
  else if (choice == 8) {
    printf("e-rules: %s\n",
	   contextfree_grammar_get_e_rules(contextfree_grammar));
    printf ("e-rules-free ... ");
    if (contextfree_grammar_is_e_rules_free(contextfree_grammar))
      printf ("yes\n"); else printf("no\n");
    printf("Equivalent grammar without e-rules:\n");
    contextfree_grammar_remove_e_rules(contextfree_grammar);
    vypis_contextfree(contextfree_grammar);
    printf("simple rules: %s\n",
	   contextfree_grammar_get_simple_rules(contextfree_grammar));
    printf("Equivalent grammar without simple rules:\n");
    contextfree_grammar_remove_simple_rules(contextfree_grammar);
    vypis_contextfree(contextfree_grammar);
    printf ("Self ... ");
    if (contextfree_grammar_is_self(contextfree_grammar))
      printf ("yes\n"); 
    else {
      printf("no -> removing redundant symbols -> Self ... ");
      contextfree_grammar_remove_redundant_symbols(contextfree_grammar);
      if (contextfree_grammar_is_self(contextfree_grammar))
	printf ("yes\n"); else printf("no\n");
    }
    free(contextfree_grammar);
  }
  else if (choice == 9) {
    printf("Recursive symbols: %s\n",
	   contextfree_grammar_get_recursive_symbols(contextfree_grammar));
    printf("Equivalent grammar without left recursion:\n");
    contextfree_grammar_eliminate_left_recursion(contextfree_grammar);
    vypis_contextfree(contextfree_grammar);
    free(contextfree_grammar);
  }
  else if (choice == 10) {
    printf ("in Chomsky normal form ... ");
    if (contextfree_grammar_is_in_chomsky_normal_form(contextfree_grammar))
      printf ("yes\n");
    else {
      printf ("no -> equivalent grammar in Chomsky normal form:\n");
      contextfree_grammar_to_self(contextfree_grammar);
      contextfree_grammar_to_chomsky_normal_form(contextfree_grammar);
      vypis_contextfree(contextfree_grammar);
    }
    free(contextfree_grammar);
  }
  else if (choice == 11) {
    printf ("in Greibach normal form ... ");
    if (contextfree_grammar_is_in_greibach_normal_form(contextfree_grammar))
      printf ("yes\n");
    else {
      printf ("no -> equivalent grammar in Greibach normal form:\n");
      contextfree_grammar_to_self(contextfree_grammar);
      contextfree_grammar_eliminate_left_recursion(contextfree_grammar);
      contextfree_grammar_to_greibach_normal_form(contextfree_grammar);
      vypis_contextfree(contextfree_grammar);
    }
    free(contextfree_grammar);
  }
  else if (choice == 12) {
    printf("Equivalent stack automata (top-down):\n");
    stack_automata =
      contextfree_grammar_to_stack_automata(contextfree_grammar, 1);
    vypis_stack(stack_automata);
    free(stack_automata);
    free(contextfree_grammar);
  }
  else if (choice == 13) {
    printf("Equivalent stack automata (bottom-up):\n");
    stack_automata =
      contextfree_grammar_to_stack_automata(contextfree_grammar, 0);
    vypis_stack(stack_automata);
    free(stack_automata);
    free(contextfree_grammar);
  }
  else if (choice == 14) {
    vypis_stack(stack_automata);
    printf("Equivalent contextfree grammar:\n");
    contextfree_grammar =
      stack_automata_to_contextfree_grammar(stack_automata);
    vypis_contextfree(contextfree_grammar);
    free(contextfree_grammar);
    free(stack_automata);
  }
  /* free(.._id); */
}

int
main_client(int argc, char *argv[])
{
  formal_set_error(E_SERVER_NOERR);
  if (formal_get_error() != E_SERVER_NOERR) { 
    printf("\n"); printf("PACKAGE-client: initialization error\n");
    return 1;
  }
  demo();
  return 0;
}
