/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include "../server-error.h"
#include "gui.h"
#include "error.h"

void invalid_input_message(void)
{
    GtkWidget *dialog;

    dialog = gnome_message_box_new(_("Invalid input! Try again."),
				    GNOME_MESSAGE_BOX_ERROR,
				    GNOME_STOCK_BUTTON_OK,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gnome_dialog_run(GNOME_DIALOG(dialog));
} /* invalid_input_message */

gboolean check_error(int serror)
{
    GtkWidget *dialog;
    gchar *message;

    switch (serror) {
    case E_SERVER_NOERR:
	return FALSE;
    case E_SERVER_A_STATE_MAX:
	message = g_strdup(_("Maximal states count reached."));
	break;
    case E_SERVER_A_STATE_BAD:
	message = g_strdup(_("Unknown state."));
	break;
    case E_SERVER_A_ENTRY_SYMBOL_MAX:
	message = g_strdup(_("Maximal entry symbols count reached."));
	break;
    case E_SERVER_A_ENTRY_SYMBOL_BAD:
	message = g_strdup(_("Unknown entry symbol."));
	break;
    case E_SERVER_A_TRANSITION_BAD:
	message = g_strdup(_("Unknown transition."));
	break;
    case E_SERVER_AF_SENTENCE_EMPTY:
	message = g_strdup(_("Sentence is empty!"));
	break;
    case E_SERVER_AF_STATE_CANT_REACH:
	message = g_strdup(_("State is unreachable"));
	break;
    case E_SERVER_AF_NOT_DETERMINISTIC:
	message = g_strdup(_("Finite automata is not deterministic"));
	break;
    case E_SERVER_AS_STACK_SYMBOL_MAX:
	message = g_strdup(_("Maximal stack symbols count reached."));
	break;
    case E_SERVER_AS_STACK_SYMBOL_BAD:
	message = g_strdup(_("Unknown stack symbol."));
	break;
    case E_SERVER_AS_STATE_STACK_CANT_REACH:
	message = g_strdup(_("This pair of state & stack can't be reached."));
	break;
    case E_SERVER_AS_STACK_CANT_POP:
	message = g_strdup(_("Stack can't be popped."));
	break;
    case E_SERVER_A_STATE_NAME_EXISTS:
	message = g_strdup(_("State with this name already exists."));
	break;
    case E_SERVER_A_ENTRY_SYMBOL_NAME_EXISTS:
	message = g_strdup(_("Entry symbol with this name already exists."));
	break;
    case E_SERVER_AS_STACK_SYMBOL_NAME_EXISTS:
	message = g_strdup(_("Stack symbol with this name already exists."));
	break;
    case E_SERVER_G_NONTERMINAL_SYMBOL_MAX:
	message = g_strdup(_("Maximal nonterminal symbols count reached."));
	break;
    case E_SERVER_G_NONTERMINAL_SYMBOL_BAD:
	message = g_strdup(_("Unknown nonterminal symbol"));
	break;
    case E_SERVER_G_TERMINAL_SYMBOL_MAX:
	message = g_strdup(_("Maximal terminal symbols count reached."));
	break;
    case E_SERVER_G_TERMINAL_SYMBOL_BAD:
	message = g_strdup(_("Unknown terminal symbol"));
	break;
    case E_SERVER_G_RULE_BAD:
	message = g_strdup(_("Unknown rule"));
	break;
    case E_SERVER_GR_RULE_BAD:
	message = g_strdup(_("Rule is not suitable for regular grammar"));
	break;
    case E_SERVER_GC_RULE_BAD:
	message = g_strdup(_("Rule is not suitable for contextfree grammar"));
	break;
    case E_SERVER_GC_NOT_ERULES_FREE:
	message = g_strdup(_("Grammar is not empty rules free"));
	break;
    case E_SERVER_GC_NOT_SELF:
	message = g_strdup(_("Grammar is not self"));
	break;
    case E_SERVER_GC_HAS_SIMPLE_RULES:
	message = g_strdup(_("Grammar contains simple rules"));
	break;
    case E_SERVER_GC_IS_LEFT_RECURSIVE:
	message = g_strdup(_("Grammar is left recursive"));
	break;
    case E_SERVER_G_NONTERMINAL_SYMBOL_NAME_EXISTS:
	message = g_strdup(_("Nonterminal symbol with this name already exists."));
	break;
    case E_SERVER_G_TERMINAL_SYMBOL_NAME_EXISTS:
	message = g_strdup(_("Terminal symbol with this name already exists."));
	break;
    default:
	return FALSE;
    }

    dialog = gnome_message_box_new(message,
				   GNOME_MESSAGE_BOX_ERROR,
				   GNOME_STOCK_BUTTON_OK,
				   NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gnome_dialog_run(GNOME_DIALOG(dialog));

    g_free(message);
    return TRUE;
} /* check_error */
