/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include "../client-interface.h"
#include "data.h"
#include "functions.h"
#include "parse.h"

#define XSTR (unsigned char *)
#define STR (char *)

char *filename = NULL;

/*
 *   Loading from xml file
 */

static void add_xml_state_to_data(xmlNodePtr xmlnode, gchar *type)
{
    gchar *name, *atribute, *tmp, *x, *y;
    gchar *id = NULL;

    if (!xmlnode->name || g_strcasecmp(STR xmlnode->name, "State") != 0)
	return;
    
    tmp = STR xmlGetProp(xmlnode, XSTR "name");
    g_return_if_fail(tmp != NULL);
    name = deparen(tmp);
    g_free(tmp);
    atribute = STR xmlGetProp(xmlnode, XSTR "atribute");
    
    x = STR xmlGetProp(xmlnode, XSTR "x");
    if (!x)
	x = "0.0";
    
    y = STR xmlGetProp(xmlnode, XSTR "y");
    if (!y)
	y = "0.0";

    if (g_strcasecmp(type, "finite") == 0) {
	id = get_id(finite_automata_add_state(fa, name));
	add_coordinate(atoi(id), atof(x), atof(y));
	if (atribute != NULL) {
	    int i;
	    for (i = 0; i < strlen(atribute); i++) {
		if (atribute[i] == 's')
		    finite_automata_set_start_state(fa, id);
		else if (atribute[i] == 'e')
		    finite_automata_set_end_state(fa, id);
	    }
	}
    } else if (g_strcasecmp(type, "stack") == 0) {
	id = get_id(stack_automata_add_state(sa, name));
	if (atribute != NULL) {
	    int i;
	    for (i = 0; i < strlen(atribute); i++) {
		if (atribute[i] == 's')
		    stack_automata_set_start_state(sa, id);
		else if (atribute[i] == 'e')
		    stack_automata_set_end_state(sa, id);
	    }
	}
    }
    g_free(id);
} /* add_xml_state_to_data */

static void add_xml_symbol_to_data(xmlNodePtr xmlnode, gchar *type)
{
    gchar *name, *atribute, *tmp;
    gchar *id = NULL;

    if (!xmlnode->name)
	return;
    
    tmp = STR xmlGetProp(xmlnode, XSTR "name");
    g_return_if_fail(tmp != NULL);
    name = deparen(tmp);
    g_free(tmp);
    atribute = STR xmlGetProp(xmlnode, XSTR "atribute");

    if (g_strcasecmp(STR xmlnode->name, "Entry_Symbol") == 0) {
	if (g_strcasecmp(type, "finite") == 0)
	    finite_automata_add_entry_symbol(fa, name);
	else if (g_strcasecmp(type, "stack") == 0)
	    stack_automata_add_entry_symbol(sa, name);

    } else if (g_strcasecmp(STR xmlnode->name, "Stack_Symbol") == 0) {
	if (g_strcasecmp(type, "stack") == 0) {
	    id = get_id(stack_automata_add_stack_symbol(sa, name));
	    if (atribute != NULL)
		stack_automata_set_start_stack_symbol(sa, id);
	}

    } else if (g_strcasecmp(STR xmlnode->name, "Terminal_Symbol") == 0) {
	if (g_strcasecmp(type, "regular") == 0)
	    regular_grammar_add_terminal_symbol(rg, name);
	else if (g_strcasecmp(type, "contextfree") == 0)
	    contextfree_grammar_add_terminal_symbol(cg, name);
	else
	    grammar_add_terminal_symbol(g, name);

    } else if (g_strcasecmp(STR xmlnode->name, "Nonterminal_Symbol") == 0) {
	if (g_strcasecmp(type, "regular") == 0) {
	    id = get_id(regular_grammar_add_nonterminal_symbol(rg, name));
	    if (atribute != NULL)
		regular_grammar_set_start_symbol(rg, id);
	} else if (g_strcasecmp(type, "contextfree") == 0) {
	    id = get_id(contextfree_grammar_add_nonterminal_symbol(cg, name));
	    if (atribute != NULL)
		contextfree_grammar_set_start_symbol(cg, id);
	} else {
	    id = get_id(grammar_add_nonterminal_symbol(g, name));
	    if (atribute != NULL)
		grammar_set_start_symbol(g, id);
	}
    }

    if (id) g_free(id);
} /* add_xml_symbol_to_data */

static void add_xml_transition_to_data(xmlNodePtr xmlnode, gchar *type)
{
    xmlNodePtr parent;
    xmlNodePtr node;
    GString *from_stack = g_string_sized_new(10);
    GString *to_stack = g_string_sized_new(10);
    gchar *from, *entry_symbol, *to;
    gchar *from_id, *symbol_id, *to_id;
    Symbol *symbol;
    State *state;

    if (!xmlnode->name)
	return;
    
    from = STR xmlGetProp(xmlnode, XSTR "from");
    g_return_if_fail(from != NULL);
    state = find_state_by_name(from);
    g_return_if_fail(state != NULL);
    from_id = g_strdup_printf("%i", state->id);

    entry_symbol = STR xmlGetProp(xmlnode, XSTR "entry_symbol");
    if (entry_symbol == NULL)
	symbol_id = g_strdup_printf("%i", 0);
    else {
	symbol = find_symbol_by_name(entry_symbol, aut.entry_symbols);
	g_return_if_fail(symbol != NULL);
	symbol_id = g_strdup_printf("%i", symbol->id);	
    }

    to = STR xmlGetProp(xmlnode, XSTR "to");
    g_return_if_fail(to != NULL);
    state = find_state_by_name(to);
    g_return_if_fail(state != NULL);
    to_id = g_strdup_printf("%i", state->id);

    if (g_strcasecmp(type, "finite") == 0)
	finite_automata_add_transition(fa, from_id, symbol_id, to_id);
    else if (g_strcasecmp(type, "stack") == 0) {
	for (parent = xmlnode->childs; parent != NULL; parent = parent->next) {
	    for (node = parent->childs; node != NULL; node = node->next) {
		gchar *id;
		gchar *name = STR xmlGetProp(node, XSTR "name");

		symbol = find_symbol_by_name(name, aut.stack_symbols);
		g_return_if_fail(symbol != NULL);
		id = g_strdup_printf("%i", symbol->id);
		
		if (g_strcasecmp(STR parent->name, "From_Stack") == 0) {
		    from_stack = g_string_append(from_stack, id);
		    from_stack = g_string_append(from_stack, " ");
		}
		else if (g_strcasecmp(STR parent->name, "To_Stack") == 0) {
		    to_stack = g_string_append(to_stack, id);
		    to_stack = g_string_append(to_stack, " ");
		}
		g_free(id);
	    }
	}	
	stack_automata_add_transition(sa, from_id, symbol_id, to_id,
				      from_stack->str, to_stack->str);
    }
    g_string_free(from_stack, TRUE);
    g_string_free(to_stack, TRUE);
    g_free(from_id);
    g_free(symbol_id);
    g_free(to_id);
} /* add_xml_transition_to_data */

static void add_xml_rule_to_data(xmlNodePtr xmlnode, gchar *type)
{
    xmlNodePtr parent;
    xmlNodePtr node;
    gchar *name;
    GString *left = g_string_sized_new(5);
    GString *right = g_string_sized_new(10);

    if (!xmlnode->name)
	return;

    for (parent = xmlnode->childs; parent != NULL; parent = parent->next) {
	for (node = parent->childs; node != NULL; node = node->next) {
	    Symbol *symbol;
	    gchar *id;
	    name = STR xmlGetProp(node, XSTR "name");
	    symbol = find_symbol_by_name(name, gram.nonterminal_symbols);
	    if (symbol == NULL)
		symbol = find_symbol_by_name(name, gram.terminal_symbols);
	    g_return_if_fail(symbol != NULL);
	    id = g_strdup_printf("%i", symbol->id);

	    if (g_strcasecmp(STR parent->name, "Left") == 0) {
		left = g_string_append(left, id);
		left = g_string_append(left, " ");
	    }
	    else if (g_strcasecmp(STR parent->name, "Right") == 0) {
		right = g_string_append(right, id);
		right = g_string_append(right, " ");
	    }
	    g_free(id);
	}
    }

    if (g_strcasecmp(type, "regular") == 0)
	regular_grammar_add_rule(rg, left->str, right->str);
    else if (g_strcasecmp(type, "contextfree") == 0)
	contextfree_grammar_add_rule(cg, left->str, right->str);
    else
	grammar_add_rule(g, left->str, right->str);

    g_string_free(left, TRUE);
    g_string_free(right, TRUE);
} /* add_xml_rule_to_data */


static void parse_doc(xmlDocPtr doc)
{
    xmlNodePtr parent;
    xmlNodePtr node;
    gchar *type;
  
    type = STR xmlGetProp(doc->root, XSTR "type");
    if (g_strcasecmp(type, "finite") == 0) {
	fa = formal_new_finite_automata();
	set_current(FINITE);
    } else if (g_strcasecmp(type, "stack") == 0) {
	sa = formal_new_stack_automata();
	set_current(STACK);
    } else if (g_strcasecmp(type, "regular") == 0) {
	rg = formal_new_regular_grammar();
	set_current(REGULAR);
    } else if (g_strcasecmp(type, "contextfree") == 0) {
	cg = formal_new_contextfree_grammar();
	set_current(CONTEXTFREE);
    } else if (g_strcasecmp(type, "") == 0) {
	g = formal_new_grammar();
	set_current(GRAMMAR);
    }

    clear_all_data(); /* WARN: where to call this? */

    for (parent = doc->root->childs; parent != NULL; parent = parent->next) {
	if (g_strcasecmp(STR parent->name, "States") == 0) {
	    for (node = parent->childs; node != NULL; node = node->next)
		add_xml_state_to_data(node, type);

	} else if ((g_strcasecmp(STR parent->name, "Entry_Symbols") == 0)
		   || (g_strcasecmp(STR parent->name, "Stack_Symbols") == 0)
		   || (g_strcasecmp(STR parent->name, "Terminal_Symbols") == 0)
		   || (g_strcasecmp(STR parent->name, "Nonterminal_Symbols")
		       == 0)) {
	    for (node = parent->childs; node != NULL; node = node->next)
		add_xml_symbol_to_data(node, type);
	    
	} else if (g_strcasecmp(STR parent->name, "Transitions") == 0) {
	    clear_all_data();
	    if (get_current() == FINITE)
		parse(finite_automata_get_look(fa), NULL, NULL);
	    else if (get_current() == STACK)
		parse(stack_automata_get_look(sa), NULL, NULL);

	    for (node = parent->childs; node != NULL; node = node->next)
		add_xml_transition_to_data(node, type);

	} else if (g_strcasecmp(STR parent->name, "Rules") == 0) {
	    clear_all_data();

	    if (get_current() == REGULAR)
		parse(regular_grammar_get_look(rg), NULL, NULL);
	    else if (get_current() == CONTEXTFREE)
		parse(contextfree_grammar_get_look(cg), NULL, NULL);
	    else if (get_current() == GRAMMAR)
		parse(grammar_get_look(g), NULL, NULL);

	    for (node = parent->childs; node != NULL; node = node->next)
		add_xml_rule_to_data(node, type);
	} else if (g_strcasecmp(STR parent->name, "Comment") == 0) {
	    set_comment(STR xmlNodeGetContent(parent));
	}
    }
} /* parse_doc */

gboolean read_xml_file(char *fname)
{
    xmlDocPtr doc;

    g_return_val_if_fail(fname != NULL, FALSE);

    if (!g_file_exists(fname))
	return FALSE;

    doc = xmlParseFile(fname);

    if (!doc || !doc->root || !doc->root->name) {
	xmlFreeDoc(doc);
	return FALSE;
    }

    if (!((g_strcasecmp(STR doc->root->name, "Automata") == 0)
	  || (g_strcasecmp(STR doc->root->name, "Grammar") == 0))) {
	xmlFreeDoc(doc);
	return FALSE;
    }

    parse_doc(doc);
    xmlFreeDoc(doc);

    g_free(filename);
    filename = g_strdup(fname);

    return TRUE;
} /* read_xml_file */


/*
 *   Saving to xml file
 */

static
void write_symbol_to_xml(xmlNodePtr xmlnode, Symbol *symbol, gchar *desc)
{
    xmlNodePtr newxml;

    g_return_if_fail(xmlnode != NULL);
    g_return_if_fail(symbol != NULL);

    newxml = xmlNewChild(xmlnode, NULL, XSTR desc, NULL);
    xmlSetProp(newxml, XSTR "name", XSTR symbol->name);
    if (symbol->atribute != NULL)
	xmlSetProp(newxml, XSTR "atribute", XSTR symbol->atribute);
} /* write_symbol_to_xml */

static void write_state_to_xml(xmlNodePtr xmlnode, State *state)
{
    xmlNodePtr newxml;

    g_return_if_fail(xmlnode != NULL);
    g_return_if_fail(state != NULL);

    newxml = xmlNewChild(xmlnode, NULL, XSTR "State", NULL);

    xmlSetProp(newxml, XSTR "name", XSTR state->name);
    if (state->atribute != NULL)
	xmlSetProp(newxml, XSTR "atribute", XSTR state->atribute);

    if (state->group != NULL) {
	double x, y;
	gchar *value;

	gtk_object_get(GTK_OBJECT(state->group), "x", &x, NULL);
	gtk_object_get(GTK_OBJECT(state->group), "y", &y, NULL);

	value = g_strdup_printf("%f", x);
	xmlSetProp(newxml, XSTR "x", XSTR value);
	g_free(value);
	value = g_strdup_printf("%f", y);
	xmlSetProp(newxml, XSTR "y", XSTR value);
	g_free(value);
    }
} /* write_state_to_xml */

static void write_transition_to_xml(xmlNodePtr xmlnode, Transition *trans)
{
    xmlNodePtr newxml;
    xmlNodePtr newauxxml;
    GSList *l;
    GSList *k;

    g_return_if_fail(xmlnode != NULL);
    g_return_if_fail(trans != NULL);

    if (get_current() == FINITE)
	for (l = trans->entry_symbols; l != NULL; l = l->next) {
	    Symbol *symbol = l->data;

	    newxml = xmlNewChild(xmlnode, NULL, XSTR "Transition", NULL);
	    xmlSetProp(newxml, XSTR "from", XSTR trans->from->name);
	    xmlSetProp(newxml, XSTR "entry_symbol", XSTR symbol->name);
	    xmlSetProp(newxml, XSTR "to", XSTR trans->to->name);
	}
    else if (get_current() == STACK) {
	newxml = xmlNewChild(xmlnode, NULL, XSTR "Transition", NULL);

	xmlSetProp(newxml, XSTR "from", XSTR trans->from->name);
	if (trans->entry_symbols != NULL) {
	    Symbol *symbol = trans->entry_symbols->data;
	    xmlSetProp(newxml, XSTR "entry_symbol", XSTR symbol->name);
	}
	xmlSetProp(newxml, XSTR "to", XSTR trans->to->name);
	
	newauxxml = xmlNewChild(newxml, NULL, XSTR "From_Stack", NULL);
	for (k = trans->from_stack; k != NULL; k = k->next) {
	    Symbol *symbol = k->data;
	    write_symbol_to_xml(newauxxml, symbol, "Symbol");
	}
	newauxxml = xmlNewChild(newxml, NULL, XSTR "To_Stack", NULL);
	for (k = trans->to_stack; k != NULL; k = k->next) {
	    Symbol *symbol = k->data;
	    write_symbol_to_xml(newauxxml, symbol, "Symbol");
	}
    }
} /* write_transition_to_xml */

static void write_rule_to_xml(xmlNodePtr xmlnode, Rule *rule)
{
    xmlNodePtr newxml;
    xmlNodePtr newauxxml;
    GSList *l;

    g_return_if_fail(xmlnode != NULL);
    g_return_if_fail(rule != NULL);

    newauxxml = xmlNewChild(xmlnode, NULL, XSTR "Rule", NULL);

    newxml = xmlNewChild(newauxxml, NULL, XSTR "Left", NULL);
    for (l = rule->left; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	write_symbol_to_xml(newxml, symbol, "Symbol");
    }
    newxml = xmlNewChild(newauxxml, NULL, XSTR "Right", NULL);
    for (l = rule->right; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	write_symbol_to_xml(newxml, symbol, "Symbol");
    }
} /* write_rule_to_xml */

static void write_automata_to_xml(xmlNodePtr xmlnode)
{
    xmlNodePtr newxml;
    GSList *l;

    g_return_if_fail(xmlnode != NULL);

    newxml = xmlNewChild(xmlnode, NULL, XSTR "Entry_Symbols", NULL);
    for (l = aut.entry_symbols; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	write_symbol_to_xml(newxml, symbol, "Entry_Symbol");
    }
    if (get_current() == STACK) {
	newxml = xmlNewChild(xmlnode, NULL, XSTR "Stack_Symbols", NULL);
	for (l = aut.stack_symbols; l != NULL; l = l->next) {
	    Symbol *symbol = l->data;
	    write_symbol_to_xml(newxml, symbol, "Stack_Symbol");
	}
    }
    newxml = xmlNewChild(xmlnode, NULL, XSTR "States", NULL);
    for (l = aut.states; l != NULL; l = l->next) {
	State *state = l->data;
	write_state_to_xml(newxml, state);
    }
    newxml = xmlNewChild(xmlnode, NULL, XSTR "Transitions", NULL);
    for (l = aut.transitions; l != NULL; l = l->next) {
	Transition *trans = l->data;
	write_transition_to_xml(newxml, trans);
    }
} /* write_automata_to_xml */

static void write_grammar_to_xml(xmlNodePtr xmlnode)
{
    xmlNodePtr newxml;
    GSList *l;

    g_return_if_fail(xmlnode != NULL);

    newxml = xmlNewChild(xmlnode, NULL, XSTR "Nonterminal_Symbols", NULL);
    for (l = gram.nonterminal_symbols; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	write_symbol_to_xml(newxml, symbol, "Nonterminal_Symbol");
    }
    newxml = xmlNewChild(xmlnode, NULL, XSTR "Terminal_Symbols", NULL);
    for (l = gram.terminal_symbols; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	write_symbol_to_xml(newxml, symbol, "Terminal_Symbol");
    }
    newxml = xmlNewChild(xmlnode, NULL, XSTR "Rules", NULL);
    for (l = gram.rules; l != NULL; l = l->next) {
	Rule *rule = l->data;
	write_rule_to_xml(newxml, rule);
    }
} /* write_grammar_to_xml */

gboolean write_xml_file(char *fname)
{
    xmlDocPtr doc;
    xmlNodePtr newxml;

    g_return_val_if_fail(fname != NULL, FALSE);

    doc = xmlNewDoc(XSTR "1.0");
    switch (get_current()) {
    case FINITE:
	doc->root = xmlNewDocNode(doc, NULL, XSTR "Automata", NULL);
	xmlSetProp(doc->root, XSTR "type", XSTR "finite");
	write_automata_to_xml(doc->root);
	break;
    case STACK:
	doc->root = xmlNewDocNode(doc, NULL, XSTR "Automata", NULL);
	xmlSetProp(doc->root, XSTR "type", XSTR "stack");
	write_automata_to_xml(doc->root);
	break;
    case REGULAR:
	doc->root = xmlNewDocNode(doc, NULL, XSTR "Grammar", NULL);
	xmlSetProp(doc->root, XSTR "type", XSTR "regular");
	write_grammar_to_xml(doc->root);
	break;
    case CONTEXTFREE:
	doc->root = xmlNewDocNode(doc, NULL, XSTR "Grammar", NULL);
	xmlSetProp(doc->root, XSTR "type", XSTR "contextfree");
	write_grammar_to_xml(doc->root);
	break;
    case GRAMMAR:
	doc->root = xmlNewDocNode(doc, NULL, XSTR "Grammar", NULL);
	xmlSetProp(doc->root, XSTR "type", XSTR "");
	write_grammar_to_xml(doc->root);
	break;
    default:
      	xmlFreeDoc(doc);
	return FALSE;
    }

    newxml = xmlNewChild(doc->root, NULL, XSTR "Comment", XSTR get_comment());

    if (xmlSaveFile(fname, doc) == -1) {
      	xmlFreeDoc(doc);
	return FALSE;
    }

    g_free(filename);
    filename = g_strdup(fname);

    xmlFreeDoc(doc);

    return TRUE;
} /* write_xml_file */
