#ifndef FUNCTIONS_H
#define FUNCTIONS_H

/* data */
extern automata aut;
extern grammar gram;

extern GSList *states;
extern GSList *symbols;
extern GSList *rules;
extern GSList *stack;
extern GSList *pairs;

/* functions */
extern void setup_comment(void);
extern void set_comment(gchar *text);
extern gchar *get_comment(void);

extern void set_current(KIND kind);
extern KIND get_current(void);

extern State *add_state(int id, char *name, char *atribute);
extern Transition *add_transition(int from, int to);
extern gboolean add_symbol(int id, gchar *name, gchar *atribute, gchar *type);
extern void delete_state(State *state);
extern void delete_transition(Transition *trans);
extern void delete_symbol(Symbol *symbol);
extern void delete_rule(Rule *rule);
extern void add_state_to_list(int id);
extern void add_symbol_to_list(int id, gchar *type);
extern void add_to_stack(int id);
extern void add_coordinate(int id, double x, double y);
extern void setup_aux(void);
extern void add_aux_to_list(gchar *extra);
extern void add_state_to_transition(int id);
extern void add_entry_symbol_to_transition(int id);
extern void add_stack_symbol_to_transition(int id, gchar *direction);
extern void add_symbol_to_rule(int id, gchar *type, gchar *side);
extern State *find_state_by_group(GnomeCanvasGroup *group);
extern State *find_state_by_id(int id);
extern State *find_state_by_name(gchar *name);
extern Symbol *find_symbol_by_id(int id, GSList *list);
extern Symbol *find_symbol_by_name(gchar *name, GSList *list);
extern Transition *find_transition_by_group(GnomeCanvasGroup *group);
extern Coordinate *find_coordinate_by_id(int id);
extern void clear_other_data(void);
extern void clear_all_data(void);
extern gchar *get_fin_aut_description(gboolean printable);
extern gchar *get_stack_aut_description(gboolean printable);
extern gchar *get_string_of_states(void);
extern gchar *get_string_of_ending_states(void);
extern gchar *get_gram_description(gboolean printable);
extern gchar *get_string_of_symbols(GSList *list, gboolean set);
extern gchar *get_string_of_entry_symbols(Transition *trans);
extern gchar *get_id_string_of_symbols(gchar *sentence, GSList *list1, GSList *list2);
extern gchar *deparen(gchar *text);
extern gchar *debrack(gchar *text);
extern gchar *delatex(gchar *text);
extern gboolean bidirectional(Transition *trans);
extern void draw_all_states(void);
extern void draw_all_transitions(void);

#endif
