/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <libgnomeui/gnome-canvas.h>
#include <stdlib.h>
#include "../client-interface.h"
#include "data.h"
#include "functions.h"
#include "files.h"
#include "export.h"
#include "parse.h"
#include "table.h"
#include "graphics.h"
#include "gui.h"
#include "menu.h"
#include "error.h"

/*
 * GUI variables
 */

GtkWidget *app;

void nothing_cb(GtkWidget *widget, gpointer data)
{
} /* nothing_cb */

static GtkWidget *
make_hbox_with_label_and_entry(char *label, char *def, GtkWidget **entry)
{
    GtkWidget *hbox;
    hbox = gtk_hbox_new(GNOME_PAD,FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(label),
                       FALSE, FALSE, 0);
    
    *entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(*entry), def);
    gtk_entry_select_region(GTK_ENTRY(*entry), 0,
			    GTK_ENTRY(*entry)->text_length);
    gtk_box_pack_start(GTK_BOX(hbox), *entry,
                       FALSE, FALSE, 0);
    gtk_widget_show_all(hbox);
    return hbox;
} /* make_hbox_with_label_and_entry */

static GtkWidget *make_states_combo(GSList *states)
{
    GList *list = NULL;
    GtkWidget *combo = gtk_combo_new();
    GSList *l;

    for (l = states; l != NULL; l = l->next) {
	State *state = l->data;
	list = g_list_append(list, state->name);
    }

    if (list == NULL)
	list = g_list_append(list, " ");
    gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
    gtk_widget_show(combo);
    return combo;
} /* make_states_combo */

static GtkWidget *make_symbols_combo(GSList *symlist)
{
    GList *list = NULL;
    GtkWidget *combo = gtk_combo_new();
    GSList *l;

    for (l = symlist; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	list = g_list_append(list, symbol->name);
    }

    if (list == NULL)
	list = g_list_append(list, " ");
    gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
    gtk_widget_show(combo);
    return combo;
} /* make_symbols_combo */

static GtkWidget *make_states_text()
{
    GtkWidget *entry = gtk_text_new(NULL, NULL);
    GtkWidget *swindow = gtk_scrolled_window_new(NULL, NULL);
    GSList *l;

    for (l = states; l != NULL; l = l->next) {
	State *state = l->data;
	gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, state->name, -1);
	if (l->next != NULL)
	    gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, ", ", -1);
    }

    gtk_text_set_editable(GTK_TEXT(entry), FALSE);
    gtk_text_set_word_wrap(GTK_TEXT(entry), TRUE);
    gtk_widget_show(entry);

    gtk_widget_set_usize(GTK_WIDGET(swindow), 200, 100);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_AUTOMATIC);
    gtk_widget_show(swindow);
    gtk_container_add(GTK_CONTAINER(swindow), entry);
    return swindow;
} /* make_states_text */

static GtkWidget *make_symbols_text()
{
    GtkWidget *entry = gtk_text_new(NULL, NULL);
    GtkWidget *swindow = gtk_scrolled_window_new(NULL, NULL);
    GSList *l;

    for (l = symbols; l != NULL; l = l->next) {
	Symbol *symbol = l->data;
	gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, symbol->name, -1);
	if (l->next != NULL)
	    gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, ", ", -1);
    }

    gtk_text_set_editable(GTK_TEXT(entry), FALSE);
    gtk_text_set_word_wrap(GTK_TEXT(entry), TRUE);
    gtk_widget_show(entry);

    gtk_widget_set_usize(GTK_WIDGET(swindow), 200, 100);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_AUTOMATIC);
    gtk_widget_show(swindow);
    gtk_container_add(GTK_CONTAINER(swindow), entry);
    return swindow;
} /* make_symbols_text */

static GtkWidget *make_rules_text()
{
    GtkWidget *entry = gtk_text_new(NULL, NULL);
    GtkWidget *swindow = gtk_scrolled_window_new(NULL, NULL);
    GSList *l, *k;

    for (l = rules; l != NULL; l = l->next) {
	Rule *rule = l->data;
	for (k = rule->left; k != NULL; k = k->next) {
	    Symbol *symbol = k->data;
	    gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL,
			    symbol->name, -1);
	    if (k->next != NULL)
		gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, ", ", -1);
	}
	gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, "-> ", -1);

	for (k = rule->right; k != NULL; k = k->next) {
	    Symbol *symbol = k->data;
	    gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL,
			    symbol->name, -1);
	    if (k->next != NULL)
		gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, ", ", -1);
	}
	gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, "\n", -1);
    }

    gtk_text_set_editable(GTK_TEXT(entry), FALSE);
/*    gtk_text_set_word_wrap(GTK_TEXT(entry), TRUE);*/
    gtk_widget_show(entry);

/*    gtk_widget_set_usize(GTK_WIDGET(swindow), 200, 100);*/
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_AUTOMATIC);
    gtk_widget_show(swindow);
    gtk_container_add(GTK_CONTAINER(swindow), entry);
    return swindow;
} /* make_rules_text */

static void about_cb(GtkWidget *widget, gpointer data)
{
    static GtkWidget* dialog = NULL;

    const char *authors[] = {
	"Miroslav Kue <kurem@upcase.inf.upol.cz>",
	"Jan Outrata <outrataj@upcase.inf.upol.cz>",
	NULL
    };

    if (dialog) {
	gdk_window_show(dialog->window);
	gdk_window_raise(dialog->window);
	return;
    }

    dialog = gnome_about_new("GAAE", VERSION,
			     "(c) 2001 Miroslav Kue",
			     authors,
			     "GAAE is simple automata editor\n"
			     "written for us (by us).",
			     NULL);
    
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_signal_connect(GTK_OBJECT(dialog),
		       "destroy",
		       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
		       &dialog);

    gtk_widget_show(dialog);
} /* about_cb */

void add_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    gint result;
    char *name;

    dialog = gnome_dialog_new(_("Add state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      _("state"),
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));

    if (result == 0) {
	int id;
	name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));

	if (get_current() == FINITE) {
	    id = atoi(get_id(finite_automata_add_state(fa, name)));
	    if (!check_error(formal_get_error())) {
		if (data == NULL)
		    add_coordinate(id, 30.0, 5.0);
		else {
		    add_coordinate(id,
				   ((Coordinate *)data)->x,
				   ((Coordinate *)data)->y);
		    g_free(data);
		}
		show_all();
	    }
	}
	else if (get_current() == STACK) {
	    stack_automata_add_state(sa, name);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	g_free(name);
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* add_state_cb */

void add_entry_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    gint result;
    char *name;

    dialog = gnome_dialog_new(_("Add entry symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      _("entry_symbol"),
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));

	if (get_current() == FINITE)
	    finite_automata_add_entry_symbol(fa, name);
	else if (get_current() == STACK)
	    stack_automata_add_entry_symbol(sa, name);

	if (!check_error(formal_get_error()))
	    show_all();
	g_free(name);
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* add_entry_symbol_cb */

void add_stack_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    gint result;
    char *name;

    dialog = gnome_dialog_new(_("Add stack symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      _("stack_symbol"),
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));
	
	stack_automata_add_stack_symbol(sa, name);

	if (!check_error(formal_get_error()))
	    show_all();
	g_free(name);
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* add_stack_symbol_cb */

void add_nonterminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    gint result;
    char *name;

    dialog = gnome_dialog_new(_("Add nonterminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      _("nonterminal_symbol"),
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));
	if (get_current() == REGULAR)
	    regular_grammar_add_nonterminal_symbol(rg, name);
	else if (get_current() == CONTEXTFREE)
	    contextfree_grammar_add_nonterminal_symbol(cg, name);
	else if (get_current() == GRAMMAR)
	    grammar_add_nonterminal_symbol(g, name);

	if (!check_error(formal_get_error()))
	    show_all();
	g_free(name);
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* add_nonterminal_symbol_cb */

void add_terminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    gint result;
    char *name;

    dialog = gnome_dialog_new(_("Add terminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      _("terminal_symbol"),
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));
	if (get_current() == REGULAR)
	    regular_grammar_add_terminal_symbol(rg, name);
	else if (get_current() == CONTEXTFREE)
	    contextfree_grammar_add_terminal_symbol(cg, name);
	else if (get_current() == GRAMMAR)
	    grammar_add_terminal_symbol(g, name);

	if (!check_error(formal_get_error()))
	    show_all();
	g_free(name);
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* add_terminal_symbol_cb */

void add_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *entry, *from, *to, *from_stack, *to_stack;
    gint result;
    char *str_from, *str_to, *str_symbol;

    dialog = gnome_dialog_new(_("Add Transition"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("From: ")),
                       FALSE, FALSE, 5);
    from = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), from, TRUE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Entry Symbol: ")),
                       FALSE, FALSE, 5);
    entry = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("To: ")),
                       FALSE, FALSE, 5);
    to = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), to, TRUE, FALSE, 0);

    gtk_widget_show_all(hbox);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    if (get_current() == STACK) {
	hbox = gtk_hbox_new(FALSE, 5);

	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("From Stack: ")),
			   FALSE, FALSE, 5);
	from_stack = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), from_stack, TRUE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("To Stack: ")),
			   FALSE, FALSE, 5);
	to_stack = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), to_stack, TRUE, FALSE, 0);
	
	gtk_widget_show_all(hbox);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
			   FALSE, FALSE, 0);
    }

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
    	if (result == 0) {
	    State * state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(from)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_from = g_strdup_printf("%i", state->id);

	    str_symbol = get_id_string_of_symbols( /* potential abuse */
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(entry)->entry)),
		aut.entry_symbols, NULL);
	    if (str_symbol == NULL) {
		invalid_input_message();
		g_free(str_from); 
		continue;
	    }
	    
	    state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(to)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		g_free(str_from); g_free(str_symbol);
		continue;
	    }
	    str_to = g_strdup_printf("%i", state->id);
	    
	    if (get_current() == FINITE)
		finite_automata_add_transition(fa,
					       str_from, str_symbol, str_to);
	    else if (get_current() == STACK) {
		gchar *str_from_stack, *str_to_stack;

		str_from_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(from_stack)),
		    aut.stack_symbols, NULL);
		if (str_from_stack == NULL) {
		    invalid_input_message();
		    g_free(str_from); g_free(str_symbol); g_free(str_to);
		    continue;
		}

		str_to_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(to_stack)),
		    aut.stack_symbols, NULL);
		if (str_to_stack == NULL) {
		    invalid_input_message();
		    g_free(str_from); g_free(str_symbol);
		    g_free(str_to); g_free(str_from_stack);
		    continue;
		}

		stack_automata_add_transition(sa,
					      str_from, str_symbol, str_to,
					      str_from_stack, str_to_stack);
		g_free(str_from_stack); g_free(str_to_stack);
	    }
	    g_free(str_to);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result == 0 || result == 1) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0) {
	    break;
	}
    }
} /* add_transition_cb */

void add_rule_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wleft, *wright;
    gint result;

    dialog = gnome_dialog_new(_("Add Rule"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Left: ")),
                       FALSE, FALSE, 5);
    wleft = make_symbols_combo(gram.nonterminal_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wleft, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Right: ")),
                       FALSE, FALSE, 5);
    wright = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wright, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *left, *right;
	
	    left = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wleft)->entry)),
		gram.nonterminal_symbols, NULL);

	    if (left == NULL) {
		invalid_input_message();
		continue;
	    }

	    right = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(wright)),
		gram.terminal_symbols, gram.nonterminal_symbols);

	    if (right == NULL) {
		invalid_input_message();
		g_free(left);
		continue;
	    }

	    if (get_current() == REGULAR)
		regular_grammar_add_rule(rg, left, right);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_add_rule(cg, left, right);
	    else if (get_current() == GRAMMAR)
		grammar_add_rule(g, left, right);

	    g_free(left); g_free(right);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* add_rule_cb */

void remove_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wstate;
    gint result;
    char *str_state;

    dialog = gnome_dialog_new(_("Remove state state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wstate = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstate,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    State *state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }		
	    str_state = g_strdup_printf("%i", state->id);
	
	    if (get_current() == FINITE)
		finite_automata_remove_state(fa, str_state);
	    else if (get_current() == STACK)
		stack_automata_remove_state(sa, str_state);

	    g_free(str_state);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* remove_state_cb */

void remove_entry_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    gint result;
    char *str_sym;

    dialog = gnome_dialog_new(_("Remove entry symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		aut.entry_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_sym = g_strdup_printf("%i", symbol->id);
	
	    if (get_current() == FINITE)
		finite_automata_remove_entry_symbol(fa, str_sym);
	    else if (get_current() == STACK)
		stack_automata_remove_entry_symbol(sa, str_sym);

	    g_free(str_sym);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* remove_entry_symbol_cb */

void remove_stack_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    gint result;
    char *str_sym;

    dialog = gnome_dialog_new(_("Remove stack symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(aut.stack_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		aut.stack_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_sym = g_strdup_printf("%i", symbol->id);
	
	    stack_automata_remove_stack_symbol(sa, str_sym);

	    g_free(str_sym);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
}/* remove_stack_symbol_cb */

void remove_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *entry, *from, *to, *from_stack, *to_stack;
    gint result;
    char *str_from, *str_to, *str_symbol;

    dialog = gnome_dialog_new(_("Remove Transition"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("From: ")),
                       FALSE, FALSE, 5);
    from = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), from, TRUE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Entry Symbol: ")),
                       FALSE, FALSE, 5);
    entry = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("To: ")),
                       FALSE, FALSE, 5);
    to = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), to, TRUE, FALSE, 0);

    gtk_widget_show_all(hbox);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    if (get_current() == STACK) {
	hbox = gtk_hbox_new(FALSE, 5);

	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("From Stack: ")),
			   FALSE, FALSE, 5);
	from_stack = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), from_stack, TRUE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("To Stack: ")),
			   FALSE, FALSE, 5);
	to_stack = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), to_stack, TRUE, FALSE, 0);
	
	gtk_widget_show_all(hbox);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
			   FALSE, FALSE, 0);
    }

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
    	if (result == 0) {
	    State * state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(from)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_from = g_strdup_printf("%i", state->id);

	    str_symbol = get_id_string_of_symbols( /* potential abuse */
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(entry)->entry)),
		aut.entry_symbols, NULL);
	    if (str_symbol == NULL) {
		invalid_input_message();
		g_free(str_from); 
		continue;
	    }
	    
	    state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(to)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		g_free(str_from); g_free(str_symbol);
		continue;
	    }
	    str_to = g_strdup_printf("%i", state->id);
	    
	    if (get_current() == FINITE)
		finite_automata_remove_transition(fa,
					       str_from, str_symbol, str_to);
	    else if (get_current() == STACK) {
		gchar *str_from_stack, *str_to_stack;

		str_from_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(from_stack)),
		    aut.stack_symbols, NULL);
		if (str_from_stack == NULL) {
		    invalid_input_message();
		    g_free(str_from); g_free(str_symbol); g_free(str_to);
		    continue;
		}

		str_to_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(to_stack)),
		    aut.stack_symbols, NULL);
		if (str_to_stack == NULL) {
		    invalid_input_message();
		    g_free(str_from); g_free(str_symbol);
		    g_free(str_to); g_free(str_from_stack);
		    continue;
		}

		stack_automata_remove_transition(sa,
						 str_from, str_symbol, str_to,
						 str_from_stack, str_to_stack);
		g_free(str_from_stack); g_free(str_to_stack);
	    }
	    g_free(str_to);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result == 0 || result == 1) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0) {
	    break;
	}
    }
} /* remove_transition_cb */

void remove_terminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    gint result;
    char *str_sym;

    dialog = gnome_dialog_new(_("Remove terminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(gram.terminal_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		gram.terminal_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_sym = g_strdup_printf("%i", symbol->id);
	
	    if (get_current() == REGULAR)
		regular_grammar_remove_terminal_symbol(rg, str_sym);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_remove_terminal_symbol(cg, str_sym);
	    else if (get_current() == GRAMMAR)
		grammar_remove_terminal_symbol(g, str_sym);

	    g_free(str_sym);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* remove_terminal_symbol_cb */

void remove_nonterminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    gint result;
    char *str_sym;

    dialog = gnome_dialog_new(_("Remove nonterminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(gram.nonterminal_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		gram.nonterminal_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_sym = g_strdup_printf("%i", symbol->id);
	
	    if (get_current() == REGULAR)
		regular_grammar_remove_nonterminal_symbol(rg, str_sym);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_remove_nonterminal_symbol(cg, str_sym);
	    else if (get_current() == GRAMMAR)
		grammar_remove_nonterminal_symbol(g, str_sym);

	    g_free(str_sym);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* remove_nonterminal_symbol_cb */

void remove_rule_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wleft, *wright;
    GtkWidget *hbox;
    gint result;

    dialog = gnome_dialog_new(_("Remove rule"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Left: ")),
                       FALSE, FALSE, 5);
    wleft = make_symbols_combo(gram.nonterminal_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wleft, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Right: ")),
                       FALSE, FALSE, 5);
    wright = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wright, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *left, *right;
	
	    left = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wleft)->entry)),
		gram.nonterminal_symbols, NULL);

	    if (left == NULL) {
		invalid_input_message();
		continue;
	    }

	    right = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(wright)),
		gram.terminal_symbols, gram.nonterminal_symbols);

	    if (right == NULL) {
		invalid_input_message();
		g_free(left);
		continue;
	    }

	    if (get_current() == REGULAR)
		regular_grammar_remove_rule(rg, left, right);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_remove_rule(cg, left, right);
	    else if (get_current() == GRAMMAR)
		grammar_remove_rule(g, left, right);

	    g_free(left); g_free(right);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* remove_rule_cb */

void edit_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wname, *wnew;
    gint result;

    dialog = gnome_dialog_new(_("Edit State"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("State:")),
                       FALSE, FALSE, 5);
    wname = make_symbols_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), wname, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Name:")),
                       FALSE, FALSE, 5);
    wnew = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wnew, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *id, *name;
	
	    id = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wname)->entry)),
		aut.states, NULL);
	    if (id == NULL) {
		invalid_input_message();
		continue;
	    }

	    name = deparen(gtk_entry_get_text(GTK_ENTRY(wnew)));

	    if (get_current() == FINITE)
		finite_automata_edit_state(fa, id, name);
	    else if (get_current() == STACK)
		stack_automata_edit_state(sa, id, name);
	    g_free(id); g_free(name);

	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* edit_state_cb */

void edit_entry_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wname, *wnew;
    gint result;

    dialog = gnome_dialog_new(_("Edit entry symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Entry symbol:")),
                       FALSE, FALSE, 5);
    wname = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wname, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Name:")),
                       FALSE, FALSE, 5);
    wnew = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wnew, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *id, *name;
	
	    id = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wname)->entry)),
		aut.entry_symbols, NULL);
	    if (id == NULL) {
		invalid_input_message();
		continue;
	    }

	    name = deparen(gtk_entry_get_text(GTK_ENTRY(wnew)));

	    if (get_current() == FINITE)
		finite_automata_edit_entry_symbol(fa, id, name);
	    else if (get_current() == STACK)
		stack_automata_edit_entry_symbol(sa, id, name);
	    g_free(id); g_free(name);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* edit_entry_symbol_cb */

void edit_stack_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wname, *wnew;
    gint result;

    dialog = gnome_dialog_new(_("Edit stack symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Stack symbol:")),
                       FALSE, FALSE, 5);
    wname = make_symbols_combo(aut.stack_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wname, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Name:")),
                       FALSE, FALSE, 5);
    wnew = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wnew, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *id, *name;
	
	    id = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wname)->entry)),
		aut.stack_symbols, NULL);
	    if (id == NULL) {
		invalid_input_message();
		continue;
	    }

	    name = deparen(gtk_entry_get_text(GTK_ENTRY(wnew)));

	    stack_automata_edit_stack_symbol(sa, id, name);
	    g_free(id); g_free(name);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* edit_stack_symbol_cb */

void edit_terminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wname, *wnew;
    gint result;

    dialog = gnome_dialog_new(_("Edit terminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Terminal symbol:")),
                       FALSE, FALSE, 5);
    wname = make_symbols_combo(gram.terminal_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wname, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Name:")),
                       FALSE, FALSE, 5);
    wnew = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wnew, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *id, *name;
	
	    id = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wname)->entry)),
		gram.terminal_symbols, NULL);
	    if (id == NULL) {
		invalid_input_message();
		continue;
	    }

	    name = deparen(gtk_entry_get_text(GTK_ENTRY(wnew)));

	    if (get_current() == REGULAR)
		regular_grammar_edit_terminal_symbol(rg, id, name);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_edit_terminal_symbol(cg, id, name);
	    else if (get_current() == GRAMMAR)
		grammar_edit_terminal_symbol(g, id, name);

	    g_free(id); g_free(name);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* edit_terminal_symbol_cb */

void edit_nonterminal_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wname, *wnew;
    gint result;

    dialog = gnome_dialog_new(_("Edit nonterminal symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Nonterminal symbol:")),
                       FALSE, FALSE, 5);
    wname = make_symbols_combo(gram.nonterminal_symbols);
    gtk_box_pack_start(GTK_BOX(hbox), wname, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Name:")),
                       FALSE, FALSE, 5);
    wnew = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wnew, FALSE, FALSE, 0);

    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *id, *name;
	
	    id = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wname)->entry)),
		gram.nonterminal_symbols, NULL);
	    if (id == NULL) {
		invalid_input_message();
		continue;
	    }

	    name = deparen(gtk_entry_get_text(GTK_ENTRY(wnew)));

	    if (get_current() == REGULAR)
		regular_grammar_edit_nonterminal_symbol(rg, id, name);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_edit_nonterminal_symbol(cg, id, name);
	    else if (get_current() == GRAMMAR)
		grammar_edit_nonterminal_symbol(g, id, name);

	    g_free(id); g_free(name);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* edit_nonterminal_symbol_cb */

void set_start_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wstate;
    gint result;
    char *str_state;
    
    dialog = gnome_dialog_new(_("Set start state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    
    wstate = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstate,
		       FALSE, FALSE, 0);
    
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    State *state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_state = g_strdup_printf("%i", state->id);
	
	    if (get_current() == FINITE)
		finite_automata_set_start_state(fa, str_state);
	    else if (get_current() == STACK)
		stack_automata_set_start_state(sa, str_state);

	    g_free(str_state);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* set_start_state_cb */

void set_end_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wstate;
    gint result;
    char *str_state;

    dialog = gnome_dialog_new(_("Set end state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wstate = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstate,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    State *state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_state = g_strdup_printf("%i", state->id);
	
	    if (get_current() == FINITE)
		finite_automata_set_end_state(fa, str_state);
	    else if (get_current() == STACK)
		stack_automata_set_end_state(sa, str_state);

	    g_free(str_state);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* set_end_state_cb */

void reset_end_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wstate;
    gint result;
    char *str_state;

    dialog = gnome_dialog_new(_("Reset end state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wstate = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstate,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    State *state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_state = g_strdup_printf("%i", state->id);
	
	    if (get_current() == FINITE)
		finite_automata_reset_end_state(fa, str_state);
	    else if (get_current() == STACK)
		stack_automata_reset_end_state(sa, str_state);
	    
	    g_free(str_state);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* reset_end_state_cb */

void set_start_stack_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *stack_symbol;
    gint result;
    char *str_symbol;

    dialog = gnome_dialog_new(_("Set start stack symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    stack_symbol = make_symbols_combo(aut.stack_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), stack_symbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(stack_symbol)->entry)),
		aut.stack_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_symbol = g_strdup_printf("%i", symbol->id);

	    stack_automata_set_start_stack_symbol(sa, str_symbol);

	    g_free(str_symbol);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* set_start_stack_symbol_cb */

void set_start_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *non_symbol;
    gint result;
    char *str_symbol;

    dialog = gnome_dialog_new(_("Set start symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    non_symbol = make_symbols_combo(gram.nonterminal_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), non_symbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(non_symbol)->entry)),
		gram.nonterminal_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_symbol = g_strdup_printf("%i", symbol->id);

	    if (get_current() == REGULAR)
		regular_grammar_set_start_symbol(rg, str_symbol);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_set_start_symbol(cg, str_symbol);
	    else if (get_current() == GRAMMAR)
		grammar_set_start_symbol(g, str_symbol);

	    g_free(str_symbol);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* set_start_symbol_cb */

void direct_add_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    gint result;

    dialog = gnome_dialog_new(_("Add transition"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    char *str_symbol, *str_from, *str_to;
	    State *state;
	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		aut.entry_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		continue;
	    }
	    str_symbol = g_strdup_printf("%i", symbol->id);

	    state = states->data;
	    g_return_if_fail(state != NULL);
	    str_from = g_strdup_printf("%i", state->id);
	    state = states->next->data;
	    g_return_if_fail(state != NULL);
	    str_to = g_strdup_printf("%i", state->id);

	    finite_automata_add_transition(fa, str_from, str_symbol, str_to);

	    g_free(str_symbol); g_free(str_from); g_free(str_to);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* direct_add_transition_cb */

void direct_set_start_state_cb(GtkWidget *widget, gpointer data)
{
    char *id;
    State *state = find_state_by_group(data);
    g_return_if_fail(state != NULL);
    
    id = g_strdup_printf("%i", state->id);
    finite_automata_set_start_state(fa, id);

    g_free(id);
    if (!check_error(formal_get_error()))
	show_all();
} /* direct_set_start_state_cb */

void direct_set_end_state_cb(GtkWidget *widget, gpointer data)
{
    char *id;
    State *state = find_state_by_group(data);
    g_return_if_fail(state != NULL);
    
    id = g_strdup_printf("%i", state->id);
    finite_automata_set_end_state(fa, id);

    g_free(id);
    if (!check_error(formal_get_error()))
	show_all();
} /* direct_set_end_state_cb */

void direct_reset_end_state_cb(GtkWidget *widget, gpointer data)
{
    char *id;
    State *state = find_state_by_group(data);
    g_return_if_fail(state != NULL);
    
    id = g_strdup_printf("%i", state->id);
    finite_automata_reset_end_state(fa, id);

    g_free(id);
    if (!check_error(formal_get_error()))
	show_all();
} /* direct_reset_end_state_cb */

void direct_edit_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wname;
    State *state;
    gint result;

    state = find_state_by_group(data);
    g_return_if_fail(state != NULL);

    dialog = gnome_dialog_new(_("Edit state"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Name: "),
                                                      state->name,
                                                      &wname),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wname);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wname));
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	gchar *str_state = g_strdup_printf("%i", state->id);
	gchar *name = deparen(gtk_entry_get_text(GTK_ENTRY(wname)));

	finite_automata_edit_state(fa, str_state, name);
	g_free(name); g_free(str_state);
	if (!check_error(formal_get_error()))
	    show_all();
    }
    if (result >= 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* direct_edit_state_cb */

void direct_delete_state_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    State *state;
    gint result;

    state = find_state_by_group(data);
    g_return_if_fail(state != NULL);

    dialog = gnome_message_box_new(_("Really delete this state?"),
				    GNOME_MESSAGE_BOX_QUESTION,
				    GNOME_STOCK_BUTTON_OK,
				    GNOME_STOCK_BUTTON_CANCEL,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	gchar *str_state = g_strdup_printf("%i", state->id);
	finite_automata_remove_state(fa, str_state);

	g_free(str_state);
	if (!check_error(formal_get_error()))
	    show_all();
    }
} /* direct_delete_state_cb */

void direct_add_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    Transition *trans;
    gint result;

    trans = find_transition_by_group(data);
    g_return_if_fail(trans != NULL);

    dialog = gnome_dialog_new(_("Add entry symbol to transition"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(aut.entry_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    gchar *str_from = g_strdup_printf("%i", trans->from->id);
	    gchar *str_to = g_strdup_printf("%i", trans->to->id);
	    gchar *str_symbol;

	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		aut.entry_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		g_free(str_from); g_free(str_to);
		continue;
	    }
	    str_symbol = g_strdup_printf("%i", symbol->id);

	    finite_automata_add_transition(fa, str_from, str_symbol, str_to);

	    g_free(str_symbol); g_free(str_from); g_free(str_to);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* direct_add_symbol_cb */

void direct_delete_symbol_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsymbol;
    Transition *trans;
    gint result;

    trans = find_transition_by_group(data);
    g_return_if_fail(trans != NULL);

    dialog = gnome_dialog_new(_("Delete entry symbol from transition"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    wsymbol = make_symbols_combo(trans->entry_symbols);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wsymbol,
		       FALSE, FALSE, 0);

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));
	if (result == 0) {
	    gchar *from = g_strdup_printf("%i", trans->from->id);
	    gchar *to = g_strdup_printf("%i", trans->to->id);
	    gchar *str_symbol;

	    Symbol *symbol = find_symbol_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wsymbol)->entry)),
		aut.entry_symbols);
	    if (symbol == NULL) {
		invalid_input_message();
		g_free(from); g_free(to);
		continue;
	    }
	    str_symbol = g_strdup_printf("%i", symbol->id);

	    finite_automata_remove_transition(fa, from, str_symbol, to);

	    g_free(str_symbol); g_free(from); g_free(to);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* direct_delete_symbol_cb */

void direct_delete_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    Transition *trans;
    gint result;

    trans = find_transition_by_group(data);
    g_return_if_fail(trans != NULL);

    dialog = gnome_message_box_new(_("Really delete this transition?"),
				    GNOME_MESSAGE_BOX_QUESTION,
				    GNOME_STOCK_BUTTON_OK,
				    GNOME_STOCK_BUTTON_CANCEL,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	gchar *from = g_strdup_printf("%i", trans->from->id);
	gchar *to = g_strdup_printf("%i", trans->to->id);
	GSList *l;

	for (l = trans->entry_symbols; l != NULL; l = l->next) {
	    Symbol *symbol = l->data;
	    gchar *str_symbol = g_strdup_printf("%i", symbol->id);
	    finite_automata_remove_transition(fa, from, str_symbol, to);
	    g_free(str_symbol);
	}
	g_free(from); g_free(to);
	if (!check_error(formal_get_error()))
	    show_all();
    }
} /* direct_delete_transition */

void direct_delete_stack_trans_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GList *list = data;
    gint result;

    dialog = gnome_message_box_new(_("Really delete selected transitions?"),
				    GNOME_MESSAGE_BOX_QUESTION,
				    GNOME_STOCK_BUTTON_OK,
				    GNOME_STOCK_BUTTON_CANCEL,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    
    if (result == 0) {
	GList *k;
	for (k = list; k != NULL; k = k->next) {
	    Transition *trans
		= gtk_clist_get_row_data(GTK_CLIST(my_clist),
					 GPOINTER_TO_INT(k->data));
	    gchar *from = g_strdup_printf("%i", trans->from->id);
	    gchar *to = g_strdup_printf("%i", trans->to->id);
	    GSList *l = trans->entry_symbols;
	    gchar *str_symbol, *from_stack, *to_stack;
	    GString *aux;
	    
	    if (l != NULL) {
		Symbol *symbol = l->data;
		str_symbol = g_strdup_printf("%i", symbol->id);
	    } else
		str_symbol = g_strdup("0");

	    aux = g_string_sized_new(1);
	    for (l = trans->from_stack; l != NULL; l = l->next) {
		Symbol *symbol = l->data;
		g_string_sprintfa(aux, "%i ", symbol->id);
	    }
	    from_stack = g_strdup(aux->str);

	    aux = g_string_truncate(aux, 0);
	    for (l = trans->to_stack; l != NULL; l = l->next) {
		Symbol *symbol = l->data;
		g_string_sprintfa(aux, "%i ", symbol->id);
	    }
	    to_stack = g_strdup(aux->str);
	    
	    stack_automata_remove_transition(sa, from, str_symbol, to,
					     from_stack, to_stack);

	    g_free(from); g_free(to); g_free(str_symbol);
	    g_free(from_stack); g_free(to_stack);
	    g_string_free(aux, TRUE);
	}
	if (!check_error(formal_get_error()))
	    show_all();
    }
} /* direct_delete_stack_trans_cb */

void direct_delete_rule_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GList *list = data;
    gint result;

    dialog = gnome_message_box_new(_("Really delete selected rules?"),
				    GNOME_MESSAGE_BOX_QUESTION,
				    GNOME_STOCK_BUTTON_OK,
				    GNOME_STOCK_BUTTON_CANCEL,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    result = gnome_dialog_run(GNOME_DIALOG(dialog));

    if (result == 0) {
	GList *k;
	for (k = list; k != NULL; k = k->next) {
	    Rule *rule = gtk_clist_get_row_data(GTK_CLIST(my_clist),
						GPOINTER_TO_INT(k->data));
	    gchar *left, *right;
	    GSList *l;
	    GString *aux;

	    aux = g_string_sized_new(1);
	    for (l = rule->left; l != NULL; l = l->next) {
		Symbol *symbol = l->data;
		g_string_sprintfa(aux, "%i ", symbol->id);
	    }
	    left = g_strdup(aux->str);
	    
	    aux = g_string_truncate(aux, 0);
	    for (l = rule->right; l != NULL; l = l->next) {
		Symbol *symbol = l->data;
		g_string_sprintfa(aux, "%i ", symbol->id);
	    }
	    right = g_strdup(aux->str);

	    if (get_current() == REGULAR)
		regular_grammar_remove_rule(rg, left, right);
	    else if (get_current() == CONTEXTFREE)
		contextfree_grammar_remove_rule(cg, left, right);
	    else if (get_current() == GRAMMAR)
		grammar_remove_rule(g, left, right);

	    g_string_free(aux, TRUE);
	    g_free(left); g_free(right);
	}
	if (!check_error(formal_get_error()))
	    show_all();
    }
} /* direct_delete_rule_cb */

void get_inaccessible_states_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    
    clear_other_data();
    parse(finite_automata_get_inaccessible_states(fa), NULL, "y");

    dialog = gnome_dialog_new(_("Inaccessible states"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_states_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_inaccessible_states_cb */

void get_redundant_symbols_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;

    clear_other_data();    
    parse(contextfree_grammar_get_redundant_symbols(cg), NULL, "y");

    dialog = gnome_dialog_new(_("Redundant symbols"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_symbols_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_redundant_symbols_cb */

void get_recursive_symbols_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    
    clear_other_data();
    parse(contextfree_grammar_get_recursive_symbols(cg), NULL, "y");

    dialog = gnome_dialog_new(_("Recursive symbols"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_symbols_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_recursive_symbols_cb */

void get_inaccessible_symbols_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    
    clear_other_data();
    parse(contextfree_grammar_get_inaccessible_symbols(cg), NULL, "y");

    dialog = gnome_dialog_new(_("Inaccessible symbols"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_symbols_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_inaccessible_symbols_cb */

void get_simple_rules_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    
    clear_other_data();
    parse(contextfree_grammar_get_simple_rules(cg), NULL, "y");

    dialog = gnome_dialog_new(_("Simple rules"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_rules_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_simple_rules_cb */

void get_e_rules_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    
    clear_other_data();
    parse(contextfree_grammar_get_e_rules(cg), NULL, "y");

    dialog = gnome_dialog_new(_("Rules containig empty symbol"),
                              GNOME_STOCK_BUTTON_OK,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
		       make_rules_text(),
		       TRUE, TRUE, 0);

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* get_e_rules_cb */

void check_sentence_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wsentence;
    gint result;

    dialog = gnome_dialog_new(_("Check sentence"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
                       make_hbox_with_label_and_entry(_("Sentence: "),
                                                      _(""),
                                                      &wsentence),
                       FALSE, FALSE, 0);
    gtk_widget_grab_focus(wsentence);
    gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(wsentence));

    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    bool recognised = FALSE;
	    char *sentence = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(wsentence)),
		aut.entry_symbols, NULL);

	    if (sentence == NULL) {
		invalid_input_message();
		continue;
	    }

	    if (get_current() == FINITE)
		recognised = finite_automata_check_sentence(fa, sentence);
	    else if (get_current() == STACK)
		recognised = stack_automata_check_sentence(sa, sentence);

	    if (recognised) {
		GtkWidget *info = gnome_message_box_new(
		    _("Sentence recognised"),
		    GNOME_MESSAGE_BOX_INFO,
		    GNOME_STOCK_BUTTON_OK,
		    NULL);
		gnome_dialog_set_parent(GNOME_DIALOG(info), GTK_WINDOW(app));
		gnome_dialog_run(GNOME_DIALOG(info));
	  } else {
		GtkWidget *info = gnome_message_box_new(
		    _("Sentence was NOT recognised"),
		    GNOME_MESSAGE_BOX_GENERIC,
		    GNOME_STOCK_BUTTON_OK,
		    NULL);
		gnome_dialog_set_parent(GNOME_DIALOG(info), GTK_WINDOW(app));
		gnome_dialog_run(GNOME_DIALOG(info));
	    }
	    g_free(sentence);
	} else if (result > 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* check_sentence_cb */

void set_configuration_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *hbox;
    GtkWidget *wstate, *wsentence, *wstack;
    gint result;

    dialog = gnome_dialog_new(_("Set configuration"),
                              GNOME_STOCK_BUTTON_OK,
                              GNOME_STOCK_BUTTON_CANCEL,
                              NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("State:")),
                       FALSE, FALSE, 5);
    wstate = make_states_combo(aut.states);
    gtk_box_pack_start(GTK_BOX(hbox), wstate, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox),
                       gtk_label_new(_("Sentence:")),
                       FALSE, FALSE, 5);
    wsentence = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), wsentence, FALSE, FALSE, 0);

    if (get_current() == STACK) {
	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("Stack:")),
			   FALSE, FALSE, 5);
	wstack = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), wstack, FALSE, FALSE, 0);
    }

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
                       FALSE, FALSE, 0);
    gtk_widget_show_all(hbox);
    while (1) {
	result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	if (result == 0) {
	    char *sstate, *sentence;
	    State *state = find_state_by_name(
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
	    if (state == NULL) {
		invalid_input_message();
		continue;
	    }
	    sstate = g_strdup_printf("%i", state->id);

	    sentence = get_id_string_of_symbols(
		gtk_entry_get_text(GTK_ENTRY(wsentence)),
		aut.entry_symbols, NULL);

	    if (sentence == NULL) {
		invalid_input_message();
		g_free(sstate);
		continue;
	    }

	    if (get_current() == FINITE)
		finite_automata_set_configuration(fa, sstate, sentence);
	    else if (get_current() == STACK) {
		gchar *stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(wstack)),
		    aut.stack_symbols, NULL);

		if (stack == NULL) {
		    invalid_input_message();
		    g_free(sstate); g_free(sentence);
		    continue;
		}
		stack_automata_set_configuration(sa, sstate, sentence, stack);
		g_free(stack);
	    }
	    g_free(sstate); g_free(sentence);
	    if (!check_error(formal_get_error()))
		show_all();
	}
	if (result >= 0) {
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	    break;
	} else if (result < 0)
	    break;
    }
} /* set_configuration_cb */

void is_deterministic_cb(GtkWidget *widget, gpointer data)
{
    bool result = FALSE;
    if (get_current() == FINITE)
	result = finite_automata_is_deterministic(fa);
    else if (get_current() == STACK)
	result = stack_automata_is_deterministic(sa);

    if (result) {
	gnome_app_message(GNOME_APP(app), _("Automata is deterministic"));
    } else if (get_current() == FINITE) {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To deterministic"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("Automata is NOT deterministic"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_deterministic_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    } else {
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("Automata is NOT deterministic"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), text,
			   FALSE, FALSE, 0);
	if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	    gnome_dialog_close(GNOME_DIALOG(dialog));
    }
} /* is_deterministic_cb */

void is_top_stack_working_cb(GtkWidget *widget, gpointer data)
{
    if (stack_automata_is_top_stack_working(sa)) {
	gnome_ok_dialog_parented(_("Automata is working with top of a stack"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To top stack working"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("Automata is NOT working with top of stack"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_top_stack_working_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }
} /* is_top_stack_working_cb */

void is_empty_stack_recognising_cb(GtkWidget *widget, gpointer data)
{
    if (stack_automata_is_empty_stack_checking(sa)) {
	gnome_ok_dialog_parented(_("Automata recognises by empty stack"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To empty stack recognising"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("Automata is NOT recognising by empty stack"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_empty_stack_recognising_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }
} /* is_empty_stack_recognising_cb */

void get_classification_cb(GtkWidget *widget, gpointer data)
{
    gchar *result = NULL;
    if (get_current() == REGULAR)
	result = regular_grammar_get_classification(rg);
    else if (get_current() == CONTEXTFREE)
	result = contextfree_grammar_get_classification(cg);
    else if (get_current() == GRAMMAR)
	result = grammar_get_classification(g);
    gnome_app_message(GNOME_APP(app), result);
} /* get_classification_cb */

void is_e_rules_free_cb(GtkWidget *widget, gpointer data)
{
    if (contextfree_grammar_is_e_rules_free(cg)) {
	gnome_ok_dialog_parented(_("This grammar is empty rules free"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("Remove E rules"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("This grammar contains empty rules"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run =gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		remove_e_rules_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }    
} /* is_e_rules_free_cb */

void is_self_cb(GtkWidget *widget, gpointer data)
{
    if (contextfree_grammar_is_self(cg)) {
	gnome_ok_dialog_parented(_("This grammar is self"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To self"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("This grammar is not self"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_self_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }    
} /* is_self_cb */

void is_in_chomsky_nf_cb(GtkWidget *widget, gpointer data)
{
    if (contextfree_grammar_is_in_chomsky_normal_form(cg)) {
	gnome_ok_dialog_parented(_("This grammar is in Chomsky normal form"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To Chomsky normal form"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("This grammar is not in Chomsky normal form"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_chomsky_nf_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }    
} /* is_in_chomsky_nf_cb */

void is_in_greibach_nf_cb(GtkWidget *widget, gpointer data)
{
    if (contextfree_grammar_is_in_greibach_normal_form(cg)) {
	gnome_ok_dialog_parented(_("This grammar is in Greibach normal form"),
				 GTK_WINDOW(app));	    
    } else {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("To Greibach normal form"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("This grammar is not in Greibach normal form"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		to_greibach_nf_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    }    
} /* is_in_greibach_nf_cb */

void is_recursive_cb(GtkWidget *widget, gpointer data)
{
    if (contextfree_grammar_is_recursive(cg)) {
	int run;
	GtkWidget *text;
	GtkWidget *dialog = gnome_dialog_new(_("Test"),
					     _("Eliminate left recursion"),
					     GNOME_STOCK_BUTTON_OK,
					     NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	
	text = gtk_label_new(_("This grammar is recursive"));
	gtk_widget_show(text);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
			   text,
			   FALSE, FALSE, 0);
	
	if ((run = gnome_dialog_run(GNOME_DIALOG(dialog))) >= 0) {
	    if (run == 0)
		eliminate_left_recursion_cb(dialog, NULL);
	    gnome_dialog_close(GNOME_DIALOG(dialog));
	}
    } else {
	gnome_ok_dialog_parented(_("This grammar is not recursive"),
				 GTK_WINDOW(app));	    
    }
} /* is_recursive_cb */

void do_finite_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;

    clear_other_data();
    parse(finite_automata_get_transition_destinations(fa), NULL, "y");

    switch (g_slist_length(states))
    {
    case 0:
	dialog = gnome_message_box_new(_("No valid transition from this state"),
				       GNOME_MESSAGE_BOX_INFO,
				       GNOME_STOCK_BUTTON_OK,
				       NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	gnome_dialog_run(GNOME_DIALOG(dialog));
	break;
    case 1:
    {
	State *state = states->data;
	gchar *sstate = g_strdup_printf("%i", state->id);
	finite_automata_do_transition(fa, sstate);
	g_free(sstate);
	if (!check_error(formal_get_error()))
	    show_all();
	break;
    }
    default:
    {
	GtkWidget *hbox;
	GtkWidget *wstate;
	dialog = gnome_dialog_new(_("Destinations"),
				  GNOME_STOCK_BUTTON_OK,
				  GNOME_STOCK_BUTTON_CANCEL,
				  NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	hbox = gtk_hbox_new(FALSE, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox),
			   gtk_label_new(_("Choose destination:")),
			   FALSE, FALSE, 5);
	wstate = make_states_combo(states);
	gtk_box_pack_start(GTK_BOX(hbox), wstate, FALSE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
			   FALSE, FALSE, 0);
	gtk_widget_show_all(hbox);
	
	while (1) {
	    int result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	    if (result == 0) {
		char *sstate;
		State *state = find_state_by_name(
		    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstate)->entry)));
		if (state == NULL) {
		    invalid_input_message();
		    continue;
		}
		sstate = g_strdup_printf("%i", state->id);

		finite_automata_do_transition(fa, sstate);
		g_free(sstate);
		if (!check_error(formal_get_error()))
		    show_all();
	    }
	    if (result >= 0) {
		gnome_dialog_close(GNOME_DIALOG(dialog));
		break;
	    } else if (result < 0)
		break;
	}
	break;
    } /* default */
    } /* switch */
} /* do_finite_transition_cb */

void do_stack_transition_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *wstack, *wstate;
    Pair *pair;
    GSList *l;
    GList *stacks = NULL;
    GList *states = NULL;
    GList *t;
    gchar *from_stack, *to_stack, *sstate, *tmp;
    int result;

    clear_other_data();
    parse(stack_automata_get_from_stacks(sa), NULL, "y");

    switch (g_slist_length(pairs)) {
    case 0:
	from_stack = g_strdup("");
	break;
    case 1:
	    pair = pairs->data;
	    if (pair->stack != NULL) {
		tmp = get_string_of_symbols(pair->stack, FALSE);
		from_stack = get_id_string_of_symbols(tmp,
						      aut.stack_symbols, NULL);
		g_free(tmp);
	    } else
		from_stack = g_strdup("");
	break;
    default:
	for (l = pairs; l != NULL; l = l->next) {
	    pair = l->data;
	    if (pair->stack != NULL) {
		stacks = g_list_append(stacks,
				       get_string_of_symbols(pair->stack,
							     FALSE));
	    } else
		stacks = g_list_append(stacks, " ");
	}
	dialog = gnome_dialog_new(_("Select symbols to remove from stack"),
				  GNOME_STOCK_BUTTON_OK,
				  GNOME_STOCK_BUTTON_CANCEL,
				  NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	wstack = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(wstack), stacks);
	gtk_widget_show(wstack);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstack,
			   FALSE, FALSE, 0);

	while (1) {
	    result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	    if (result == 0) {
		from_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstack)->entry)),
		    aut.stack_symbols, NULL);
			
		if (from_stack == NULL) {
		    invalid_input_message();
		    continue;
		}
		gnome_dialog_close(GNOME_DIALOG(dialog));
		break;
	    }
	    if (result > 0) {
		gnome_dialog_close(GNOME_DIALOG(dialog));
		for (t = stacks; t != NULL; t = t->next)
		    g_free(t->data);
		g_list_free(stacks); stacks = NULL;
		return;
	    } else if (result < 0) {
		for (t = stacks; t != NULL; t = t->next)
		    g_free(t->data);
		g_list_free(stacks); stacks = NULL;
		return;
	    }
	}
    }

    for (t = stacks; t != NULL; t = t->next)
	g_free(t->data);
    g_list_free(stacks); stacks = NULL;

    clear_other_data();
    parse(stack_automata_get_transition_destinations(sa, from_stack),
	  NULL, "y");

    switch (g_slist_length(pairs)) {
    case 0:
	dialog = gnome_message_box_new(_("No valid transition from this state"),
				       GNOME_MESSAGE_BOX_INFO,
				       GNOME_STOCK_BUTTON_OK,
				       NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	gnome_dialog_run(GNOME_DIALOG(dialog));
	g_free(from_stack);
	break;
    case 1:
    {
	State *state;
	pair = pairs->data;
	state = pair->state;
	sstate = g_strdup_printf("%i", state->id);
	if (pair->stack != NULL) {
	    tmp = get_string_of_symbols(pair->stack, FALSE);
	    to_stack = get_id_string_of_symbols(tmp, aut.stack_symbols, NULL);
	    g_free(tmp);
	} else
	    to_stack = g_strdup("");

	stack_automata_do_transition(sa, sstate, from_stack, to_stack);
	g_free(from_stack); g_free(sstate); g_free(to_stack);
	if (!check_error(formal_get_error()))
	    show_all();
	break;
    }
    default:
	for (l = pairs; l != NULL; l = l->next) {
	    pair = l->data;
	    if (pair->stack != NULL) {
		stacks = g_list_append(stacks,
				       get_string_of_symbols(pair->stack,
							     FALSE));
	    } else
		stacks = g_list_append(stacks, " ");
	    states = g_list_append(states, pair->state->name);
	}
	dialog = gnome_dialog_new(_("Select symbols to remove from stack"),
				  GNOME_STOCK_BUTTON_OK,
				  GNOME_STOCK_BUTTON_CANCEL,
				  NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	wstate = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(wstate), states);
	gtk_widget_show(wstate);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstate,
			   FALSE, FALSE, 0);

	wstack = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(wstack), stacks);
	gtk_widget_show(wstack);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), wstack,
			   FALSE, FALSE, 0);

	while (1) {
	    result = gnome_dialog_run(GNOME_DIALOG(dialog));    
	    if (result == 0) {
		State *state;
		to_stack = get_id_string_of_symbols(
		    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstack)->entry)),
		    aut.stack_symbols, NULL);

		if (from_stack == NULL) {
		    invalid_input_message();
		    continue;
		}
		
		state = find_state_by_name(
		    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wstack)->entry)));

		if (state == NULL) {
		    invalid_input_message();
		    g_free(to_stack);
		    continue;
		}
		sstate = g_strdup_printf("%i", state->id);

		stack_automata_do_transition(sa, sstate, from_stack, to_stack);
		gnome_dialog_close(GNOME_DIALOG(dialog));
		g_free(from_stack); g_free(to_stack); g_free(sstate);
		if (!check_error(formal_get_error()))
		    show_all();
		break;
	    }
	    if (result > 0) {
		gnome_dialog_close(GNOME_DIALOG(dialog));
		break;
	    } else if (result < 0)
		break;
	}
    }
    for (t = stacks; t != NULL; t = t->next)
	g_free(t->data);
    g_list_free(stacks); stacks = NULL;
} /* do_stack_transition_cb */

void to_deterministic_cb(GtkWidget *widget, gpointer data)
{
    finite_automata_to_deterministic(fa);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_deterministic_cb */

void to_reduced_cb(GtkWidget *widget, gpointer data)
{
    finite_automata_to_reduced(fa);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_reduced_cb */

void to_top_stack_working_cb(GtkWidget *widget, gpointer data)
{
    stack_automata_to_top_stack_working(sa);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_top_stack_working_cb */

void to_empty_stack_recognising_cb(GtkWidget *widget, gpointer data)
{
    stack_automata_to_empty_stack_checking(sa);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_empty_stack_recognising_cb */

void to_end_states_recognising_cb(GtkWidget *widget, gpointer data)
{
    stack_automata_to_end_states_checking(sa);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_end_states_recognising_cb */

void to_extended_cb(GtkWidget *widget, gpointer data)
{
    if (get_current() == REGULAR)
	regular_grammar_to_extended(rg);
    else if (get_current() == CONTEXTFREE)
	contextfree_grammar_to_extended(cg);
    else if (get_current() == GRAMMAR)
	grammar_to_extended(g);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_extended_cb */

void to_self_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_to_self(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_self_cb */

void to_chomsky_nf_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_to_chomsky_normal_form(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_chomsky_nf_cb */

void to_greibach_nf_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_to_greibach_normal_form(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* to_greibach_nf_cb */

void remove_redundant_symbols_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_remove_redundant_symbols(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* remove_redundant_symbols_cb */

void remove_e_rules_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_remove_e_rules(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* remove_e_rules_cb */

void remove_simple_rules_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_remove_simple_rules(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* remove_simple_rules_cb*/

void eliminate_left_recursion_cb(GtkWidget *widget, gpointer data)
{
    contextfree_grammar_eliminate_left_recursion(cg);
    if (!check_error(formal_get_error()))
	show_all();
} /* eliminate_left_recursion_cb */

void convert_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;

    dialog = gnome_message_box_new(_("Really convert?"
				     " Current document will be lost"),
				    GNOME_MESSAGE_BOX_WARNING,
				    GNOME_STOCK_BUTTON_OK,
				    GNOME_STOCK_BUTTON_CANCEL,
				    NULL);

    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    if (gnome_dialog_run(GNOME_DIALOG(dialog)) != 0)
	return;

    switch (get_current()) {
    case FINITE:
	rg = finite_automata_to_regular_grammar(fa);
	set_current(REGULAR); break;
    case STACK:
	cg = stack_automata_to_contextfree_grammar(sa);
	set_current(CONTEXTFREE); break;
    case REGULAR:
	fa = regular_grammar_to_finite_automata(rg);
	set_current(FINITE); break;
    case CONTEXTFREE:
	dialog = gnome_message_box_new(_("Should the resulting automata work"
					 " top-down style?"),
				       GNOME_MESSAGE_BOX_QUESTION,
				       GNOME_STOCK_BUTTON_OK,
				       GNOME_STOCK_BUTTON_NO,
				       NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0)
	    sa = contextfree_grammar_to_stack_automata(cg, TRUE);
	else
	    sa = contextfree_grammar_to_stack_automata(cg, FALSE);
	set_current(STACK); break;
    case GRAMMAR:
	break;
    case NOTHING:
	break;
    }

    if (filename != NULL && g_file_exists(filename)) {
	g_free(filename);
	filename = NULL;
    }
    set_comment(_("Converted form another type."));
    show_all();
    show_current_menu();
} /* convert_cb */

void comment_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;
    GtkWidget *entry;
    GtkWidget *swindow;
    int result = 0;

    dialog = gnome_dialog_new(_("Comment"),
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
    
    entry = gtk_text_new(NULL, NULL);
    swindow = gtk_scrolled_window_new(NULL, NULL);
    
    gtk_text_insert(GTK_TEXT(entry), NULL, NULL, NULL, get_comment(), -1);
    gtk_text_set_editable(GTK_TEXT(entry), TRUE);

    gtk_widget_set_usize(GTK_WIDGET(swindow), 400, 200);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_AUTOMATIC);

    gtk_container_add(GTK_CONTAINER(swindow), entry);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), swindow,
		       TRUE, TRUE, 0);

    gtk_widget_show_all(dialog);
    result = gnome_dialog_run(GNOME_DIALOG(dialog));
    if (result == 0) {
	char *text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
	set_comment(text);
	g_free(text);
	gnome_dialog_close(GNOME_DIALOG(dialog));
    } else if (result >0)
	gnome_dialog_close(GNOME_DIALOG(dialog));
} /* comment_cb */

static void undo_cb(GtkWidget *widget, gpointer data)
{
    switch (get_current()) {
    case FINITE:
	finite_automata_undo(fa); break;
    case STACK:
	stack_automata_undo(sa); break;
    case REGULAR:
	regular_grammar_undo(rg); break;
    case CONTEXTFREE:
	contextfree_grammar_undo(cg); break;
    case GRAMMAR:
	grammar_undo(g); break;
    case NOTHING:
	break;
    }
    show_all();
} /* undo_cb */

static void redo_cb(GtkWidget *widget, gpointer data)
{
    switch (get_current()) {
    case FINITE:
	finite_automata_redo(fa); break;
    case STACK:
	stack_automata_redo(sa); break;
    case REGULAR:
	regular_grammar_redo(rg); break;
    case CONTEXTFREE:
	contextfree_grammar_redo(cg); break;
    case GRAMMAR:
	grammar_redo(g); break;
    case NOTHING:
	break;
    }
    show_all();
} /* redo_cb */


/*
 *  New type
 */

static void new_cb(GtkWidget *widget, gpointer data)
{
    if (get_current() != NOTHING) {
	GtkWidget *message;
	message = gnome_message_box_new(_("Continue?"
					  " All unsaved data will be lost!"),
					GNOME_MESSAGE_BOX_WARNING,
					GNOME_STOCK_BUTTON_OK,
					GNOME_STOCK_BUTTON_CANCEL,
					NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(message), GTK_WINDOW(app));

	if (gnome_dialog_run(GNOME_DIALOG(message)) != 0)
    	    return;
    }

    switch (GPOINTER_TO_INT(data)) {
    case FINITE:
	fa = formal_new_finite_automata(); break;
    case STACK:
	sa = formal_new_stack_automata(); break;
    case REGULAR:
	rg = formal_new_regular_grammar(); break;
    case CONTEXTFREE:
	cg = formal_new_contextfree_grammar(); break;
    case GRAMMAR:
	g = formal_new_grammar(); break;
    }
    set_current(GPOINTER_TO_INT(data));

    if (filename != NULL && g_file_exists(filename)) {
	g_free(filename);
	filename = NULL;
    }
    set_comment(" ");
    show_all();
    show_current_menu();

    gnome_app_flash(GNOME_APP(app), _("Created new document"));
} /* new_cb */


/*
 *  File opening dialog
 */

static void open_ok(GtkWidget *widget, GtkFileSelection *fsel)
{
    char *fname;
    g_return_if_fail(GTK_IS_FILE_SELECTION(fsel));

    fname = gtk_file_selection_get_filename(fsel);
    if (!fname || !read_xml_file(fname)) {
	GtkWidget *dlg;

	dlg = gnome_error_dialog_parented(_("Cannot open file"),
					  GTK_WINDOW(fsel));
	gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
    } else {
	gtk_widget_destroy(GTK_WIDGET(fsel));
	show_all();
	show_current_menu();
    }
} /* open_ok */

static void open_cb(GtkWidget *widget, gpointer data)
{
    GtkFileSelection *fsel;

    if (get_current() != NOTHING) {
	GtkWidget *message;
	message = gnome_message_box_new(_("Continue?"
					  " All unsaved data will be lost!"),
					GNOME_MESSAGE_BOX_WARNING,
					GNOME_STOCK_BUTTON_OK,
					GNOME_STOCK_BUTTON_CANCEL,
					NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(message), GTK_WINDOW(app));

	if (gnome_dialog_run(GNOME_DIALOG(message)) != 0)
    	    return;
    }
    
    fsel = GTK_FILE_SELECTION(gtk_file_selection_new(_("Open")));
         
    gtk_signal_connect (GTK_OBJECT(fsel->ok_button),
			"clicked",
			GTK_SIGNAL_FUNC(open_ok),
			fsel);

    gtk_signal_connect_object (GTK_OBJECT(fsel->cancel_button),
			       "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			       GTK_OBJECT(fsel));

    gtk_window_position(GTK_WINDOW(fsel), GTK_WIN_POS_MOUSE);
    gtk_window_set_transient_for(GTK_WINDOW(fsel), GTK_WINDOW(app));

    gtk_widget_show(GTK_WIDGET(fsel));
} /* open_cb */

/*
 *  File saving dialogs
 */

static void save_ok(GtkWidget *widget, GtkFileSelection *fsel)
{
    char *fname;
    g_return_if_fail(GTK_IS_FILE_SELECTION(fsel));

    fname = gtk_file_selection_get_filename(fsel);
    if (g_file_exists(fname)) {
	GtkWidget *dlg = gnome_message_box_new(_("File exists. Overwrite?"),
					       GNOME_MESSAGE_BOX_WARNING,
					       GNOME_STOCK_BUTTON_OK,
					       GNOME_STOCK_BUTTON_NO,
					       NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(app));

	if (gnome_dialog_run(GNOME_DIALOG(dlg)) != 0)
	    return;
    }

    if (!fname || !write_xml_file(fname)) {
	GtkWidget *dlg;

	dlg = gnome_error_dialog_parented(_("Cannot save file"),
					  GTK_WINDOW(fsel));
	gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
    } else {
	gchar *title;
	if (filename)
	    title = g_strdup_printf("GAAE - %s", filename);
	else
	    title = g_strdup_printf("GAAE");
	gtk_window_set_title(GTK_WINDOW(app), title);
	g_free(title);
	gtk_widget_destroy(GTK_WIDGET(fsel));
    }
} /* save_ok */

static void save_as_cb(GtkWidget *widget, gpointer data)
{
    GtkFileSelection *fsel;

    fsel = GTK_FILE_SELECTION(gtk_file_selection_new(_("Save As")));

    if (filename)
	gtk_file_selection_set_filename(fsel, filename);

    gtk_signal_connect (GTK_OBJECT (fsel->ok_button),
			"clicked",
			GTK_SIGNAL_FUNC (save_ok),
			fsel);

    gtk_signal_connect_object (GTK_OBJECT (fsel->cancel_button),
			       "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
			       GTK_OBJECT(fsel));

    gtk_window_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
    gtk_window_set_transient_for (GTK_WINDOW(fsel), GTK_WINDOW(app));

    gtk_widget_show(GTK_WIDGET (fsel));
} /* save_as_cb */

static void save_cb(GtkWidget *widget, gpointer data)
{
    if (!filename) {
	save_as_cb(NULL, NULL);
    } else if (!write_xml_file(filename)) {
	gnome_app_error(GNOME_APP(app), _("Cannot save file"));
    }
} /* save_cb */

static void export_ok(GtkWidget *widget, GtkFileSelection *fsel)
{
    char *fname;
    g_return_if_fail(GTK_IS_FILE_SELECTION(fsel));

    fname = gtk_file_selection_get_filename(fsel);

    if (g_file_exists(fname)) {
	GtkWidget *dlg = gnome_message_box_new(_("File exists. Overwrite?"),
					       GNOME_MESSAGE_BOX_WARNING,
					       GNOME_STOCK_BUTTON_OK,
					       GNOME_STOCK_BUTTON_NO,
					       NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(app));

	if (gnome_dialog_run(GNOME_DIALOG(dlg)) != 0)
	    return;
    }

    if (!fname || !export(fname)) {
	GtkWidget *dlg;

	dlg = gnome_error_dialog_parented(_("Cannot export file"),
					  GTK_WINDOW(fsel));
	gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
    } else {
	gtk_widget_destroy(GTK_WIDGET(fsel));
    }
} /* export_ok */

static void export_pic_toggle(GtkWidget *widget, gpointer data)
{
    if (GTK_TOGGLE_BUTTON(widget)->active)
	gtk_widget_set_sensitive(GTK_WIDGET(data), FALSE);
    else
	gtk_widget_set_sensitive(GTK_WIDGET(data), TRUE);
} /* export_pic_toggle */

static void export_cb(GtkWidget *widget, gpointer data)
{
    GtkFileSelection *fsel;
    GtkWidget *dialog;
    GtkWidget *winfo, *wcomment, *wsize1, *wsize2, *wsize3, *wpic;
    GtkWidget *hbox, *vbox, *frame;
    int result;

    dialog = gnome_dialog_new(_("Export options"),
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

    hbox = gtk_hbox_new(FALSE, FALSE);
    vbox = gtk_vbox_new(FALSE, 10);
    frame = gtk_frame_new(_("Font size"));

    winfo = gtk_check_button_new_with_label(_("With aditional info"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(winfo),
				 get_options_info());
    gtk_box_pack_start(GTK_BOX(vbox), winfo, FALSE, FALSE, 0);

    wcomment = gtk_check_button_new_with_label(_("With commentary"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wcomment),
				 get_options_comment());
    gtk_box_pack_start(GTK_BOX(vbox), wcomment, FALSE, FALSE, 0);

    if (get_current() == FINITE) {
	wpic = gtk_check_button_new_with_label(_("Metapost picture"));
	gtk_signal_connect(GTK_OBJECT(wpic), "toggled",
			   GTK_SIGNAL_FUNC(export_pic_toggle), frame);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpic),
				     get_options_picture());
	gtk_box_pack_start(GTK_BOX(vbox), wpic, FALSE, FALSE, 0);
    }
    gtk_container_add(GTK_CONTAINER(hbox), vbox);

    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(frame), vbox);

    wsize1 = gtk_radio_button_new_with_label(NULL, _("10pt"));
    gtk_box_pack_start(GTK_BOX(vbox), wsize1, FALSE, FALSE, 0);

    wsize2 = gtk_radio_button_new_with_label(
	gtk_radio_button_group(GTK_RADIO_BUTTON(wsize1)), _("11pt"));
    gtk_box_pack_start(GTK_BOX(vbox), wsize2, FALSE, FALSE, 0);

    wsize3 = gtk_radio_button_new_with_label(
	gtk_radio_button_group(GTK_RADIO_BUTTON(wsize2)), _("12pt"));
    gtk_box_pack_start(GTK_BOX(vbox), wsize3, FALSE, FALSE, 0);

    if (get_options_size() == 10)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wsize1), TRUE);
    else if (get_options_size() == 11)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wsize2), TRUE);
    else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wsize3), TRUE);

    gtk_container_add(GTK_CONTAINER(hbox), frame);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), hbox,
		       FALSE, FALSE, 0);
    gtk_widget_show_all(hbox);

    result = gnome_dialog_run(GNOME_DIALOG(dialog));    
    if (result == 0) {
	set_options_info(
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(winfo)));
	set_options_comment(
	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wcomment)));
	if (get_current() == FINITE)
	    set_options_picture(
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wpic)));
	
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wsize1)))
	    set_options_size(10);
	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wsize2)))
	    set_options_size(11);
	else
	    set_options_size(12);
	gnome_dialog_close(GNOME_DIALOG(dialog));
    } else if (result > 0) {	
	gnome_dialog_close(GNOME_DIALOG(dialog));
	return;
    } else
	return;

    fsel = GTK_FILE_SELECTION(gtk_file_selection_new(_("Export As")));

    gtk_signal_connect (GTK_OBJECT (fsel->ok_button),
			"clicked",
			GTK_SIGNAL_FUNC (export_ok),
			fsel);

    gtk_signal_connect_object (GTK_OBJECT (fsel->cancel_button),
			       "clicked",
			       GTK_SIGNAL_FUNC (gtk_widget_destroy), 
			       GTK_OBJECT(fsel));

    gtk_window_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
    gtk_window_set_transient_for (GTK_WINDOW(fsel), GTK_WINDOW(app));

    gtk_widget_show(GTK_WIDGET (fsel));
} /* export_cb */

/*
 *  Font selection dialog
 */

static void font_ok(GtkWidget *widget, GtkFontSelectionDialog *dialog)
{
    char *fontname;
    g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(dialog));

    fontname = gtk_font_selection_dialog_get_font_name(
	GTK_FONT_SELECTION_DIALOG(dialog));

    if (!fontname)
	gnome_error_dialog_parented(_("Cannot load font"),
				    GTK_WINDOW(dialog));
    else {
	g_free(font);
	font = g_strdup(fontname);
	g_free(fontname);
	gtk_widget_destroy(GTK_WIDGET(dialog));
	show_all();
    }
} /* font_ok */

static void select_font_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog;

    dialog = gtk_font_selection_dialog_new(_("Font selection"));

    gtk_signal_connect(GTK_OBJECT(
	GTK_FONT_SELECTION_DIALOG(dialog)->ok_button), "clicked",
		       GTK_SIGNAL_FUNC(font_ok), dialog);

    gtk_widget_destroy(GTK_WIDGET(
	GTK_FONT_SELECTION_DIALOG(dialog)->apply_button));

    gtk_signal_connect_object(GTK_OBJECT(
	GTK_FONT_SELECTION_DIALOG(dialog)->cancel_button), "clicked",
			      GTK_SIGNAL_FUNC (gtk_widget_destroy), 
			      GTK_OBJECT(dialog));

    gtk_widget_show_all(dialog);
} /* select_font_cb */

/*
 *  Menus section
 */

void show_current_menu()
{
    gchar *title;
    if (filename)
	title = g_strdup_printf("GAAE - %s", filename);
    else
	title = g_strdup_printf("GAAE");
    gtk_window_set_title(GTK_WINDOW(app), title);
    g_free(title);

    gnome_app_remove_menu_range(GNOME_APP(app), "/", 2, 1);
    
    switch (get_current()) {
    case FINITE:
	gnome_app_insert_menus(GNOME_APP(app), GNOME_MENU_EDIT_STRING,
			       top_fa_menu);
	gnome_app_install_menu_hints(GNOME_APP(app), top_fa_menu);
	break;
    case STACK:
	gnome_app_insert_menus(GNOME_APP(app), GNOME_MENU_EDIT_STRING,
			       top_sa_menu);
	gnome_app_install_menu_hints(GNOME_APP(app), top_sa_menu);
	break;
    case REGULAR:
	gnome_app_insert_menus(GNOME_APP(app), GNOME_MENU_EDIT_STRING,
			       top_rg_menu);
	gnome_app_install_menu_hints(GNOME_APP(app), top_rg_menu);
	break;
    case CONTEXTFREE:
	gnome_app_insert_menus(GNOME_APP(app), GNOME_MENU_EDIT_STRING,
			       top_cg_menu);
	gnome_app_install_menu_hints(GNOME_APP(app), top_cg_menu);
	break;
    case GRAMMAR:
	gnome_app_insert_menus(GNOME_APP(app), GNOME_MENU_EDIT_STRING,
			       top_g_menu);
	gnome_app_install_menu_hints(GNOME_APP(app), top_g_menu);
	break;
    case NOTHING:
	break;
    }
} /* make_menu */


static GnomeUIInfo new_menu[] = {
    GNOMEUIINFO_MENU_NEW_ITEM("New _finite automata",
			      "Make a new, empty finite automata",
			      new_cb, GINT_TO_POINTER(FINITE)),
    GNOMEUIINFO_MENU_NEW_ITEM("New _stack automata",
			      "Make a new, empty stack automata",
			      new_cb, GINT_TO_POINTER(STACK)),
    GNOMEUIINFO_MENU_NEW_ITEM("New _regular grammar",
			      "Make a new, empty regular grammar",
			      new_cb, GINT_TO_POINTER(REGULAR)),
    GNOMEUIINFO_MENU_NEW_ITEM("New _contextfree grammar",
			      "Make a new, empty contextfree grammar",
			      new_cb, GINT_TO_POINTER(CONTEXTFREE)),
    GNOMEUIINFO_MENU_NEW_ITEM("New _grammar",
			      "Make a new, empty, unspecified grammar",
			      new_cb, GINT_TO_POINTER(GRAMMAR)),
    GNOMEUIINFO_END
};

static GnomeUIInfo file_menu[] = {
    GNOMEUIINFO_MENU_NEW_SUBTREE(new_menu),
    GNOMEUIINFO_MENU_OPEN_ITEM(open_cb, NULL),
    GNOMEUIINFO_MENU_SAVE_ITEM(save_cb, NULL),
    GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_as_cb, NULL),
    GNOMEUIINFO_ITEM_NONE(N_("_Export..."), N_("Export to LaTeX"), export_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_EXIT_ITEM(gtk_main_quit, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] = {
    GNOMEUIINFO_MENU_UNDO_ITEM(undo_cb, NULL),
    GNOMEUIINFO_MENU_REDO_ITEM(redo_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_PREFERENCES_ITEM(select_font_cb, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
    GNOMEUIINFO_HELP("GAAE"),
    GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
    GNOMEUIINFO_ITEM_NONE(N_("No document"), NULL, nothing_cb),

    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
};


void create_main_window(void)
{
    GtkWidget *bar;

    app = gnome_app_new("GAAE", "GAAE frontend");

    gnome_app_create_menus(GNOME_APP(app), main_menu);
    
    bar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
    gnome_app_set_statusbar(GNOME_APP(app), bar);
    gnome_app_install_menu_hints(GNOME_APP(app), main_menu);

    gtk_window_set_default_size(GTK_WINDOW(app), 600, 400);
    
    gtk_signal_connect (GTK_OBJECT (app), "delete_event",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);

    set_current(NOTHING);
    setup_options();
    setup_comment();

    main_vbox = gtk_vbox_new (FALSE, 10);
    gnome_app_set_contents(GNOME_APP(app), main_vbox);

    gtk_widget_show_all (app);
} /* create_main_window */
