/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Finite Automata menu
 */

static GnomeUIInfo fa_new_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Add _state"),
			  N_("Add new state"),
			  add_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _entry symbol"),
			  N_("Add new entry symbol"),
			  add_entry_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _transition"),
			  N_("Add new transition"),
			  add_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_edit_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Edit _state"),
			  N_("Edit existing state"),
			  edit_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Edit _entry symbol"),
			  N_("Edit existing entry symbol"),
			  edit_entry_symbol_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_remove_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Remove _state"),
			  N_("Remove existing state"),
			  remove_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _entry symbol"),
			  N_("Remove existing entry symbol"),
			  remove_entry_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _transition"),
			  N_("Remove existing transition"),
			  remove_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_set_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Set _starting state"),
			  N_("Set starting state"),
			  set_start_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Set _ending state"),
			  N_("Set ending state"),
			  set_end_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Reset en_ding state"),
			  N_("Reset ending state"),
			  reset_end_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Set _configuration"),
			  N_("Set configuration of automata"),
			  set_configuration_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_test_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Check sentence"),
			  N_("Check if sentence is recognized by automata"),
			  check_sentence_cb),
    GNOMEUIINFO_ITEM_NONE(N_("_Is deterministic?"),
			  N_("Check if automata is deterministic"),
			  is_deterministic_cb),
    GNOMEUIINFO_ITEM_NONE(N_("_Get inaccessible states"),
			  N_("Get inaccessible states"),
			  get_inaccessible_states_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("_Do transition"),
			  N_("Do transition from current state"),
			  do_finite_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_convert_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("To _deterministic"),
			  N_("Convert automata to deterninistic one"),
			  to_deterministic_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To _reduced"),
			  N_("Convert automata to reduced one"),
			  to_reduced_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("To regular _grammar"),
			  N_("Convert automata to equivalent regular grammar"),
			  convert_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo fa_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Add"), fa_new_menu),
    GNOMEUIINFO_SUBTREE(N_("_Edit"), fa_edit_menu),
    GNOMEUIINFO_SUBTREE(N_("_Remove"), fa_remove_menu),
    GNOMEUIINFO_SUBTREE(N_("_Set"), fa_set_menu),
    GNOMEUIINFO_SUBTREE(N_("_Test"), fa_test_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_SUBTREE(N_("_Convert"), fa_convert_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("Comment..."),
			  N_("Comment your masterpiece"),
			  comment_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo top_fa_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("F_inite automata"), fa_menu),
    GNOMEUIINFO_END
};

/*
 *  Stack Automata menu
 */

static GnomeUIInfo sa_new_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Add _state"),
			  N_("Add new state"),
			  add_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _entry symbol"),
			  N_("Add new entry symbol"),
			  add_entry_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add stac_k symbol"),
			  N_("Add new stack symbol"),
			  add_stack_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _transition"),
			  N_("Add new transition"),
			  add_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_edit_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Edit _state"),
			  N_("Edit existing state"),
			  edit_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Edit _entry symbol"),
			  N_("Edit existing entry symbol"),
			  edit_entry_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Edit s_tack symbol"),
			  N_("Edit existing stack symbol"),
			  edit_stack_symbol_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_remove_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Remove _state"),
			  N_("Remove existing state"),
			  remove_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _entry symbol"),
			  N_("Remove existing entry symbol"),
			  remove_entry_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("_Remove stack symbol"),
			  N_("Remove existing stack symbol"),
			  remove_stack_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _transition"),
			  N_("Remove existing transition"),
			  remove_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_set_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Set _starting state"),
			  N_("Set starting state"),
			  set_start_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Set _ending state"),
			  N_("Set ending state"),	
			  set_end_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("_Reset ending state"),
			  N_("Reset endinging state"),
			  reset_end_state_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Set s_tarting stack symbol"),
			  N_("Set starting stack symbol"),
			  set_start_stack_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Set con_figuration"),
			  N_("Set configuration of automata"),
			  set_configuration_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_test_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Check sentence"),
			  N_("Check if sentence is recognized by automata"),
			  check_sentence_cb),
    GNOMEUIINFO_ITEM_NONE(N_("_Is deterministic?"),
			  N_("Check if automata is deterministic"),
			  is_deterministic_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is _top stack working?"),
			  N_("Check if automata is working with top of the stack"),
			  is_top_stack_working_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is _empty stack recognizing?"),
			  N_("Check if automata recognizes by empty stack"),
			  is_empty_stack_recognising_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("_Do transition"),
			  N_("Do transition from current state"),
			  do_stack_transition_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_convert_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("To _top stack working"),
			  N_("Convert automata to work with top of the stack"),
			  to_top_stack_working_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To _empty stack recognizing"),
			  N_("Convert automata to recognize by empty stack"),
			  to_empty_stack_recognising_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To end _states recognizing"),
			  N_("Convert automata to recognize by end states"),
			  to_end_states_recognising_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("To contextfree _grammar"),
			  N_("Convert automata to equivalent contextfree grammar"),
			  convert_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo sa_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Add"), sa_new_menu),
    GNOMEUIINFO_SUBTREE(N_("_Edit"), sa_edit_menu),
    GNOMEUIINFO_SUBTREE(N_("_Remove"), sa_remove_menu),
    GNOMEUIINFO_SUBTREE(N_("_Set"), sa_set_menu),
    GNOMEUIINFO_SUBTREE(N_("_Test"), sa_test_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_SUBTREE(N_("_Convert"), sa_convert_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("Comment..."),
			  N_("Comment your masterpiece"),
			  comment_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo top_sa_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Stack automata"), sa_menu),
    GNOMEUIINFO_END
};

/*
 *  Common Grammar menus
 */

static GnomeUIInfo grammar_add_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Add _terminal symbol"),
			  N_("Add new terminal symbol"),
			  add_terminal_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _nonterminal symbol"),
			  N_("Add new nonterminal symbol"),
			  add_nonterminal_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Add _rule"),
			  N_("Add new rule"),
			  add_rule_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo grammar_edit_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Edit _terminal symbol"),
			  N_("Edit existing terminal symbol"),
			  edit_terminal_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Edit _nonterminal symbol"),
			  N_("Edit existing nonterminal symbol"),
			  edit_nonterminal_symbol_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo grammar_remove_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("Remove _terminal symbol"),
			  N_("Remove existing terminal symbol"),
			  remove_terminal_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _nonterminal symbol"),
			  N_("Remove existing nonterminal symbol"),
			  remove_nonterminal_symbol_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _rule"),
			  N_("Remove existing rule"),
			  remove_rule_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo grammar_set_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Set starting symbol"),
			  N_("Set starting (nonterminal) symbol"),
			  set_start_symbol_cb),
    GNOMEUIINFO_END
};

/*
 *  Grammar menu
 */

static GnomeUIInfo g_test_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Get clasification"),
			  N_("Get clasification of this grammar"),
			  get_classification_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo g_convert_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("To _extended"),
			  N_("Convert grammar to its extended form"),
			  to_extended_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo g_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Add"), grammar_add_menu),
    GNOMEUIINFO_SUBTREE(N_("_Edit"), grammar_edit_menu),
    GNOMEUIINFO_SUBTREE(N_("_Remove"), grammar_remove_menu),
    GNOMEUIINFO_SUBTREE(N_("_Set"), grammar_set_menu),
    GNOMEUIINFO_SUBTREE(N_("_Test"), g_test_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_SUBTREE(N_("_Convert"), g_convert_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("Comment..."),
			  N_("Comment your masterpiece"),
			  comment_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo top_g_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Grammar"), g_menu),
    GNOMEUIINFO_END
};

/*
 *  Regular Grammar menu
 */

static GnomeUIInfo rg_test_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Get clasification"),
			  N_("Get clasification of this grammar"),
			  get_classification_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo rg_convert_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("To _extended"),
			  N_("Convert grammar to its extended form"),
			  to_extended_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("To _finite automata"),
			  N_("Convert grammar to equivalent finite automata"),
			  convert_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo rg_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Add"), grammar_add_menu),
    GNOMEUIINFO_SUBTREE(N_("_Edit"), grammar_edit_menu),
    GNOMEUIINFO_SUBTREE(N_("_Remove"), grammar_remove_menu),
    GNOMEUIINFO_SUBTREE(N_("_Set"), grammar_set_menu),
    GNOMEUIINFO_SUBTREE(N_("_Test"), rg_test_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_SUBTREE(N_("_Convert"), rg_convert_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("Comment..."),
			  N_("Comment your masterpiece"),
			  comment_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo top_rg_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Regular grammar"), rg_menu),
    GNOMEUIINFO_END
};

/*
 *  Contextfree Grammar menu
 */

static GnomeUIInfo cg_test_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("_Get clasification"),
			  N_("Get clasification of this grammar"),
			  get_classification_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Get _redundant symbols"),
			  N_("Get redundant symbols in grammar"),
			  get_redundant_symbols_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Get rec_ursive symbols"),
			  N_("Get recursive symbols in grammar"),
			  get_recursive_symbols_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Get _inaccessible symbols"),
			  N_("Get inaccessible symbols in grammar"),
			  get_inaccessible_symbols_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Get _simple rules"),
			  N_("Get simple rules"),
			  get_simple_rules_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Get _E rules"),
			  N_("Get rules containing empty symbol"),
			  get_e_rules_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is _free of E rules?"),
			  N_("Is grammar free of rules containing empty symbol?"),
			  is_e_rules_free_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is se_lf?"),
			  N_("Is grammar self?"),
			  is_self_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is in Chomsk_y normal form?"),
			  N_("Is grammar in Chomsky normal form?"),
			  is_in_chomsky_nf_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is in Greibac_h normal form?"),
			  N_("Is grammar in Greibach normal form?"),
			  is_in_greibach_nf_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Is re_cursive?"),
			  N_("Is grammar recursive?"),
			  is_recursive_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo cg_convert_menu[] = {
    GNOMEUIINFO_ITEM_NONE(N_("To _extended"),
			  N_("Convert grammar to its extended form"),
			  to_extended_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To sel_f"),
			  N_("Convert grammar to <something>"),
			  to_self_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To _Chomsky normal form"),
			  N_("Convert grammar to Chomsky normal form"),
			  to_chomsky_nf_cb),
    GNOMEUIINFO_ITEM_NONE(N_("To _Greibach normal form"),
			  N_("Convert grammar to Greibach normal form"),
			  to_greibach_nf_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove _redundant symbols"),
			  N_("Remove redundant symbols"),
			  remove_redundant_symbols_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Remove E r_ules"),
			  N_("Remove rules containing enpty symbol"),
			  remove_e_rules_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Re_move simple rules"),
			  N_("Remove all simple rules"),
			  remove_simple_rules_cb),
    GNOMEUIINFO_ITEM_NONE(N_("Eliminate left recurs_ion"),
			  N_("Convert grammar to equivalent one without left recursion"),
			  eliminate_left_recursion_cb),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("To _stack automata"),
			  N_("Convert grammar to equivalent stack automata"),
			  convert_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo cg_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Add"), grammar_add_menu),
    GNOMEUIINFO_SUBTREE(N_("_Edit"), grammar_edit_menu),
    GNOMEUIINFO_SUBTREE(N_("_Remove"), grammar_remove_menu),
    GNOMEUIINFO_SUBTREE(N_("_Set"), grammar_set_menu),
    GNOMEUIINFO_SUBTREE(N_("_Test"), cg_test_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_SUBTREE(N_("_Convert"), cg_convert_menu),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("Comment..."),
			  N_("Comment your masterpiece"),
			  comment_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo top_cg_menu[] = {
    GNOMEUIINFO_SUBTREE(N_("_Contextfree grammar"), cg_menu),
    GNOMEUIINFO_END
};

