/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <stdio.h>
#include "data.h"
#include "functions.h"

static void explore(gchar *sentence, gchar *upper, gchar *extra)
{
    gchar **words;

    words = g_strsplit(sentence, " ", 0); /*WARN: name with space won't work*/
    
    if (strcmp(words[0], "state") == 0) {
	if (upper == NULL)
	    add_state_to_list(atoi(words[1]));
	else if (strcmp(upper, "states") == 0)
	    if (extra == NULL)
		add_state(atoi(words[1]), words[2], words[3]);
	    else
		add_state_to_list(atoi(words[1]));
	else if (strcmp(upper, "transition") == 0)
	    add_state_to_transition(atoi(words[1]));
	else if (strcmp(upper, "") == 0)
	    add_state_to_list(atoi(words[1]));

    } else if (strcmp(words[0], "entry_symbol") == 0) {
	if (strcmp(upper, "entry_symbols") == 0)
	    if (extra == NULL)
		add_symbol(atoi(words[1]), words[2], NULL, words[0]);
	    else
		add_symbol_to_list(atoi(words[1]), "sentence");
	else if (strcmp(upper, "transition") == 0)
	    add_entry_symbol_to_transition(atoi(words[1]));

    } else if (strcmp(words[0], "stack_symbol") == 0) {
	if (strcmp(upper, "stack_symbols") == 0)
	    add_symbol(atoi(words[1]), words[2], words[3], words[0]);
	else if (strcmp(upper, "from_stack") == 0)
	    add_stack_symbol_to_transition(atoi(words[1]), upper);
	else if (strcmp(upper, "to_stack") == 0)
	    add_stack_symbol_to_transition(atoi(words[1]), upper);
	else if (strcmp(upper, "stack") == 0)
	    add_to_stack(atoi(words[1]));

    } else if (strcmp(words[0], "nonterminal_symbol") == 0) {
	if (strcmp(upper, "nonterminal_symbols") == 0)
	    if (extra == NULL)
		add_symbol(atoi(words[1]), words[2], words[3], words[0]);
	    else
		add_symbol_to_list(atoi(words[1]), words[0]);
	else if (strcmp(upper, "left") == 0)
	    add_symbol_to_rule(atoi(words[1]), words[0], upper);
	else if (strcmp(upper, "right") == 0)
	    add_symbol_to_rule(atoi(words[1]), words[0], upper);
	else if (strcmp(upper, "symbols") == 0)
	    add_symbol_to_list(atoi(words[1]), words[0]);

    } else if (strcmp(words[0], "terminal_symbol") == 0) {
	if (strcmp(upper, "terminal_symbols") == 0)
	    add_symbol(atoi(words[1]), words[2], NULL, words[0]);
	else if (strcmp(upper, "left") == 0)
	    add_symbol_to_rule(atoi(words[1]), words[0], upper);
	else if (strcmp(upper, "right") == 0)
	    add_symbol_to_rule(atoi(words[1]), words[0], upper);
	else if (strcmp(upper, "symbols") == 0)
	    add_symbol_to_list(atoi(words[1]), words[0]);
    }

    g_strfreev(words);
} /* explore */

void parse(gchar *description, gchar *upper, gchar *extra)
{
    int level = 0;
    int i = 0;
    GString *list;
    GString *section;

    list = g_string_sized_new(16);
    section = g_string_sized_new(16);

    for (i = 0; i < strlen(description); i++) {
	switch (description[i]) {
	case '(' :
	    level++;
	    if (level == 2)
		list = g_string_truncate(list, 0);
	    if (level > 1)
		list = g_string_append_c(list, description[i]);
	    break;
	case ')' :
	    level--;
	    if (level > 0)
 		list = g_string_append_c(list, description[i]);
	    if (level == 1) {
		/* ugly damn hack */
		if (strcmp(section->str, "finite_automata") == 0)
		    set_current(FINITE);
		else if (strcmp(section->str, "stack_automata") == 0)
		    set_current(STACK);
                /* FIX: ask mico || make all possibilities.*/
		else if (strcmp(section->str, "grammar") == 0) {
		    if (get_current() == REGULAR || get_current() == FINITE)
			set_current(REGULAR);
		    else if (get_current() == CONTEXTFREE
			     || get_current() == STACK)
			set_current(CONTEXTFREE);
		}
		/* ugly damn hack */		    
		if ((strcmp(section->str, "transitions") == 0)
		    || (strcmp(section->str, "rules") == 0)
		    || (strcmp(section->str, "stacks") == 0)
		    || (strcmp(section->str, "state-stack-pairs") ==  0)) {
		    setup_aux();
		    parse(list->str, section->str, extra);
		    add_aux_to_list(extra);
		} else
		    parse(list->str, section->str, extra);
		list = g_string_truncate(list, 0);
	    }
	    if (level == 0 && list->len > 0) {
		explore(list->str, upper, extra);
	    }
	    break;
	case '\007' :
	    if (level == 1) {
		if (description[i+1] == '(' || description[i+1] == ')')
		    list = g_string_append_c(list, description[++i]);
		else
		    list = g_string_append_c(list, description[i]);
	    } else {
		list = g_string_append_c(list, description[i]);
		list = g_string_append_c(list, description[++i]);
	    }
	    break;
	case ' ' :
	    list = g_string_append_c(list, description[i]);
	    break;
	default :
	  if (level == 1)
	    section = g_string_append_c(section, description[i]);
	  list = g_string_append_c(list, description[i]);
	  break;
	}
    }
    g_string_free(section, TRUE);
    g_string_free(list, TRUE);
} /* parse */

gchar * get_id(gchar *sentence)
{
    gchar **words;
    gchar * id;

    words = g_strsplit(sentence, " ", 0);
    id = g_strdup(words[1]);

    g_strfreev(words);
    return id;
} /* get_id */
