/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Miroslav Kure <kurem@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include "../client-interface.h"
#include "data.h"
#include "functions.h"
#include "parse.h"
#include "gui.h"
#include "graphics.h"
#include "table.h"

GtkWidget *my_clist;
GtkWidget *main_vbox;

static GnomeUIInfo stack_popup[] = {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new state..."),
			      N_("Add new state to automata"),
			      add_state_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new entry symbol..."),
			      N_("Add new entry symbol to automata"),
			      add_entry_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new stack symbol..."),
			      N_("Add new stack symbol to automata"),
			      add_stack_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new transition..."),
			      N_("Add new transition to automata"),
			      add_transition_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("_Delete selected transitions"),
			  N_("Delete all selected transitions"),
			  direct_delete_stack_trans_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo stack_mini_popup[] = {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new state..."),
			      N_("Add new state to automata"),
			      add_state_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new entry symbol..."),
			      N_("Add new entry symbol to automata"),
			      add_entry_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new stack symbol..."),
			      N_("Add new stack symbol to automata"),
			      add_stack_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new transition..."),
			      N_("Add new transition to automata"),
			      add_transition_cb, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo grammar_popup[] = {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new nonterminal symbol..."),
			      N_("Add new nonterminal symbol to grammar"),
			      add_nonterminal_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new terminal symbol..."),
			      N_("Add new terminal symbol to grammar"),
			      add_terminal_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new rule..."),
			      N_("Add new rule to grammar"),
			      add_rule_cb, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_ITEM_NONE(N_("_Delete selected rules"),
			  N_("Delete all selected rules"),
			  direct_delete_rule_cb),
    GNOMEUIINFO_END
};

static GnomeUIInfo grammar_mini_popup[] = {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new nonterminal symbol..."),
			      N_("Add new nonterminal symbol to grammar"),
			      add_nonterminal_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new terminal symbol..."),
			      N_("Add new terminal symbol to grammar"),
			      add_terminal_symbol_cb, NULL),
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add new rule..."),
			      N_("Add new rule to grammar"),
			      add_rule_cb, NULL),
    GNOMEUIINFO_END
};

static GtkWidget *make_hbox_with_2labels(char *label, char *def)
{
    GtkWidget *hbox, *text;
    hbox = gtk_hbox_new(FALSE, FALSE);

    text = gtk_label_new(label);
    gtk_widget_set_usize(text, 100, 10);
    gtk_box_pack_start(GTK_BOX(hbox), text, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(text), GTK_JUSTIFY_RIGHT);

    text = gtk_label_new(def);
    gtk_box_pack_start(GTK_BOX(hbox), text, FALSE, FALSE, 0);
    gtk_widget_show_all(hbox);
    return hbox;
} /* make_hbox_with_2labels */

static GtkWidget *make_scroll_window(GtkWidget *box)
{
    GtkWidget *swindow = gtk_scrolled_window_new(NULL, NULL);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow),
				   GTK_POLICY_AUTOMATIC,
				   GTK_POLICY_NEVER);

    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swindow), box);
    gtk_widget_show_all(swindow);
    return swindow;
} /* make_scroll_window */

static gint select_row_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    if (event->type == GDK_BUTTON_PRESS && event->button.button == 3) {
	if (get_current() == STACK) {
	    GList *selection = GTK_CLIST(my_clist)->selection;
	    if (selection != NULL)
		gnome_popup_menu_do_popup(gnome_popup_menu_new(stack_popup),
					  NULL, NULL, NULL, selection);
	    else
		gnome_popup_menu_do_popup(gnome_popup_menu_new(
		    stack_mini_popup), NULL, NULL, NULL, NULL);
	} else if (get_current() == REGULAR
		   || get_current() == CONTEXTFREE
		   || get_current() == GRAMMAR) {
	    GList *selection = GTK_CLIST(my_clist)->selection;
	    if (selection != NULL)
		gnome_popup_menu_do_popup(gnome_popup_menu_new(grammar_popup),
					  NULL, NULL, NULL, selection);
	    else
		gnome_popup_menu_do_popup(gnome_popup_menu_new(
		    grammar_mini_popup), NULL, NULL, NULL, NULL);
	}
	return TRUE;
    }
    return FALSE;
} /* select_row_cb */

static void make_grammar_table(GtkWidget *vbox)
{
    GtkWidget *interbox = gtk_vbox_new(FALSE, 0);
    gchar *text;

    text = get_gram_description(FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("G = ", text),
                       FALSE, FALSE, 10);
    g_free(text);

    text = get_string_of_symbols(gram.nonterminal_symbols, TRUE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("N = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_symbols(gram.terminal_symbols, TRUE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("T = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    gtk_box_pack_start(GTK_BOX(vbox), make_scroll_window(interbox),
                       FALSE, FALSE, 0);
} /* make_grammar_table */

static void add_rule_to_clist(Rule *rule)
{
    char *texts[3];
    int row_number;

    g_return_if_fail(rule != NULL);
    
    texts[0] = get_string_of_symbols(rule->left, FALSE);
    texts[1] = "";
    texts[2] = get_string_of_symbols(rule->right, FALSE);
    
    row_number = gtk_clist_append(GTK_CLIST(my_clist), texts);
    gtk_clist_set_row_data(GTK_CLIST(my_clist), row_number, rule);
} /* add_rule_to_clist */

static void make_rules_table(void)
{
    GSList *l;
    for (l = gram.rules; l != NULL; l = l->next) {
	Rule *rule = l->data;
	add_rule_to_clist(rule);
    }
} /* make_rules_table */

void show_grammar_frame(void)
{
    GtkWidget *frame;
    GtkWidget *swindow;

    gchar *clist_titles[] = { N_("Left"), N_("->"), N_("Right") };

    gtk_widget_destroy(main_vbox);

    my_clist = gtk_clist_new_with_titles (3, clist_titles);

    gtk_clist_set_column_width(GTK_CLIST(my_clist), 0, 250);
    gtk_clist_set_column_width(GTK_CLIST(my_clist), 2, 250);
    gtk_clist_set_column_justification(GTK_CLIST(my_clist), 0,
				       GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification(GTK_CLIST(my_clist), 1,
				       GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification(GTK_CLIST(my_clist), 2,
				       GTK_JUSTIFY_CENTER);

    gtk_clist_set_sort_type(GTK_CLIST(my_clist), GTK_SORT_ASCENDING);
    gtk_clist_set_sort_column(GTK_CLIST(my_clist), 0);
    gtk_clist_column_titles_passive(GTK_CLIST (my_clist));
    gtk_clist_set_selection_mode(GTK_CLIST(my_clist), GTK_SELECTION_MULTIPLE);

    gtk_signal_connect(GTK_OBJECT(my_clist), "button_press_event",
		       GTK_SIGNAL_FUNC(select_row_cb), NULL);

    swindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_container_border_width(GTK_CONTAINER (swindow), 5);
    gtk_container_add(GTK_CONTAINER(swindow), my_clist);

    frame = gtk_frame_new(N_("P: (rules)"));
    gtk_container_border_width(GTK_CONTAINER (frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), swindow);

    main_vbox = gtk_vbox_new (FALSE, 0);
    make_grammar_table(main_vbox);
    make_rules_table();
    gtk_box_pack_start(GTK_BOX(main_vbox), frame, TRUE, TRUE, 0);

    gnome_app_set_contents(GNOME_APP(app), main_vbox);
    gtk_widget_show_all (app);
} /* show grammar_frame */


void make_finite_automata_table(GtkWidget *vbox)
{
    GtkWidget *interbox = gtk_vbox_new(FALSE, 0);
    gchar *text;

    text = get_fin_aut_description(FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("A = ", text),
                       FALSE, FALSE, 10);
    g_free(text);

    text = get_string_of_symbols(aut.entry_symbols, TRUE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("T = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_states();    
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("Q = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_ending_states();
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("F = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_symbols(symbols, FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("sentence = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    gtk_box_pack_start(GTK_BOX(vbox), make_scroll_window(interbox),
                       FALSE, FALSE, 0);
} /* make_finite_automata_table */

static void make_stack_automata_table(GtkWidget *vbox)
{
    GtkWidget *interbox = gtk_vbox_new(FALSE, 0);
    gchar *text;

    text = get_stack_aut_description(FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("A = ", text),
                       FALSE, FALSE, 10);
    g_free(text);

    text = get_string_of_symbols(aut.entry_symbols, TRUE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("T = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_states();    
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("Q = ", text),
                       FALSE, FALSE, 0);
    g_free(text);


    text = get_string_of_symbols(aut.stack_symbols, TRUE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("Z = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_ending_states();
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("F = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_symbols(symbols, FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("sentence = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    text = get_string_of_symbols(stack, FALSE);
    gtk_box_pack_start(GTK_BOX(interbox),
                       make_hbox_with_2labels("stack = ", text),
                       FALSE, FALSE, 0);
    g_free(text);

    gtk_box_pack_start(GTK_BOX(vbox), make_scroll_window(interbox),
                       FALSE, FALSE, 0);
} /* make_stack_automata_table */

static void add_transition_to_clist(Transition *trans, gboolean colorize)
{
    char *texts[5];
    int row_number;

    g_return_if_fail(trans != NULL);
    
    texts[0] = trans->from->name;
    texts[1] = get_string_of_symbols(trans->entry_symbols, FALSE);
    texts[2] = trans->to->name;
    texts[3] = get_string_of_symbols(trans->from_stack, FALSE);
    texts[4] = get_string_of_symbols(trans->to_stack, FALSE);
    
    row_number = gtk_clist_append(GTK_CLIST(my_clist), texts);
    gtk_clist_set_row_data(GTK_CLIST(my_clist), row_number, trans);
    if (colorize) {
	GdkColor color;
	color.red = 62000; color.green = 41000; color.blue = 41000;
	gdk_colormap_alloc_color(gdk_colormap_get_system(), &color,
				 FALSE, TRUE);
	gtk_clist_set_background(GTK_CLIST(my_clist), row_number, &color);
    }
} /* add_transition_to_clist */

static void make_transitions_table(void)
{
    GSList *l;
    for (l = aut.transitions; l != NULL; l = l->next) {
	Transition *trans = l->data;

	if (states != NULL) {
	    State *state = states->data;
	    if (trans->from == state) {
		add_transition_to_clist(trans, TRUE);
		continue;
	    }
	}
	add_transition_to_clist(trans, FALSE);
    }
} /* make_transitions_table */

void show_stack_automata_frame(void)
{
    GtkWidget *frame;
    GtkWidget *swindow;
    int i;
    gchar *clist_titles[] = { N_("From State"), N_("Entry Symbol"),
			      N_("To State"),
			      N_("From Stack"), N_("To Stack") };

    gtk_widget_destroy(main_vbox);

    my_clist = gtk_clist_new_with_titles (5, clist_titles);

    gtk_clist_set_sort_type(GTK_CLIST(my_clist), GTK_SORT_ASCENDING);
    gtk_clist_set_sort_column(GTK_CLIST(my_clist), 0);
    gtk_clist_column_titles_passive(GTK_CLIST (my_clist));
    gtk_clist_set_selection_mode(GTK_CLIST(my_clist), GTK_SELECTION_MULTIPLE);

    gtk_signal_connect(GTK_OBJECT(my_clist), "button_press_event",
		       GTK_SIGNAL_FUNC(select_row_cb), NULL);

    swindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
    gtk_container_border_width(GTK_CONTAINER (swindow), 5);
    gtk_container_add(GTK_CONTAINER(swindow), my_clist);

    frame = gtk_frame_new(N_("d: (transitions)"));
    gtk_container_border_width(GTK_CONTAINER (frame), 10);
    gtk_container_add(GTK_CONTAINER(frame), swindow);

    main_vbox = gtk_vbox_new (FALSE, 0);
    make_stack_automata_table(main_vbox);
    make_transitions_table();
    
    for (i = 0; i < 5; i++) {
	gtk_clist_set_column_min_width(GTK_CLIST(my_clist), i, 90);

	gtk_clist_set_column_width(GTK_CLIST(my_clist), i,
				   gtk_clist_optimal_column_width(
				       GTK_CLIST(my_clist), i));

	gtk_clist_set_column_justification(GTK_CLIST(my_clist), i,
					   GTK_JUSTIFY_CENTER);
    }

    gtk_box_pack_start(GTK_BOX(main_vbox), frame, TRUE, TRUE, 0);

    gnome_app_set_contents(GNOME_APP(app), main_vbox);
    gtk_widget_show_all (app);
} /* show_stack_automata_frame */


void show_all()
{
    switch (get_current()) {
    case FINITE:
    {
	gchar *state, *sentence;
	clear_all_data();
	parse(finite_automata_get_look(fa), NULL, NULL);
	finite_automata_get_configuration(fa, &state, &sentence);
	parse(state, NULL, "y");
	parse(sentence, NULL, "y");
	show_finite_automata_frame();
	g_free(state); g_free(sentence);
	break;
    }
    case STACK:
    {
	gchar *state, *sentence, *stack;
	clear_all_data();
	parse(stack_automata_get_look(sa), NULL, NULL);
	stack_automata_get_configuration(sa, &state, &sentence, &stack);
	parse(state, NULL, "y");
	parse(sentence, NULL, "y");
	parse(stack, NULL, "y");
	show_stack_automata_frame();
	g_free(state); g_free(sentence); g_free(stack);
	break;
    }
    case REGULAR:
	clear_all_data();
	parse(regular_grammar_get_look(rg), NULL, NULL);
	show_grammar_frame();
	break;
    case CONTEXTFREE:
	clear_all_data();
	parse(contextfree_grammar_get_look(cg), NULL, NULL);
	show_grammar_frame();
	break;
    case GRAMMAR:
	clear_all_data();
	parse(grammar_get_look(g), NULL, NULL);
	show_grammar_frame();
	break;
    case NOTHING:
	return;
    }
} /* show_all */
