/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "interface.h"

extern Formal_mico_var formal_mico;

struct finite_automata_struc {
  FiniteAutomata_mico_var object;
};

struct regular_grammar_struc {
  RegularGrammar_mico_var object;
};

extern "C" char *
finite_automata_add_state(finite_automata_struc *automata, const char *name)
{
  return automata->object->addState(name);
}

extern "C" bool 
finite_automata_remove_state(finite_automata_struc *automata,
			     const char *state_id)
{
  return automata->object->removeState(state_id);
}

extern "C" char *
finite_automata_edit_state(finite_automata_struc *automata,
			   const char *state_id, const char *new_name)
{
  return automata->object->editState(state_id, new_name);
}

extern "C" char *
finite_automata_add_entry_symbol(finite_automata_struc *automata,
				 const char *name)
{
  return automata->object->addEntrySymbol(name);
}

extern "C" bool
finite_automata_remove_entry_symbol(finite_automata_struc *automata,
				    const char *symbol_id)
{
  return automata->object->removeEntrySymbol(symbol_id);
}

extern "C" char *
finite_automata_edit_entry_symbol(finite_automata_struc *automata,
				  const char *symbol_id, const char *new_name)
{
  return automata->object->editEntrySymbol(symbol_id, new_name);
}

extern "C" bool
finite_automata_set_start_state(finite_automata_struc *automata,
				const char *state_id)
{
  return automata->object->setStartState(state_id);
}

extern "C" bool
finite_automata_set_end_state(finite_automata_struc *automata,
			      const char *state_id)
{
  return automata->object->setEndState(state_id);
}

extern "C" bool
finite_automata_reset_end_state(finite_automata_struc *automata,
				const char *state_id)
{
  return automata->object->resetEndState(state_id);
}

extern "C" bool 
finite_automata_add_transition(finite_automata_struc *automata,
			       const char* from_state_id,
			       const char* symbol_id, const char* to_state_id)
{
  return 
    automata->object->addTransition(from_state_id, symbol_id, to_state_id);
}

extern "C" bool 
finite_automata_remove_transition(finite_automata_struc *automata,
				  const char* from_state_id,
				  const char* symbol_id, 
				  const char* to_state_id)
{
  return 
    automata->object->removeTransition(from_state_id, symbol_id, to_state_id);
}

extern "C" bool
finite_automata_set_configuration(finite_automata_struc *automata,
				  const char* state_id, const char* sentence)
{
  return automata->object->setConfiguration(state_id, sentence);
}

extern "C" void 
finite_automata_get_configuration(finite_automata_struc *automata, 
				  char** state_id, char** sentence)
{
  automata->object->getConfiguration(*state_id, *sentence);
}

extern "C" char *
finite_automata_get_look(finite_automata_struc *automata)
{
  return automata->object->getLook();
}

extern "C" char *
finite_automata_get_transition_destinations(finite_automata_struc *automata)
{
  return automata->object->getTransitionDestinations();
}

extern "C" bool 
finite_automata_do_transition(finite_automata_struc *automata, 
			      const char* state_id)
{
  return automata->object->doTransition(state_id);
}

extern "C" bool 
finite_automata_check_sentence(finite_automata_struc *automata,
			       const char* sentence)
{
  return automata->object->checkSentence(sentence);
}

extern "C" bool 
finite_automata_is_deterministic(finite_automata_struc *automata)
{
  return automata->object->isDeterministic();
}

extern "C" void
finite_automata_to_deterministic(finite_automata_struc *automata)
{
  automata->object->toDeterministic();
}

extern "C" char *
finite_automata_get_inaccessible_states(finite_automata_struc *automata)
{
  return automata->object->getInaccessibleStates();
}

extern "C" void 
finite_automata_to_reduced(finite_automata_struc *automata)
{
  automata->object->toReduced();
}

extern "C" regular_grammar_struc *
finite_automata_to_regular_grammar(finite_automata_struc *automata)
{
  regular_grammar_struc *result = new regular_grammar_struc;
  result->object = automata->object->toRegularGrammar();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" void 
finite_automata_undo(finite_automata_struc *automata)
{
  automata->object->undo();
}

extern "C" void 
finite_automata_redo(finite_automata_struc *automata)
{
  automata->object->redo();
}

struct stack_automata_struc {
  StackAutomata_mico_var object;
};

struct contextfree_grammar_struc {
  ContextfreeGrammar_mico_var object;
};

extern "C" char *
stack_automata_add_state(stack_automata_struc *automata, const char *name)
{
  return automata->object->addState(name);
}

extern "C" bool
stack_automata_remove_state(stack_automata_struc *automata,
			    const char *state_id)
{
  return automata->object->removeState(state_id);
}

extern "C" char *
stack_automata_edit_state(stack_automata_struc *automata,
			  const char *state_id, const char *new_name)
{
  return automata->object->editState(state_id, new_name);
}

extern "C" char *
stack_automata_add_entry_symbol(stack_automata_struc *automata,
				const char *name)
{
  return automata->object->addEntrySymbol(name);
}

extern "C" bool
stack_automata_remove_entry_symbol(stack_automata_struc *automata, 
				   const char *symbol_id)
{
  return automata->object->removeEntrySymbol(symbol_id);
}

extern "C" char *
stack_automata_edit_entry_symbol(stack_automata_struc *automata,
				 const char *symbol_id, const char *new_name)
{
  return automata->object->editEntrySymbol(symbol_id, new_name);
}

extern "C" bool
stack_automata_set_start_state(stack_automata_struc *automata,
			       const char *state_id)
{
  return automata->object->setStartState(state_id);
}

extern "C" bool
stack_automata_set_end_state(stack_automata_struc *automata,
			     const char *state_id)
{
  return automata->object->setEndState(state_id);
}

extern "C" bool
stack_automata_reset_end_state(stack_automata_struc *automata,
			       const char *state_id)
{
  return automata->object->resetEndState(state_id);
}

extern "C" char *
stack_automata_add_stack_symbol(stack_automata_struc *automata,
				const char* name)
{
  return automata->object->addStackSymbol(name);
}

extern "C" bool
stack_automata_remove_stack_symbol(stack_automata_struc *automata, 
				   const char* symbol_id)
{
  return automata->object->removeStackSymbol(symbol_id);
}

extern "C" char *
stack_automata_edit_stack_symbol(stack_automata_struc *automata,
				 const char* symbol_id, const char* new_name)
{
  return automata->object->editStackSymbol(symbol_id, new_name);
}

extern "C" bool
stack_automata_set_start_stack_symbol(stack_automata_struc *automata,
				      const char* symbol_id)
{
  return automata->object->setStartStackSymbol(symbol_id);
}

extern "C" bool 
stack_automata_add_transition(stack_automata_struc *automata,
			      const char* from_state_id,
			      const char* symbol_id, const char* to_state_id,
			      const char* from_stack, const char* to_stack)
{
  return 
    automata->object->addTransition
    (from_state_id, symbol_id, to_state_id, from_stack, to_stack);
}

extern "C" bool
stack_automata_remove_transition(stack_automata_struc *automata,
				 const char* from_state_id,
				 const char* symbol_id,
				 const char* to_state_id,
				 const char* from_stack, const char* to_stack)
{
  return 
    automata->object->removeTransition
    (from_state_id, symbol_id, to_state_id, from_stack, to_stack);
}

extern "C" bool 
stack_automata_set_configuration(stack_automata_struc *automata,
				 const char* state_id, const char* sentence,
				 const char* stack)
{
  return 
    automata->object->setConfiguration(state_id, sentence, stack);
}

extern "C" void
stack_automata_get_configuration(stack_automata_struc *automata,
				 char** state_id, char** sentence,
				 char** stack)
{
  automata->object->getConfiguration(*state_id, *sentence, *stack);
}

extern "C" char *
stack_automata_get_look(stack_automata_struc *automata)
{
  return automata->object->getLook();
}

extern "C" char *
stack_automata_get_from_stacks(stack_automata_struc *automata)
{
  return automata->object->getFromStacks();
}

extern "C" char *
stack_automata_get_transition_destinations(stack_automata_struc *automata,
					   const char* from_stack)
{
  return automata->object->getTransitionDestinations(from_stack);
}

extern "C" bool
stack_automata_do_transition(stack_automata_struc *automata,
			     const char* state_id, const char* from_stack,
			     const char* to_stack)
{
  return automata->object->doTransition(state_id, from_stack, to_stack);
}

extern "C" bool 
stack_automata_check_sentence(stack_automata_struc *automata,
			      const char* sentence)
{
  return automata->object->checkSentence(sentence);
}

extern "C" bool 
stack_automata_is_deterministic(stack_automata_struc *automata)
{
  return automata->object->isDeterministic();
}

extern "C" bool
stack_automata_is_top_stack_working(stack_automata_struc *automata)
{
  return automata->object->isTopStackWorking();
}

extern "C" void 
stack_automata_to_top_stack_working(stack_automata_struc *automata)
{
  automata->object->toTopStackWorking();
}

extern "C" bool 
stack_automata_is_empty_stack_checking(stack_automata_struc *automata)
{
  return automata->object->isEmptyStackChecking();
}

extern "C" void
stack_automata_to_empty_stack_checking(stack_automata_struc *automata)
{
  automata->object->toEmptyStackChecking();
}

extern "C" void
stack_automata_to_end_states_checking(stack_automata_struc *automata)
{
  automata->object->toEndStatesChecking();
}

extern "C" contextfree_grammar_struc *
stack_automata_to_contextfree_grammar(stack_automata_struc *automata)
{
  contextfree_grammar_struc *result = new contextfree_grammar_struc;
  result->object = automata->object->toContextfreeGrammar();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" void 
stack_automata_undo(stack_automata_struc *automata)
{
  automata->object->undo();
}

extern "C" void 
stack_automata_redo(stack_automata_struc *automata)
{
  automata->object->redo();
}

struct grammar_struc {
  Grammar_mico_var object;
};

extern "C" char* 
grammar_add_nonterminal_symbol(grammar_struc *grammar, const char* name)
{
  return grammar->object->addNonterminalSymbol(name);
}

extern "C" bool
grammar_remove_nonterminal_symbol(grammar_struc *grammar,
				  const char* symbol_id)
{
  return grammar->object->removeNonterminalSymbol(symbol_id);
}

extern "C" char* 
grammar_edit_nonterminal_symbol(grammar_struc *grammar, const char* symbol_id,
				const char* new_name)
{
  return grammar->object->editNonterminalSymbol(symbol_id, new_name);
}

extern "C" char* 
grammar_add_terminal_symbol(grammar_struc *grammar, const char* name)
{
  return grammar->object->addTerminalSymbol(name);
}

extern "C" bool
grammar_remove_terminal_symbol(grammar_struc *grammar, const char* symbol_id)
{
  return grammar->object->removeTerminalSymbol(symbol_id);
}

extern "C" char*
grammar_edit_terminal_symbol(grammar_struc *grammar, const char* symbol_id,
			     const char* new_name)
{
  return grammar->object->editTerminalSymbol(symbol_id, new_name);
}

extern "C" bool
grammar_set_start_symbol(grammar_struc *grammar, const char* symbol_id)
{
  return grammar->object->setStartSymbol(symbol_id);
}

extern "C" bool
grammar_add_rule(grammar_struc *grammar, const char* left, const char* right)
{
  return grammar->object->addRule(left, right);
}

extern "C" bool
grammar_remove_rule(grammar_struc *grammar, const char* left,
		    const char* right)
{
  return grammar->object->removeRule(left, right);
}

extern "C" char* 
grammar_get_look(grammar_struc *grammar)
{
  return grammar->object->getLook();
}

extern "C" char*
grammar_get_classification(grammar_struc *grammar)
{
  return grammar->object->getClassification();
}

extern "C" void
grammar_to_extended(grammar_struc *grammar)
{
  grammar->object->toExtended();
}

extern "C" void 
grammar_undo(grammar_struc *grammar)
{
  grammar->object->undo();
}

extern "C" void 
grammar_redo(grammar_struc *grammar)
{
  grammar->object->redo();
}

extern "C" char* 
regular_grammar_add_nonterminal_symbol(regular_grammar_struc *grammar,
				       const char* name)
{
  return grammar->object->addNonterminalSymbol(name);
}

extern "C" bool
regular_grammar_remove_nonterminal_symbol(regular_grammar_struc *grammar,
					  const char* symbol_id)
{
  return grammar->object->removeNonterminalSymbol(symbol_id);
}

extern "C" char* 
regular_grammar_edit_nonterminal_symbol(regular_grammar_struc *grammar,
					const char* symbol_id,
					const char* new_name)
{
  return grammar->object->editNonterminalSymbol(symbol_id, new_name);
}

extern "C" char* 
regular_grammar_add_terminal_symbol(regular_grammar_struc *grammar,
				    const char* name)
{
  return grammar->object->addTerminalSymbol(name);
}

extern "C" bool
regular_grammar_remove_terminal_symbol(regular_grammar_struc *grammar, 
				       const char* symbol_id)
{
  return grammar->object->removeTerminalSymbol(symbol_id);
}

extern "C" char*
regular_grammar_edit_terminal_symbol(regular_grammar_struc *grammar,
				     const char* symbol_id,
				     const char* new_name)
{
  return grammar->object->editTerminalSymbol(symbol_id, new_name);
}

extern "C" bool
regular_grammar_set_start_symbol(regular_grammar_struc *grammar, 
				 const char* symbol_id)
{
  return grammar->object->setStartSymbol(symbol_id);
}

extern "C" bool
regular_grammar_add_rule(regular_grammar_struc *grammar, const char* left,
			 const char* right)
{
  return grammar->object->addRule(left, right);
}

extern "C" bool
regular_grammar_remove_rule(regular_grammar_struc *grammar, const char* left,
			    const char* right)
{
  return grammar->object->removeRule(left, right);
}

extern "C" char* 
regular_grammar_get_look(regular_grammar_struc *grammar)
{
  return grammar->object->getLook();
}

extern "C" char*
regular_grammar_get_classification(regular_grammar_struc *grammar)
{
  return grammar->object->getClassification();
}

extern "C" void
regular_grammar_to_extended(regular_grammar_struc *grammar)
{
  grammar->object->toExtended();
}

extern "C" finite_automata_struc *
regular_grammar_to_finite_automata(regular_grammar_struc *grammar)
{
  finite_automata_struc *result = new finite_automata_struc;
  result->object = grammar->object->toFiniteAutomata();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" void 
regular_grammar_undo(regular_grammar_struc *grammar)
{
  grammar->object->undo();
}

extern "C" void 
regular_grammar_redo(regular_grammar_struc *grammar)
{
  grammar->object->redo();
}

extern "C" char*
contextfree_grammar_add_nonterminal_symbol(contextfree_grammar_struc *grammar,
					   const char* name)
{
  return grammar->object->addNonterminalSymbol(name);
}

extern "C" bool
contextfree_grammar_remove_nonterminal_symbol
(contextfree_grammar_struc *grammar, const char* symbol_id)
{
  return grammar->object->removeNonterminalSymbol(symbol_id);
}

extern "C" char*
contextfree_grammar_edit_nonterminal_symbol
(contextfree_grammar_struc *grammar, const char* symbol_id,
 const char* new_name)
{
  return grammar->object->editNonterminalSymbol(symbol_id, new_name);
}

extern "C" char*
contextfree_grammar_add_terminal_symbol(contextfree_grammar_struc *grammar,
					const char* name)
{
  return grammar->object->addTerminalSymbol(name);
}

extern "C" bool
contextfree_grammar_remove_terminal_symbol(contextfree_grammar_struc *grammar,
					   const char* symbol_id)
{
  return grammar->object->removeTerminalSymbol(symbol_id);
}

extern "C" char*
contextfree_grammar_edit_terminal_symbol(contextfree_grammar_struc *grammar,
					 const char* symbol_id,
					 const char* new_name)
{
  return grammar->object->editTerminalSymbol(symbol_id, new_name);
}

extern "C" bool 
contextfree_grammar_set_start_symbol(contextfree_grammar_struc *grammar,
				     const char* symbol_id)
{
  return grammar->object->setStartSymbol(symbol_id);
}

extern "C" bool
contextfree_grammar_add_rule(contextfree_grammar_struc *grammar,
			     const char* left, const char* right)
{
  return grammar->object->addRule(left, right);
}

extern "C" bool
contextfree_grammar_remove_rule(contextfree_grammar_struc *grammar,
				const char* left, const char* right)
{
  return grammar->object->removeRule(left, right);
}

extern "C" char* 
contextfree_grammar_get_look(contextfree_grammar_struc *grammar)
{
  return grammar->object->getLook();
}

extern "C" char* 
contextfree_grammar_get_classification(contextfree_grammar_struc *grammar)
{
  return grammar->object->getClassification();
}

extern "C" void
contextfree_grammar_to_extended(contextfree_grammar_struc *grammar)
{
  grammar->object->toExtended();
}

extern "C" char* 
contextfree_grammar_get_redundant_symbols(contextfree_grammar_struc *grammar)
{
  return grammar->object->getRedundantSymbols();
}

extern "C" char*
contextfree_grammar_get_inaccessible_symbols
(contextfree_grammar_struc *grammar)
{
  return grammar->object->getInaccessibleSymbols();
}

extern "C" void
contextfree_grammar_remove_redundant_symbols
(contextfree_grammar_struc *grammar)
{
  grammar->object->removeRedundantSymbols();
}

extern "C" char* 
contextfree_grammar_get_e_rules(contextfree_grammar_struc *grammar)
{
  return grammar->object->getERules();
}

extern "C" bool 
contextfree_grammar_is_e_rules_free(contextfree_grammar_struc *grammar)
{
  return grammar->object->isERulesfree();
}

extern "C" void
contextfree_grammar_remove_e_rules(contextfree_grammar_struc *grammar)
{
  grammar->object->removeERules();
}

extern "C" char*
contextfree_grammar_get_simple_rules(contextfree_grammar_struc *grammar)
{
  return grammar->object->getSimpleRules();
}

extern "C" void
contextfree_grammar_remove_simple_rules(contextfree_grammar_struc *grammar)
{
  grammar->object->removeSimpleRules();
}

extern "C" bool
contextfree_grammar_is_self(contextfree_grammar_struc *grammar)
{
  return grammar->object->isSelf();
}

extern "C" void
contextfree_grammar_to_self(contextfree_grammar_struc *grammar)
{
  grammar->object->toSelf();
}

extern "C" bool
contextfree_grammar_is_in_chomsky_normal_form
(contextfree_grammar_struc *grammar)
{
  return grammar->object->isinChomskyNormalForm();
}

extern "C" void
contextfree_grammar_to_chomsky_normal_form(contextfree_grammar_struc *grammar)
{
  grammar->object->toChomskyNormalForm();
}

extern "C" char*
contextfree_grammar_get_recursive_symbols(contextfree_grammar_struc *grammar)
{
  return grammar->object->getRecursiveSymbols();
}

extern "C" bool
contextfree_grammar_is_recursive(contextfree_grammar_struc *grammar)
{
  return grammar->object->isRecursive();
}

extern "C" void
contextfree_grammar_eliminate_left_recursion
(contextfree_grammar_struc *grammar)
{
  grammar->object->eliminateLeftRecursion();
}

extern "C" bool
contextfree_grammar_is_in_greibach_normal_form
(contextfree_grammar_struc *grammar)
{
  return grammar->object->isinGreibachNormalForm();
}

extern "C" void
contextfree_grammar_to_greibach_normal_form(contextfree_grammar_struc *grammar)
{
  grammar->object->toGreibachNormalForm();
}

extern "C" stack_automata_struc *
contextfree_grammar_to_stack_automata(contextfree_grammar_struc *grammar,
				      bool top_down)
{
  stack_automata_struc *result = new stack_automata_struc;
  result->object = grammar->object->toStackAutomata(top_down);
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" void 
contextfree_grammar_undo(contextfree_grammar_struc *grammar)
{
  grammar->object->undo();
}

extern "C" void 
contextfree_grammar_redo(contextfree_grammar_struc *grammar)
{
  grammar->object->redo();
}

extern "C" finite_automata_struc *
formal_new_finite_automata()
{
  finite_automata_struc *result = new finite_automata_struc;
  result->object = formal_mico->newFiniteAutomata();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" stack_automata_struc *
formal_new_stack_automata()
{
  stack_automata_struc *result = new stack_automata_struc;
  result->object = formal_mico->newStackAutomata();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" grammar_struc *
formal_new_grammar()
{
  grammar_struc *result = new grammar_struc;
  result->object = formal_mico->newGrammar();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" regular_grammar_struc *
formal_new_regular_grammar()
{
  regular_grammar_struc *result = new regular_grammar_struc;
  result->object = formal_mico->newRegularGrammar();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" contextfree_grammar_struc *
formal_new_contextfree_grammar()
{
  contextfree_grammar_struc *result = new contextfree_grammar_struc;
  result->object = formal_mico->newContextfreeGrammar();
  if (CORBA::is_nil (result->object)) { delete result; result = NULL; }
  return result;
}

extern "C" int
formal_get_error()
{
  return formal_mico->getError();
}

extern "C" void
formal_set_error(int err)
{
  formal_mico->setError(err);
}
