/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CLIENT_INTERFACE_H__
#define __CLIENT_INTERFACE_H__

typedef int bool;

/* Wrapping of interface FiniteAutomata_mico */

struct finite_automata_struc;

/* forward declaration */
struct regular_grammar_struc;

/* derived from interface Automata_mico  - begin */

char *
finite_automata_add_state(struct finite_automata_struc *automata,
			  const char *name);

bool 
finite_automata_remove_state(struct finite_automata_struc *automata,
			     const char *state_id);

char *
finite_automata_edit_state(struct finite_automata_struc *automata,
			   const char *state_id, const char *new_name);

char *
finite_automata_add_entry_symbol(struct finite_automata_struc *automata,
				 const char *name);

bool
finite_automata_remove_entry_symbol(struct finite_automata_struc *automata,
				    const char *symbol_id);

char *
finite_automata_edit_entry_symbol(struct finite_automata_struc *automata,
				  const char *symbol_id, const char *new_name);

bool
finite_automata_set_start_state(struct finite_automata_struc *automata,
				const char *state_id);

bool
finite_automata_set_end_state(struct finite_automata_struc *automata,
			      const char *state_id);

bool
finite_automata_reset_end_state(struct finite_automata_struc *automata,
				const char *state_id);

/* derived from interface Automata_mico  - end */

bool
finite_automata_add_transition(struct finite_automata_struc *automata,
			       const char* from_state_id,
			       const char* symbol_id, const char* to_state_id);

bool
finite_automata_remove_transition(struct finite_automata_struc *automata,
				  const char* from_state_id,
				  const char* symbol_id,
				  const char* to_state_id);

bool
finite_automata_set_configuration(struct finite_automata_struc *automata,
				  const char* state_id, const char* sentence);

void
finite_automata_get_configuration(struct finite_automata_struc *automata,
				  char** state_id, char** sentence);

char *
finite_automata_get_look(struct finite_automata_struc *automata);

char *
finite_automata_get_transition_destinations
(struct finite_automata_struc *automata);

bool
finite_automata_do_transition(struct finite_automata_struc *automata,
			      const char* state_id);

bool
finite_automata_check_sentence(struct finite_automata_struc *automata,
			       const char* sentence);

bool
finite_automata_is_deterministic(struct finite_automata_struc *automata);

void 
finite_automata_to_deterministic(struct finite_automata_struc *automata);

char *
finite_automata_get_inaccessible_states
(struct finite_automata_struc *automata);

void 
finite_automata_to_reduced(struct finite_automata_struc *automata);

struct regular_grammar_struc *
finite_automata_to_regular_grammar(struct finite_automata_struc *automata);

void
finite_automata_undo(struct finite_automata_struc *automata);

void
finite_automata_redo(struct finite_automata_struc *automata);

/* Wrapping of interface StackAutomata_mico */

struct stack_automata_struc;

/* forward declaration */
struct contextfree_grammar_struc;

/* derived from interface Automata_mico  - begin */

char *
stack_automata_add_state(struct stack_automata_struc *automata,
			 const char *name);

bool
stack_automata_remove_state(struct stack_automata_struc *automata,
			    const char *state_id);

char *
stack_automata_edit_state(struct stack_automata_struc *automata,
			  const char *state_id, const char *new_name);

char *
stack_automata_add_entry_symbol(struct stack_automata_struc *automata,
				const char *name);

bool
stack_automata_remove_entry_symbol(struct stack_automata_struc *automata,
				   const char *symbol_id);

char *
stack_automata_edit_entry_symbol(struct stack_automata_struc *automata,
				 const char *symbol_id, const char *new_name);

bool
stack_automata_set_start_state(struct stack_automata_struc *automata,
			       const char *state_id);

bool
stack_automata_set_end_state(struct stack_automata_struc *automata,
			     const char *state_id);

bool
stack_automata_reset_end_state(struct stack_automata_struc *automata,
			       const char *state_id);

/* derived from interface Automata_mico  - end */

char *
stack_automata_add_stack_symbol(struct stack_automata_struc *automata,
				const char* name);

bool
stack_automata_remove_stack_symbol(struct stack_automata_struc *automata,
				   const char* symbol_id);

char *
stack_automata_edit_stack_symbol(struct stack_automata_struc *automata,
				 const char* symbol_id, const char* new_name);

bool
stack_automata_set_start_stack_symbol(struct stack_automata_struc *automata,
				      const char* symbol_id);

bool
stack_automata_add_transition(struct stack_automata_struc *automata,
			      const char* from_state_id,
			      const char* symbol_id, const char* to_state_id,
			      const char* from_stack, const char* to_stack);

bool
stack_automata_remove_transition(struct stack_automata_struc *automata,
				 const char* from_state_id,
				 const char* symbol_id,
				 const char* to_state_id,
				 const char* from_stack, const char* to_stack);

bool
stack_automata_set_configuration(struct stack_automata_struc *automata,
				 const char* state_id, const char* sentence,
				 const char* stack);

void
stack_automata_get_configuration(struct stack_automata_struc *automata,
				 char** state_id, char** sentence,
				 char** stack);

char *
stack_automata_get_look(struct stack_automata_struc *automata);

char *
stack_automata_get_from_stacks(struct stack_automata_struc *automata);

char *
stack_automata_get_transition_destinations
(struct stack_automata_struc *automata, const char* from_stack);

bool
stack_automata_do_transition(struct stack_automata_struc *automata,
			     const char* state_id, const char* from_stack,
			     const char* to_stack);

bool
stack_automata_check_sentence(struct stack_automata_struc *automata,
			      const char* sentence);

bool
stack_automata_is_deterministic(struct stack_automata_struc *automata);

bool
stack_automata_is_top_stack_working(struct stack_automata_struc *automata);

void
stack_automata_to_top_stack_working(struct stack_automata_struc *automata);

bool
stack_automata_is_empty_stack_checking(struct stack_automata_struc *automata);

void
stack_automata_to_empty_stack_checking(struct stack_automata_struc *automata);

void
stack_automata_to_end_states_checking(struct stack_automata_struc *automata);

struct contextfree_grammar_struc *
stack_automata_to_contextfree_grammar(struct stack_automata_struc *automata);

void
stack_automata_undo(struct stack_automata_struc *automata);

void
stack_automata_redo(struct stack_automata_struc *automata);

/* Wrapping of interface Grammar_mico */

struct grammar_struc;

char* 
grammar_add_nonterminal_symbol(struct grammar_struc *grammar,
			       const char* name);

bool
grammar_remove_nonterminal_symbol(struct grammar_struc *grammar,
				  const char* symbol_id);

char* 
grammar_edit_nonterminal_symbol(struct grammar_struc *grammar,
				const char* symbol_id, const char* new_name);

char* 
grammar_add_terminal_symbol(struct grammar_struc *grammar, const char* name);

bool
grammar_remove_terminal_symbol(struct grammar_struc *grammar,
			       const char* symbol_id);

char*
grammar_edit_terminal_symbol(struct grammar_struc *grammar,
			     const char* symbol_id, const char* new_name);

bool
grammar_set_start_symbol(struct grammar_struc *grammar, const char* symbol_id);

bool
grammar_add_rule(struct grammar_struc *grammar, const char* left,
		 const char* right);

bool
grammar_remove_rule(struct grammar_struc *grammar, const char* left,
		    const char* right);

char* 
grammar_get_look(struct grammar_struc *grammar);

char* 
grammar_get_classification(struct grammar_struc *grammar);

void 
grammar_to_extended(struct grammar_struc *grammar);

void
grammar_undo(struct grammar_struc *grammar);

void
grammar_redo(struct grammar_struc *grammar);

/* Wrapping of interface RegularGrammar_mico */

/* derived from interface Grammar_mico  - begin */

char* 
regular_grammar_add_nonterminal_symbol(struct regular_grammar_struc *grammar,
				       const char* name);

bool
regular_grammar_remove_nonterminal_symbol
(struct regular_grammar_struc *grammar, const char* symbol_id);

char* 
regular_grammar_edit_nonterminal_symbol(struct regular_grammar_struc *grammar,
					const char* symbol_id,
					const char* new_name);

char* 
regular_grammar_add_terminal_symbol(struct regular_grammar_struc *grammar,
				    const char* name);

bool
regular_grammar_remove_terminal_symbol(struct regular_grammar_struc *grammar,
				       const char* symbol_id);

char*
regular_grammar_edit_terminal_symbol(struct regular_grammar_struc *grammar,
				     const char* symbol_id,
				     const char* new_name);

bool
regular_grammar_set_start_symbol(struct regular_grammar_struc *grammar,
				 const char* symbol_id);

bool
regular_grammar_add_rule(struct regular_grammar_struc *grammar,
			 const char* left, const char* right);

bool
regular_grammar_remove_rule(struct regular_grammar_struc *grammar,
			    const char* left, const char* right);

char* 
regular_grammar_get_look(struct regular_grammar_struc *grammar);

char* 
regular_grammar_get_classification(struct regular_grammar_struc *grammar);

void 
regular_grammar_to_extended(struct regular_grammar_struc *grammar);

/* derived from interface Grammar_mico  - end */

struct finite_automata_struc *
regular_grammar_to_finite_automata(struct regular_grammar_struc *grammar);

void
regular_grammar_undo(struct regular_grammar_struc *grammar);

void
regular_grammar_redo(struct regular_grammar_struc *grammar);

/* Wrapping of interface ContextfreeGrammar_mico */

/* derived from interface Grammar_mico  - begin */

char*
contextfree_grammar_add_nonterminal_symbol
(struct contextfree_grammar_struc *grammar, const char* name);

bool
contextfree_grammar_remove_nonterminal_symbol
(struct contextfree_grammar_struc *grammar, const char* symbol_id);

char* 
contextfree_grammar_edit_nonterminal_symbol
(struct contextfree_grammar_struc *grammar, const char* symbol_id,
 const char* new_name);

char*
contextfree_grammar_add_terminal_symbol
(struct contextfree_grammar_struc *grammar, const char* name);

bool
contextfree_grammar_remove_terminal_symbol
(struct contextfree_grammar_struc *grammar, const char* symbol_id);

char*
contextfree_grammar_edit_terminal_symbol
(struct contextfree_grammar_struc *grammar, const char* symbol_id,
 const char* new_name);

bool
contextfree_grammar_set_start_symbol
(struct contextfree_grammar_struc *grammar, const char* symbol_id);

bool
contextfree_grammar_add_rule(struct contextfree_grammar_struc *grammar,
			     const char* left, const char* right);

bool
contextfree_grammar_remove_rule(struct contextfree_grammar_struc *grammar,
				const char* left, const char* right);

char*
contextfree_grammar_get_look(struct contextfree_grammar_struc *grammar);

char*
contextfree_grammar_get_classification
(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_to_extended(struct contextfree_grammar_struc *grammar);

/* derived from interface Grammar_mico  - end */

char* 
contextfree_grammar_get_redundant_symbols
(struct contextfree_grammar_struc *grammar);

char* 
contextfree_grammar_get_inaccessible_symbols
(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_remove_redundant_symbols
(struct contextfree_grammar_struc *grammar);

char*
contextfree_grammar_get_e_rules(struct contextfree_grammar_struc *grammar);

bool
contextfree_grammar_is_e_rules_free(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_remove_e_rules(struct contextfree_grammar_struc *grammar);

char*
contextfree_grammar_get_simple_rules
(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_remove_simple_rules
(struct contextfree_grammar_struc *grammar);

bool
contextfree_grammar_is_self(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_to_self(struct contextfree_grammar_struc *grammar);

bool
contextfree_grammar_is_in_chomsky_normal_form
(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_to_chomsky_normal_form
(struct contextfree_grammar_struc *grammar);

char*
contextfree_grammar_get_recursive_symbols
(struct contextfree_grammar_struc *grammar);

bool
contextfree_grammar_is_recursive(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_eliminate_left_recursion
(struct contextfree_grammar_struc *grammar);

bool
contextfree_grammar_is_in_greibach_normal_form
(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_to_greibach_normal_form
(struct contextfree_grammar_struc *grammar);

struct stack_automata_struc *
contextfree_grammar_to_stack_automata
(struct contextfree_grammar_struc *grammar, bool top_down);

void
contextfree_grammar_undo(struct contextfree_grammar_struc *grammar);

void
contextfree_grammar_redo(struct contextfree_grammar_struc *grammar);

/* Wrapping of interface Formal_mico */

struct finite_automata_struc *
formal_new_finite_automata();

struct stack_automata_struc *
formal_new_stack_automata();

struct grammar_struc *
formal_new_grammar();

struct regular_grammar_struc *
formal_new_regular_grammar();

struct contextfree_grammar_struc *
formal_new_contextfree_grammar();

int 
formal_get_error();

void
formal_set_error(int err);

#endif
