/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "interface_impl.h"
#include "server/conversions_l.h"
#include "server/error.h"

// Implementation for interface Automata_mico

char*
Automata_mico_impl::addState(const char* name)
{
  char* retval;
  retval = CORBA::string_dup(automata_l->addState(name).c_str());
  return retval; 
}

CORBA::Boolean
Automata_mico_impl::removeState(const char* state_id)
{
  CORBA::Boolean retval;
  retval = automata_l->removeState(state_id);
  return retval; 
}

char*
Automata_mico_impl::editState(const char* state_id, const char* new_name)
{
  char* retval;
  retval = CORBA::string_dup
    (automata_l->editState(state_id, new_name).c_str());
  return retval; 
}

char*
Automata_mico_impl::addEntrySymbol(const char* name)
{
  char* retval;
  retval = CORBA::string_dup(automata_l->addEntrySymbol(name).c_str());
  return retval; 
}

CORBA::Boolean
Automata_mico_impl::removeEntrySymbol(const char* symbol_id)
{
  CORBA::Boolean retval;
  retval = automata_l->removeEntrySymbol(symbol_id);
  return retval; 
}

char*
Automata_mico_impl::editEntrySymbol(const char* symbol_id,
				    const char* new_name)
{
  char* retval;
  retval = CORBA::string_dup
    (automata_l->editEntrySymbol(symbol_id, new_name).c_str());
  return retval; 
}

CORBA::Boolean
Automata_mico_impl::setStartState(const char* state_id)
{
  CORBA::Boolean retval;
  retval = automata_l->setStartState(state_id);
  return retval; 
}

CORBA::Boolean
Automata_mico_impl::setEndState(const char* state_id)
{
  CORBA::Boolean retval;
  retval = automata_l->setEndState(state_id);
  return retval; 
}

CORBA::Boolean
Automata_mico_impl::resetEndState(const char* state_id)
{
  CORBA::Boolean retval;
  retval = automata_l->resetEndState(state_id);
  return retval; 
}

// Implementation for interface FiniteAutomata_mico

FiniteAutomata_mico_impl::FiniteAutomata_mico_impl()
{
  automata_l = new FiniteAutomata_l;
  Automata_mico_impl::automata_l = automata_l;
}

FiniteAutomata_mico_impl::~FiniteAutomata_mico_impl()
{
  delete automata_l;
}

char*
FiniteAutomata_mico_impl::addState(const char* name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::addState(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::removeState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::removeState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char* 
FiniteAutomata_mico_impl::editState(const char* state_id, const char* new_name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::editState(state_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
FiniteAutomata_mico_impl::addEntrySymbol(const char* name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::addEntrySymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::removeEntrySymbol(const char* symbol_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::removeEntrySymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
FiniteAutomata_mico_impl::editEntrySymbol(const char* symbol_id,
					  const char* new_name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::editEntrySymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::setStartState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::setStartState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::setEndState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::setEndState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::resetEndState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::resetEndState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::addTransition(const char* from_state_id,
					const char* symbol_id,
					const char* to_state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->addTransition(from_state_id, symbol_id, to_state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::removeTransition(const char* from_state_id,
					   const char* symbol_id,
					   const char* to_state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->removeTransition(from_state_id, symbol_id, to_state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean 
FiniteAutomata_mico_impl::setConfiguration(const char* state_id,
					   const char* sentence)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->setConfiguration(state_id, sentence);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

void
FiniteAutomata_mico_impl::getConfiguration(char*& state, char*& sentence)
{
  setError(E_NOERR);
  string st, sent;
  automata_l->getConfiguration(st, sent);
  state = CORBA::string_dup(st.c_str());
  sentence = CORBA::string_dup(sent.c_str());
}

char*
FiniteAutomata_mico_impl::getLook()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->getLook().c_str());
  return retval; 
}

char*
FiniteAutomata_mico_impl::getTransitionDestinations()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->getTransitionDestinations().c_str());
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::doTransition(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->doTransition(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
FiniteAutomata_mico_impl::checkSentence(const char* sentence)
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->checkSentence(sentence);
  return retval; 
}

CORBA::Boolean 
FiniteAutomata_mico_impl::isDeterministic()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->isDeterministic();
  return retval; 
}

void
FiniteAutomata_mico_impl::toDeterministic()
{
  undo_list.save(automata_l); setError(E_NOERR);
  automata_l->toDeterministic();
  if (getError() != E_NOERR) undo_list.removeLast();
}

char*
FiniteAutomata_mico_impl::getInaccessibleStates()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->getInaccessibleStates().c_str());
  return retval; 
}

void
FiniteAutomata_mico_impl::toReduced()
{
  undo_list.save(automata_l); setError(E_NOERR);
  automata_l->toReduced();
  if (getError() != E_NOERR) undo_list.removeLast();
}

RegularGrammar_mico_ptr
FiniteAutomata_mico_impl::toRegularGrammar()
{
  setError(E_NOERR);
  RegularGrammar_mico_ptr retval;
  RegularGrammar_mico_impl *grammar_i = new RegularGrammar_mico_impl;
  *(grammar_i->grammar_l) = FiniteAutomata_l_to_RegularGrammar_l(automata_l);
  retval = grammar_i->_this();
  return retval; 
}

void
FiniteAutomata_mico_impl::undo()
{
  FiniteAutomata_l *a = undo_list.undo(automata_l);
  *automata_l = *a;
  delete a;
}

void
FiniteAutomata_mico_impl::redo()
{
  FiniteAutomata_l *a = undo_list.redo(automata_l);
  *automata_l = *a;
  delete a;
}

// Implementation for interface StackAutomata_mico

StackAutomata_mico_impl::StackAutomata_mico_impl()
{
  automata_l = new StackAutomata_l;
  Automata_mico_impl::automata_l = automata_l;
}

StackAutomata_mico_impl::~StackAutomata_mico_impl()
{
  delete automata_l;
}

char*
StackAutomata_mico_impl::addState(const char* name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::addState(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
StackAutomata_mico_impl::removeState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::removeState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char* 
StackAutomata_mico_impl::editState(const char* state_id, const char* new_name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::editState(state_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
StackAutomata_mico_impl::addEntrySymbol(const char* name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::addEntrySymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
StackAutomata_mico_impl::removeEntrySymbol(const char* symbol_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::removeEntrySymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
StackAutomata_mico_impl::editEntrySymbol(const char* symbol_id,
					  const char* new_name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = Automata_mico_impl::editEntrySymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
StackAutomata_mico_impl::setStartState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::setStartState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
StackAutomata_mico_impl::setEndState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::setEndState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
StackAutomata_mico_impl::resetEndState(const char* state_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Automata_mico_impl::resetEndState(state_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
StackAutomata_mico_impl::addStackSymbol(const char* name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->addStackSymbol(name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean
StackAutomata_mico_impl::removeStackSymbol(const char* symbol_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->removeStackSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


char*
StackAutomata_mico_impl::editStackSymbol(const char* symbol_id,
					 const char* new_name)
{
  undo_list.save(automata_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup
    (automata_l->editStackSymbol(symbol_id, new_name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean
StackAutomata_mico_impl::setStartStackSymbol(const char* symbol_id)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->setStartStackSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean
StackAutomata_mico_impl::addTransition(const char* from_state_id,
				       const char* symbol_id,
				       const char* to_state_id,
				       const char* from_stack,
				       const char* to_stack)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = 
    automata_l->addTransition
    (from_state_id, symbol_id, to_state_id, from_stack, to_stack);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean 
StackAutomata_mico_impl::removeTransition(const char* from_state_id,
					  const char* symbol_id,
					  const char* to_state_id,
					  const char* from_stack,
					  const char* to_stack)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = 
    automata_l->removeTransition
    (from_state_id, symbol_id, to_state_id, from_stack, to_stack);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean 
StackAutomata_mico_impl::setConfiguration(const char* state_id,
					  const char* sentence,
					  const char* stack)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->setConfiguration(state_id, sentence, stack);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


void
StackAutomata_mico_impl::getConfiguration(char*& state, char*& sentence,
					  char*& stack)
{
  setError(E_NOERR);
  string st, sent, stk;
  automata_l->getConfiguration(st, sent, stk);
  state = CORBA::string_dup(st.c_str());
  sentence = CORBA::string_dup(sent.c_str());
  stack = CORBA::string_dup(stk.c_str());
}


char*
StackAutomata_mico_impl::getLook()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->getLook().c_str());
  return retval; 
}


char* 
StackAutomata_mico_impl::getFromStacks()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(automata_l->getFromStacks().c_str());
  return retval; 
}


char* 
StackAutomata_mico_impl::getTransitionDestinations(const char* from_stack)
{
  setError(E_NOERR);
  char* retval;
  retval = 
    CORBA::string_dup
    (automata_l->getTransitionDestinations(from_stack).c_str());
  return retval; 
}


CORBA::Boolean 
StackAutomata_mico_impl::doTransition(const char* state_id,
				      const char* from_stack,
				      const char* to_stack)
{
  undo_list.save(automata_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->doTransition(state_id, from_stack, to_stack);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}


CORBA::Boolean 
StackAutomata_mico_impl::checkSentence(const char* sentence)
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->checkSentence(sentence);
  return retval; 
}


CORBA::Boolean
StackAutomata_mico_impl::isDeterministic()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->isDeterministic();
  return retval; 
}


CORBA::Boolean
StackAutomata_mico_impl::isTopStackWorking()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->isTopStackWorking();
  return retval; 
}


void
StackAutomata_mico_impl::toTopStackWorking()
{
  undo_list.save(automata_l); setError(E_NOERR);
  automata_l->toTopStackWorking();
  if (getError() != E_NOERR) undo_list.removeLast();
}


CORBA::Boolean
StackAutomata_mico_impl::isEmptyStackChecking()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = automata_l->isEmptyStackChecking();
  return retval; 
}


void
StackAutomata_mico_impl::toEmptyStackChecking()
{
  undo_list.save(automata_l); setError(E_NOERR);
  automata_l->toEmptyStackChecking();
  if (getError() != E_NOERR) undo_list.removeLast();
}


void
StackAutomata_mico_impl::toEndStatesChecking()
{
  undo_list.save(automata_l); setError(E_NOERR);
  automata_l->toEndStatesChecking();
  if (getError() != E_NOERR) undo_list.removeLast();
}

ContextfreeGrammar_mico_ptr
StackAutomata_mico_impl::toContextfreeGrammar()
{
  setError(E_NOERR);
  ContextfreeGrammar_mico_ptr retval;
  ContextfreeGrammar_mico_impl *grammar_i = new ContextfreeGrammar_mico_impl;
  *(grammar_i->grammar_l) =
    StackAutomata_l_to_ContextfreeGrammar_l(automata_l);
  retval = grammar_i->_this();
  return retval; 
}

void
StackAutomata_mico_impl::undo()
{
  StackAutomata_l *a = undo_list.undo(automata_l);
  *automata_l = *a;
  delete a;
}

void
StackAutomata_mico_impl::redo()
{
  StackAutomata_l *a = undo_list.redo(automata_l);
  *automata_l = *a;
  delete a;
}

// Implementation for interface Grammar_mico

Grammar_mico_impl::Grammar_mico_impl()
{
  grammar_l = new Grammar_l;
}

Grammar_mico_impl::~Grammar_mico_impl()
{
  delete grammar_l;
}

char*
Grammar_mico_impl::addNonterminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->addNonterminalSymbol(name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
Grammar_mico_impl::removeNonterminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->removeNonterminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
Grammar_mico_impl::editNonterminalSymbol(const char* symbol_id,
					 const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup
    (grammar_l->editNonterminalSymbol(symbol_id, new_name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
Grammar_mico_impl::addTerminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->addTerminalSymbol(name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
Grammar_mico_impl::removeTerminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->removeTerminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
Grammar_mico_impl::editTerminalSymbol(const char* symbol_id,
				      const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup
    (grammar_l->editTerminalSymbol(symbol_id, new_name).c_str());
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean 
Grammar_mico_impl::setStartSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->setStartSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
Grammar_mico_impl::addRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->addRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
Grammar_mico_impl::removeRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->removeRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
Grammar_mico_impl::getLook()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getLook().c_str());
  return retval; 
}

char*
Grammar_mico_impl::getClassification()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getClassification().c_str());
  return retval; 
}

void
Grammar_mico_impl::toExtended()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->toExtended();
  if (getError() != E_NOERR) undo_list.removeLast();
}

void
Grammar_mico_impl::undo()
{
  Grammar_l *g = undo_list.undo(grammar_l);
  *grammar_l = *g;
  delete g;
}

void
Grammar_mico_impl::redo()
{
  Grammar_l *g = undo_list.redo(grammar_l);
  *grammar_l = *g;
  delete g;
}

// Implementation for interface RegularGrammar_mico

RegularGrammar_mico_impl::RegularGrammar_mico_impl()
{
  grammar_l = new RegularGrammar_l;
  Grammar_mico_impl::grammar_l = grammar_l;
}

RegularGrammar_mico_impl::~RegularGrammar_mico_impl()
{
  delete grammar_l;
}

char*
RegularGrammar_mico_impl::addNonterminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::addNonterminalSymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
RegularGrammar_mico_impl::removeNonterminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeNonterminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
RegularGrammar_mico_impl::editNonterminalSymbol(const char* symbol_id,
						const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::editNonterminalSymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
RegularGrammar_mico_impl::addTerminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::addTerminalSymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
RegularGrammar_mico_impl::removeTerminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeTerminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
RegularGrammar_mico_impl::editTerminalSymbol(const char* symbol_id,
					     const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::editTerminalSymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
RegularGrammar_mico_impl::setStartSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::setStartSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
RegularGrammar_mico_impl::addRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::addRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
RegularGrammar_mico_impl::removeRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

void
RegularGrammar_mico_impl::toExtended()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  Grammar_mico_impl::toExtended();
  if (getError() != E_NOERR) undo_list.removeLast();
}

FiniteAutomata_mico_ptr
RegularGrammar_mico_impl::toFiniteAutomata()
{
  setError(E_NOERR);
  FiniteAutomata_mico_ptr retval;
  FiniteAutomata_mico_impl *automata_i = new FiniteAutomata_mico_impl;
  *(automata_i->automata_l) = RegularGrammar_l_to_FiniteAutomata_l(grammar_l);
  retval = automata_i->_this();
  return retval; 
}

void
RegularGrammar_mico_impl::undo()
{
  RegularGrammar_l *g = undo_list.undo(grammar_l);
  *grammar_l = *g;
  delete g;
}

void
RegularGrammar_mico_impl::redo()
{
  RegularGrammar_l *g = undo_list.redo(grammar_l);
  *grammar_l = *g;
  delete g;
}

// Implementation for interface ContextfreeGrammar_mico

ContextfreeGrammar_mico_impl::ContextfreeGrammar_mico_impl()
{
  grammar_l = new ContextfreeGrammar_l;
  Grammar_mico_impl::grammar_l = grammar_l;
}

ContextfreeGrammar_mico_impl::~ContextfreeGrammar_mico_impl()
{
  delete grammar_l;
}

char*
ContextfreeGrammar_mico_impl::addNonterminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::addNonterminalSymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::removeNonterminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeNonterminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
ContextfreeGrammar_mico_impl::editNonterminalSymbol(const char* symbol_id,
						const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::editNonterminalSymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
ContextfreeGrammar_mico_impl::addTerminalSymbol(const char* name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::addTerminalSymbol(name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::removeTerminalSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeTerminalSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

char*
ContextfreeGrammar_mico_impl::editTerminalSymbol(const char* symbol_id,
					     const char* new_name)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  char* retval;
  retval = Grammar_mico_impl::editTerminalSymbol(symbol_id, new_name);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::setStartSymbol(const char* symbol_id)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::setStartSymbol(symbol_id);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::addRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::addRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::removeRule(const char* left, const char* right)
{
  undo_list.save(grammar_l); setError(E_NOERR);
  CORBA::Boolean retval;
  retval = Grammar_mico_impl::removeRule(left, right);
  if (getError() != E_NOERR) undo_list.removeLast();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::toExtended()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  Grammar_mico_impl::toExtended();
  if (getError() != E_NOERR) undo_list.removeLast();
}

char*
ContextfreeGrammar_mico_impl::getRedundantSymbols()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getRedundantSymbols().c_str());
  return retval; 
}

char*
ContextfreeGrammar_mico_impl::getInaccessibleSymbols()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getInaccessibleSymbols().c_str());
  return retval; 
}

void
ContextfreeGrammar_mico_impl::removeRedundantSymbols()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->removeRedundantSymbols();
  if (getError() != E_NOERR) undo_list.removeLast();
}

char*
ContextfreeGrammar_mico_impl::getERules()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getERules().c_str());
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::isERulesfree()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->isERulesfree();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::removeERules()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->removeERules();
  if (getError() != E_NOERR) undo_list.removeLast();
}

char*
ContextfreeGrammar_mico_impl::getSimpleRules()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getSimpleRules().c_str());
  return retval; 
}

void
ContextfreeGrammar_mico_impl::removeSimpleRules()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->removeSimpleRules();
  if (getError() != E_NOERR) undo_list.removeLast();
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::isSelf()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->isSelf();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::toSelf()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->toSelf();
  if (getError() != E_NOERR) undo_list.removeLast();
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::isinChomskyNormalForm()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->isinChomskyNormalForm();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::toChomskyNormalForm()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->toChomskyNormalForm();
  if (getError() != E_NOERR) undo_list.removeLast();
}

char*
ContextfreeGrammar_mico_impl::getRecursiveSymbols()
{
  setError(E_NOERR);
  char* retval;
  retval = CORBA::string_dup(grammar_l->getRecursiveSymbols().c_str());
  return retval; 
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::isRecursive()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->isRecursive();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::eliminateLeftRecursion()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->eliminateLeftRecursion();
  if (getError() != E_NOERR) undo_list.removeLast();
}

CORBA::Boolean
ContextfreeGrammar_mico_impl::isinGreibachNormalForm()
{
  setError(E_NOERR);
  CORBA::Boolean retval;
  retval = grammar_l->isinGreibachNormalForm();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::toGreibachNormalForm()
{
  undo_list.save(grammar_l); setError(E_NOERR);
  grammar_l->toGreibachNormalForm();
  if (getError() != E_NOERR) undo_list.removeLast();
}

StackAutomata_mico_ptr
ContextfreeGrammar_mico_impl::toStackAutomata(CORBA::Boolean top_down)
{
  setError(E_NOERR);
  StackAutomata_mico_ptr retval;
  StackAutomata_mico_impl *automata_i = new StackAutomata_mico_impl;
  *(automata_i->automata_l) =
    ContextfreeGrammar_l_to_StackAutomata_l(grammar_l, top_down);
  retval = automata_i->_this();
  return retval; 
}

void
ContextfreeGrammar_mico_impl::undo()
{
  ContextfreeGrammar_l *g = undo_list.undo(grammar_l);
  *grammar_l = *g;
  delete g;
}

void
ContextfreeGrammar_mico_impl::redo()
{
  ContextfreeGrammar_l *g = undo_list.redo(grammar_l);
  *grammar_l = *g;
  delete g;
}

// Implementation for interface Formal_mico

FiniteAutomata_mico_ptr
Formal_mico_impl::newFiniteAutomata()
{
  FiniteAutomata_mico_ptr retval;
  FiniteAutomata_mico_impl *automata_i = new FiniteAutomata_mico_impl;
  retval = automata_i->_this();
  return retval; 
}

StackAutomata_mico_ptr
Formal_mico_impl::newStackAutomata()
{
  StackAutomata_mico_ptr retval;
  StackAutomata_mico_impl *automata_i = new StackAutomata_mico_impl;
  retval = automata_i->_this();
  return retval; 
}

Grammar_mico_ptr
Formal_mico_impl::newGrammar()
{
  Grammar_mico_ptr retval;
  Grammar_mico_impl *grammar_i = new Grammar_mico_impl;
  retval = grammar_i->_this();
  return retval; 
}

RegularGrammar_mico_ptr
Formal_mico_impl::newRegularGrammar()
{
  RegularGrammar_mico_ptr retval;
  RegularGrammar_mico_impl *grammar_i = new RegularGrammar_mico_impl;
  retval = grammar_i->_this();
  return retval; 
}

ContextfreeGrammar_mico_ptr
Formal_mico_impl::newContextfreeGrammar()
{
  ContextfreeGrammar_mico_ptr retval;
  ContextfreeGrammar_mico_impl *grammar_i = new ContextfreeGrammar_mico_impl;
  retval = grammar_i->_this();
  return retval; 
}

CORBA::Long
Formal_mico_impl::getError()
{
  CORBA::Long retval;
  retval = ::getError();
  return retval; 
}

void
Formal_mico_impl::setError(CORBA::Long err)
{
  ::setError(err);
}
