/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __INTERFACE_IMPL_H__
#define __INTERFACE_IMPL_H__

#include "interface.h"
#include "server/finite_automata_l.h"
#include "server/stack_automata_l.h"
#include "server/regular_grammar_l.h"
#include "server/contextfree_grammar_l.h"
#include "undoredo.h"

// Implementation for interface Automata_mico

class Automata_mico_impl : virtual public POA_Automata_mico
{
 public:
  Automata_l* automata_l;
 public:
  virtual char* addState(const char* name);
  virtual CORBA::Boolean removeState(const char* state_id);
  virtual char* editState(const char* state_id, const char* new_name);
  virtual char* addEntrySymbol(const char* name);
  virtual CORBA::Boolean removeEntrySymbol(const char* symbol_id);
  virtual char* editEntrySymbol(const char* symbol_id, const char* new_name);
  virtual CORBA::Boolean setStartState(const char* state_id);
  virtual CORBA::Boolean setEndState(const char* state_id);
  virtual CORBA::Boolean resetEndState(const char* state_id);
};

// Implementation for interface FiniteAutomata_mico

class FiniteAutomata_mico_impl : 
virtual public Automata_mico_impl, virtual public POA_FiniteAutomata_mico
{
 public:
  FiniteAutomata_l* automata_l;
 protected:
  UndoRedo<FiniteAutomata_l> undo_list;
 public:
  FiniteAutomata_mico_impl();
  ~FiniteAutomata_mico_impl();
  char* addState(const char* name);
  CORBA::Boolean removeState(const char* state_id);
  char* editState(const char* state_id, const char* new_name);
  char* addEntrySymbol(const char* name);
  CORBA::Boolean removeEntrySymbol(const char* symbol_id);
  char* editEntrySymbol(const char* symbol_id, const char* new_name);
  CORBA::Boolean setStartState(const char* state_id);
  CORBA::Boolean setEndState(const char* state_id);
  CORBA::Boolean resetEndState(const char* state_id);
  CORBA::Boolean addTransition(const char* from_state_id,
			       const char* symbol_id, const char* to_state_id);
  CORBA::Boolean removeTransition(const char* from_state_id,
				  const char* symbol_id,
				  const char* to_state_id);
  CORBA::Boolean setConfiguration(const char* state_id, const char* sentence);
  void getConfiguration(char*& state, char*& sentence);
  char* getLook();
  char* getTransitionDestinations();
  CORBA::Boolean doTransition(const char* state_id);
  CORBA::Boolean checkSentence(const char* sentence);
  CORBA::Boolean isDeterministic();
  void toDeterministic();
  char* getInaccessibleStates();
  void toReduced();
  RegularGrammar_mico_ptr toRegularGrammar();
  void undo();
  void redo();
};

// Implementation for interface StackAutomata_mico

class StackAutomata_mico_impl : 
virtual public Automata_mico_impl, virtual public POA_StackAutomata_mico
{
 public:
  StackAutomata_l* automata_l;
 protected:
  UndoRedo<StackAutomata_l> undo_list;
 public:
  StackAutomata_mico_impl();
  ~StackAutomata_mico_impl();
  char* addState(const char* name);
  CORBA::Boolean removeState(const char* state_id);
  char* editState(const char* state_id, const char* new_name);
  char* addEntrySymbol(const char* name);
  CORBA::Boolean removeEntrySymbol(const char* symbol_id);
  char* editEntrySymbol(const char* symbol_id, const char* new_name);
  CORBA::Boolean setStartState(const char* state_id);
  CORBA::Boolean setEndState(const char* state_id);
  CORBA::Boolean resetEndState(const char* state_id);
  char* addStackSymbol(const char* name);
  CORBA::Boolean removeStackSymbol(const char* symbol_id);
  char* editStackSymbol(const char* symbol_id, const char* new_name);
  CORBA::Boolean setStartStackSymbol(const char* symbol_id);
  CORBA::Boolean addTransition(const char* from_state_id,
			       const char* symbol_id, const char* to_state_id,
			       const char* from_stack, const char* to_stack);
  CORBA::Boolean removeTransition(const char* from_state_id,
				  const char* symbol_id,
				  const char* to_state_id,
				  const char* from_stack,
				  const char* to_stack);
  CORBA::Boolean setConfiguration(const char* state_id, const char* sentence,
				  const char* stack);
  void getConfiguration(char*& state, char*& sentence, char*& stack);
  char* getLook();
  char* getFromStacks();
  char* getTransitionDestinations(const char* from_stack);
  CORBA::Boolean doTransition(const char* state_id, const char* from_stack,
			      const char* to_stack);
  CORBA::Boolean checkSentence(const char* sentence);
  CORBA::Boolean isDeterministic();
  CORBA::Boolean isTopStackWorking();
  void toTopStackWorking();
  CORBA::Boolean isEmptyStackChecking();
  void toEmptyStackChecking();
  void toEndStatesChecking();
  ContextfreeGrammar_mico_ptr toContextfreeGrammar();
  void undo();
  void redo();
};

// Implementation for interface Grammar_mico

class Grammar_mico_impl : virtual public POA_Grammar_mico
{
 public:
  Grammar_l* grammar_l;
 protected:
  UndoRedo<Grammar_l> undo_list;
 public:
  Grammar_mico_impl();
  ~Grammar_mico_impl();
  virtual char* addNonterminalSymbol(const char* name);
  virtual CORBA::Boolean removeNonterminalSymbol(const char* symbol_id);
  virtual char* editNonterminalSymbol(const char* symbol_id,
				      const char* new_name);
  virtual char* addTerminalSymbol(const char* name);
  virtual CORBA::Boolean removeTerminalSymbol(const char* symbol_id);
  virtual char* editTerminalSymbol(const char* symbol_id,
				   const char* new_name);
  virtual CORBA::Boolean setStartSymbol(const char* symbol_id);
  virtual CORBA::Boolean addRule(const char* left, const char* right);
  virtual CORBA::Boolean removeRule(const char* left, const char* right);
  char* getLook();
  char* getClassification();
  virtual void toExtended();
  virtual void undo();
  virtual void redo();
};

// Implementation for interface RegularGrammar_mico

class RegularGrammar_mico_impl : 
virtual public Grammar_mico_impl, virtual public POA_RegularGrammar_mico
{
 public:
  RegularGrammar_l* grammar_l;
 protected:
  UndoRedo<RegularGrammar_l> undo_list;
 public:
  RegularGrammar_mico_impl();
  ~RegularGrammar_mico_impl();
  char* addNonterminalSymbol(const char* name);
  CORBA::Boolean removeNonterminalSymbol(const char* symbol_id);
  char* editNonterminalSymbol(const char* symbol_id, const char* new_name);
  char* addTerminalSymbol(const char* name);
  CORBA::Boolean removeTerminalSymbol(const char* symbol_id);
  char* editTerminalSymbol(const char* symbol_id, const char* new_name);
  CORBA::Boolean setStartSymbol(const char* symbol_id);
  CORBA::Boolean addRule(const char* left, const char* right);
  CORBA::Boolean removeRule(const char* left, const char* right);
  void toExtended();
  FiniteAutomata_mico_ptr toFiniteAutomata();
  void undo();
  void redo();
};

// Implementation for interface ContextfreeGrammar_mico

class ContextfreeGrammar_mico_impl :
virtual public Grammar_mico_impl, virtual public POA_ContextfreeGrammar_mico
{
 public:
  ContextfreeGrammar_l* grammar_l;
 protected:
  UndoRedo<ContextfreeGrammar_l> undo_list;
 public:
  ContextfreeGrammar_mico_impl();
  ~ContextfreeGrammar_mico_impl();
  char* addNonterminalSymbol(const char* name);
  CORBA::Boolean removeNonterminalSymbol(const char* symbol_id);
  char* editNonterminalSymbol(const char* symbol_id, const char* new_name);
  char* addTerminalSymbol(const char* name);
  CORBA::Boolean removeTerminalSymbol(const char* symbol_id);
  char* editTerminalSymbol(const char* symbol_id, const char* new_name);
  CORBA::Boolean setStartSymbol(const char* symbol_id);
  CORBA::Boolean addRule(const char* left, const char* right);
  CORBA::Boolean removeRule(const char* left, const char* right);
  void toExtended();
  char* getRedundantSymbols();
  char* getInaccessibleSymbols();
  void removeRedundantSymbols();
  char* getERules();
  CORBA::Boolean isERulesfree();
  void removeERules();
  char* getSimpleRules();
  void removeSimpleRules();
  CORBA::Boolean isSelf();
  void toSelf();
  CORBA::Boolean isinChomskyNormalForm();
  void toChomskyNormalForm();
  char* getRecursiveSymbols();
  CORBA::Boolean isRecursive();
  void eliminateLeftRecursion();
  CORBA::Boolean isinGreibachNormalForm();
  void toGreibachNormalForm();
  StackAutomata_mico_ptr toStackAutomata(CORBA::Boolean top_down);
  void undo();
  void redo();
};

// Implementation for interface Formal_mico

class Formal_mico_impl : virtual public POA_Formal_mico
{
 public:
  FiniteAutomata_mico_ptr newFiniteAutomata();
  StackAutomata_mico_ptr newStackAutomata();
  Grammar_mico_ptr newGrammar();
  RegularGrammar_mico_ptr newRegularGrammar();
  ContextfreeGrammar_mico_ptr newContextfreeGrammar();
  CORBA::Long getError();
  void setError(CORBA::Long err);
};

#endif
