/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// abstract class of an automata

#ifndef __AUTOMATA_H__
#define __AUTOMATA_H__

#include "automata_elements.h"
#include <vector>
#include <set>

class Automata {
 protected:
  set<State> states;
  set<EntrySymbol> entry_symbols;
  set<Transition> transitions;
  State start_state;
  set<State> end_states;
  State c_state; // configuration - state
  vector<EntrySymbol> c_sentence; // configuration - entry sentence
 public:
  Automata();
  virtual ~Automata();
  State addState();
  // adds new state
  // returns: new state
  // errors: E_A_STATE_MAX
  virtual bool removeState(State state);
  // removes state state
  // returns: if successful
  // errors: E_A_STATE_BAD
  set<State> getStates();
  // returns: all states
  EntrySymbol addEntrySymbol();
  // adds new entry symbol
  // returns: new entry symbol
  // errors: E_A_ENTRY_SYMBOL_MAX
  virtual bool removeEntrySymbol(EntrySymbol symbol);
  // removes entry symbol symbol
  // returns: if successful
  // errors: E_A_ENTRY_SYMBOL_BAD
  set<EntrySymbol> getEntrySymbols();
  // returns: all entry symbols
  bool removeTransition(Transition transition);
  // removes transition transition
  // returns: if successful 
  // errors: E_A_TRANSITION_BAD
  set<Transition> getTransitions();
  // returns: all transitions
  bool setStartState(State state);
  // sets start state to state
  // returns: if successful
  // errors: E_A_STATE_BAD
  State getStartState();
  // returns: start state
  bool setEndState(State state);
  // sets state to be end state
  // returns: if successful
  // errors: E_A_STATE_BAD
  bool resetEndState(State state);
  // sets state not to be end state
  // returns: if successful
  // errors: E_A_STATE_BAD
  set<State> getEndStates();
  // returns: all end states
  virtual bool checkSentence(vector<EntrySymbol> sentence) = 0;
  // returns: if sentence sentence is accepted by automata
  virtual bool isDeterministic() = 0;
  // returns: if automata is deterministic
 protected:
  set<Transition> find_transitions_with_state(State state);
  set<Transition> find_transitions_with_entry_symbol(EntrySymbol symbol);
};

#endif
