/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "automata_elements_l.h"

const char string_space = ' ';
const char string_lbracket = '(';
const char string_rbracket = ')';

State_l::State_l()
{
  name = "";
}

State_l::State_l(string n)
{
  name = n;
}

// can't be inline
string
State_l::getName()
{
  return name;
}

string
State_l::getLook(State state, t_flags flags)
{
  string result;
  if (state == 0) return result;
  result += string_lbracket;
  result += "state";
  result += string_space + State_to_string(state) + string_space;
  result += name;
  string flag_string;
  if (flags & F_START) flag_string += 's';
  if (flags & F_END) flag_string += 'e';
  if (!flag_string.empty()) result += string_space + flag_string;
  result += string_rbracket;
  return result;
}

string
State_l::State_to_string(State state)
{
  string result;
  for (; state > 0; state /= 10) result += state % 10 + '0';
  if (result.empty()) result = '0';
  reverse(result.begin(), result.end());
  return result;
}

EntrySymbol_l::EntrySymbol_l()
{
  name = "";
}

EntrySymbol_l::EntrySymbol_l(string n)
{
  name = n;
}

// can't be inline
string
EntrySymbol_l::getName()
{
  return name;
}

string
EntrySymbol_l::getLook(EntrySymbol symbol)
{
  string result;
  if (symbol == 0) return result;
  result += string_lbracket;
  result += "entry_symbol";
  result += string_space + EntrySymbol_to_string(symbol) + string_space;
  result += name + string_rbracket;
  return result;
}

string
EntrySymbol_l::EntrySymbol_to_string(EntrySymbol symbol)
{
  string result;
  for (; symbol > 0; symbol /= 10) result += symbol % 10 + '0';
  if (result.empty()) result = '0';
  reverse(result.begin(), result.end());
  return result;
}
