/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "error.h"
#include "automata_l.h"

Automata_l::Automata_l()
{

}

Automata_l::~Automata_l()
{

}

string
Automata_l::addState(string name)
{
  string result;
  // check if we already don't have state with such name
  map<State, State_l>::iterator iter_map = states.begin();
  for (; iter_map != states.end(); ++iter_map)
    if (iter_map->second.getName() == name) break;
  if (iter_map != states.end()) { 
    setError(E_A_STATE_NAME_EXISTS); return result;
  }
  State new_state = automata->addState();
  if (new_state == State_NULL) return result;
  State_l new_state_l(name);
  states[new_state] = new_state_l;
  // return look
  result = new_state_l.getLook(new_state, getStateFlags(new_state));
  return result;
}

State_l::t_flags
Automata_l::getStateFlags(State state)
{
  State_l::t_flags result = (State_l::t_flags)0;
  if (automata->getStartState() == state) 
    result = (State_l::t_flags)(result | State_l::F_START);
  set<State> end_states = automata->getEndStates();
  set<State>::iterator iter_st = end_states.find(state);
  if (iter_st != end_states.end())
    result = (State_l::t_flags)(result | State_l::F_END);
  return result;
}

bool
Automata_l::removeState(string state_id)
{
  State state = string_to_State(state_id);
  if (automata->removeState(state)) { states.erase(state); return true; }
  return false;
}

State
Automata_l::string_to_State(string state_id)
{
  State result = 0;
  int mult_factor = 1;
  reverse(state_id.begin(), state_id.end());
  for (string::const_iterator iter_s = state_id.begin();
       iter_s != state_id.end(); ++iter_s, mult_factor *= 10)
    result += (*iter_s - '0') * mult_factor;
  return result;
}

string
Automata_l::editState(string state_id, string new_name)
{
  string result;
  State state = string_to_State(state_id);
  map<State, State_l>::iterator iter_st_f = states.find(state);
  if (iter_st_f == states.end()) { setError(E_A_STATE_BAD); return result; }
  // check if we already don't have state with such name
  iter_st_f = states.begin();
  for (; iter_st_f != states.end(); ++iter_st_f)
    if (iter_st_f->second.getName() == new_name) break;
  if (iter_st_f != states.end()) { 
    setError(E_A_STATE_NAME_EXISTS); return result;
  }
  State_l new_state_l(new_name);
  states[state] = new_state_l;
  // return look
  result = new_state_l.getLook(state, getStateFlags(state));
  return result;
}

string
Automata_l::addEntrySymbol(string name)
{
  string result;
  // check if we already don't have entry symbol with such name
  map<EntrySymbol, EntrySymbol_l>::iterator iter_sy = entry_symbols.begin();
  for (; iter_sy != entry_symbols.end(); ++iter_sy) 
    if (iter_sy->second.getName() == name) break;
  if (iter_sy != entry_symbols.end()) {
    setError(E_A_ENTRY_SYMBOL_NAME_EXISTS); return result;
  }
  EntrySymbol new_symbol = automata->addEntrySymbol();
  if (new_symbol == EntrySymbol_NULL) return result;
  EntrySymbol_l new_symbol_l(name);
  entry_symbols[new_symbol] = new_symbol_l;
  // return look
  result = new_symbol_l.getLook(new_symbol);
  return result;
}

bool
Automata_l::removeEntrySymbol(string symbol_id)
{
  EntrySymbol symbol = string_to_EntrySymbol(symbol_id);
  if (automata->removeEntrySymbol(symbol)) { 
    entry_symbols.erase(symbol); return true;
  }
  return false;
}

EntrySymbol
Automata_l::string_to_EntrySymbol(string symbol_id)
{
  EntrySymbol result = 0;
  int mult_factor = 1;
  reverse(symbol_id.begin(), symbol_id.end());
  for (string::const_iterator iter_s = symbol_id.begin();
       iter_s != symbol_id.end(); ++iter_s, mult_factor *= 10)
    result += (*iter_s - '0') * mult_factor;
  return result;
}

string
Automata_l::editEntrySymbol(string symbol_id, string new_name)
{
  string result;
  EntrySymbol symbol = string_to_EntrySymbol(symbol_id);
  map<EntrySymbol, EntrySymbol_l>::iterator 
    iter_sy_f = entry_symbols.find(symbol);
  if (iter_sy_f == entry_symbols.end()) { 
    setError(E_A_ENTRY_SYMBOL_BAD); return result;
  }
  // check if we already don't have entry symbol with such name
  iter_sy_f = entry_symbols.begin();
  for (; iter_sy_f != entry_symbols.end(); ++iter_sy_f)
    if (iter_sy_f->second.getName() == new_name) break;
  if (iter_sy_f != entry_symbols.end()) { 
    setError(E_A_ENTRY_SYMBOL_NAME_EXISTS); return result;
  }
  EntrySymbol_l new_symbol_l(new_name);
  entry_symbols[symbol] = new_symbol_l;
  // return look
  result = new_symbol_l.getLook(symbol);
  return result;
}

inline bool
Automata_l::setStartState(string state_id)
{
  return automata->setStartState(string_to_State(state_id));
}

inline bool
Automata_l::setEndState(string state_id)
{
  return automata->setEndState(string_to_State(state_id));
}

inline bool
Automata_l::resetEndState(string state_id)
{
  return automata->resetEndState(string_to_State(state_id));
}
