/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// abstract class of the automata look

#ifndef __AUTOMATA_L_H__
#define __AUTOMATA_L_H__

#include "automata_elements_l.h"
#include "automata.h"
#include <map>

class Automata_l {
 protected:
  map<State, State_l> states;
  map<EntrySymbol, EntrySymbol_l> entry_symbols;
  Automata* automata;
 public:
  Automata_l();
  virtual ~Automata_l();
  string addState(string name);
  // adds new state with name name
  // returns: _State_look_
  // errors: E_A_STATE_NAME_EXISTS, E_A_STATE_MAX
  bool removeState(string state_id);
  // removes state state_id
  // returns: if successful
  // errors: E_A_STATE_BAD
  string editState(string state_id, string new_name);
  // changes the name of state state_id to new_name
  // returns: _State_look_
  // errors: E_A_STATE_BAD, E_A_STATE_NAME_EXISTS 
  string addEntrySymbol(string name);
  // adds new entry symbol with name name
  // returns: _EntrySymbol_look_
  // errors: E_A_ENTRY_SYMBOL_NAME_EXISTS, E_A_ENTRY_SYMBOL_MAX
  bool removeEntrySymbol(string symbol_id);
  // removes entry symbol symbol_id
  // returns: if successful
  // errors: E_A_ENTRY_SYMBOL_BAD
  string editEntrySymbol(string symbol_id, string new_name);
  // changes the name of entry symbol symbol_id to new_name
  // returns: _EntrySymbol_look_
  // errors: E_A_ENTRY_SYMBOL_BAD, E_A_ENTRY_SYMBOL_NAME_EXISTS
  bool setStartState(string state_id);
  // sets start state to state_id
  // returns: if successful
  // errors: E_A_STATE_BAD
  bool setEndState(string state_id);
  // sets state_id to be end state
  // returns: if successful
  // errors: E_A_STATE_BAD
  bool resetEndState(string state_id);
  // sets state_id not to be end state
  // returns: if successful
  // errors: E_A_STATE_BAD
  virtual string getLook() = 0;
  // returns: the look of the whole automata
  virtual bool checkSentence(string sentence) = 0;
  // returns: if sentence sentence is accepted by automata
  virtual bool isDeterministic() = 0;
  // returns: if automata is deterministic
 protected:
  State string_to_State(string state_id);
  State_l::t_flags getStateFlags(State state);
  EntrySymbol string_to_EntrySymbol(string symbol_id);
 public:
  // for conversion functions only
  map<State, State_l> *
    getStates_p() { return &states; }
  map<EntrySymbol, EntrySymbol_l> *
    getEntrySymbols_p() { return &entry_symbols; }
};

#endif  
