/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the contextfree grammar

#ifndef __CONTEXTFREE_GRAMMAR_H__
#define __CONTEXTFREE_GRAMMAR_H__

#include "grammar.h"
#include <map>

class ContextfreeGrammar : public Grammar {
 public:
  ContextfreeGrammar();
  ~ContextfreeGrammar();
  bool addRule(Rule& rule, vector<Symbol> left, vector<Symbol> right);
  // adds new rule consisting of left and right part
  // returns: if successful, in rule new rule
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD, E_GC_RULE_BAD
  bool addRule(vector<Symbol> left, vector<Symbol> right);
  // adds new rule consisting of left and right part
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD, E_GC_RULE_BAD
  set<NonterminalSymbol> getRedundantSymbols();
  // returns: nonterminal redundant symbols
  // note: this method returns only nonterminal symbols, which the string of
  // terminal symbols cannot be generated from
  set<Symbol> getInaccessibleSymbols();
  // returns: inaccessible symbols
  void removeRedundantSymbols();
  // removes all redundant and inaccessible symbols
  set<Rule> getERules();
  // returns: e-rules
  bool isERulesfree();
  // returns: if grammar is e-rules-free
  NonterminalSymbol removeERules();
  // removes e-rules
  // returns: start symbol (can be new)
  set<Rule> getSimpleRules();
  // returns: simple rules
  void removeSimpleRules();
  // removes all simple rules
  // errors: E_GC_NOT_ERULES_FREE
  bool isSelf();
  // returns: if grammar is self
  NonterminalSymbol toSelf();
  // converts grammar to equivalent self one
  // returns: start symbol (can be new)
  bool isinChomskyNormalForm();
  // returns: if the grammar is in Chomsky normal form
  map<NonterminalSymbol, vector<Symbol> > toChomskyNormalForm();
  // converts grammar to equivalent one in Chomsky normal form
  // returns: map of new nonterminal symbol to vector of symbols pairs
  // errors: E_GC_NOT_SELF, E_GC_HAS_SIMPLE_RULES
  set<NonterminalSymbol> getRecursiveSymbols();
  // returns: all recursive symbols
  bool isRecursive();
  // returns: if grammar is recursive
  map<NonterminalSymbol, NonterminalSymbol> eliminateLeftRecursion();
  // eliminates left recursion from grammar
  // returns: map of new nonterminal symbol to nonterminal symbol pairs
  // errors: E_GC_NOT_SELF
  bool isinGreibachNormalForm();
  // returns: if the grammar is in Greibach normal form
  map<NonterminalSymbol, TerminalSymbol> toGreibachNormalForm();
  // converts grammar to equivalent one in Greibach normal form
  // returns: map of new nonterminal symbol to terminal symbol pairs
  // errors: E_GC_NOT_SELF, E_GC_IS_LEFT_RECURSIVE
 protected:
  set<Rule> modifyRule(Rule rule, set<NonterminalSymbol> ns);
  bool hasCycle();
  bool isLeftRecursive();
 public:
  // for the sort function of the STL library, instantiated from 
  // toGreibachNormalForm, we need ordering function object and object with
  // overloaded operator () is function object
  bool operator()(NonterminalSymbol nt1, NonterminalSymbol nt2);
};

#endif
