/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "contextfree_grammar_l.h"

const char string_space = ' ';
const char string_lbracket = '(';
const char string_rbracket = ')';

ContextfreeGrammar_l::ContextfreeGrammar_l()
{
  grammar = new ContextfreeGrammar;
  Grammar_l::grammar = grammar;
}

ContextfreeGrammar_l::~ContextfreeGrammar_l()
{
  delete grammar;
  grammar = NULL;
  Grammar_l::grammar = NULL;
}

string
ContextfreeGrammar_l::getRedundantSymbols()
{
  string result;
  result += string_lbracket; result += "nonterminal_symbols";
  set<NonterminalSymbol> r_symbols = grammar->getRedundantSymbols();
  for (set<NonterminalSymbol>::const_iterator iter_sy = r_symbols.begin();
       iter_sy != r_symbols.end(); ++iter_sy)
    result += string_space +
      nonterminal_symbols[*iter_sy].getLook
      (*iter_sy, getNonterminalSymbolFlags(*iter_sy));
  result += string_rbracket;
  return result;
}

string
ContextfreeGrammar_l::getInaccessibleSymbols()
{
  string result;
  result += string_lbracket; result += "symbols";
  set<Symbol> i_symbols = grammar->getInaccessibleSymbols();
  for (set<Symbol>::const_iterator iter_sy = i_symbols.begin();
       iter_sy != i_symbols.end(); ++iter_sy)
    if (grammar->isSymbolNonterminal(*iter_sy))
      result += string_space + 
	nonterminal_symbols[*iter_sy].getLook
	(*iter_sy, getNonterminalSymbolFlags(*iter_sy));
    else
      result += string_space + terminal_symbols[*iter_sy].getLook(*iter_sy);
  result += string_rbracket;
  return result;
}

void
ContextfreeGrammar_l::removeRedundantSymbols()
{
  grammar->removeRedundantSymbols();
  map<NonterminalSymbol, NonterminalSymbol_l> new_nonterminal_symbols;
  map<TerminalSymbol, TerminalSymbol_l> new_terminal_symbols;
  set<NonterminalSymbol> ntss = grammar->getNonterminalSymbols();
  for (set<NonterminalSymbol>::const_iterator iter_sy = ntss.begin();
       iter_sy != ntss.end(); ++iter_sy)
    new_nonterminal_symbols[*iter_sy] = nonterminal_symbols[*iter_sy];
  set<TerminalSymbol> tss = grammar->getTerminalSymbols();
  for (set<TerminalSymbol>::const_iterator iter_sy = tss.begin();
       iter_sy != tss.end(); ++iter_sy)
    new_terminal_symbols[*iter_sy] = terminal_symbols[*iter_sy];
  nonterminal_symbols = new_nonterminal_symbols;
  terminal_symbols = new_terminal_symbols;
}

string
ContextfreeGrammar_l::getERules()
{
  return getRulesLook(grammar->getERules());
}

string
ContextfreeGrammar_l::getRulesLook(set<Rule> rules)
{
  string result;
  result += string_lbracket; result += "rules";
  for (set<Rule>::const_iterator iter_r = rules.begin(); iter_r != rules.end();
       ++iter_r) {
    result += string_space; result += string_lbracket; result += "rule";
    result += string_space; result += string_lbracket; result += "left";
    for (vector<Symbol>::const_iterator iter_sy = iter_r->left.begin(); 
         iter_sy != iter_r->left.end(); ++iter_sy)
      if (grammar->isSymbolNonterminal(*iter_sy))
        result += string_space + 
          nonterminal_symbols[*iter_sy].getLook
          (*iter_sy, getNonterminalSymbolFlags(*iter_sy));
      else
        result += string_space + terminal_symbols[*iter_sy].getLook(*iter_sy);
    result += string_rbracket;
    result += string_space; result += string_lbracket; result += "right";
    for (vector<Symbol>::const_iterator iter_sy = iter_r->right.begin(); 
         iter_sy != iter_r->right.end(); ++iter_sy)
      if (grammar->isSymbolNonterminal(*iter_sy))
        result += string_space + 
          nonterminal_symbols[*iter_sy].getLook
          (*iter_sy, getNonterminalSymbolFlags(*iter_sy));
      else
        result += string_space + terminal_symbols[*iter_sy].getLook(*iter_sy);
    result += string_rbracket;
    result += string_rbracket;
  }
  result += string_rbracket;
  return result;
}

bool
ContextfreeGrammar_l::isERulesfree()
{
  return grammar->isERulesfree();
}

void
ContextfreeGrammar_l::removeERules()
{
  NonterminalSymbol old_start_symbol = grammar->removeERules();
  NonterminalSymbol start_symbol = grammar->getStartSymbol();
  if (old_start_symbol != start_symbol) {
    string new_name = nonterminal_symbols[old_start_symbol].getName() + "'";
    nonterminal_symbols[start_symbol] = NonterminalSymbol_l(new_name);
  }
}

string
ContextfreeGrammar_l::getSimpleRules()
{
  return getRulesLook(grammar->getSimpleRules());
}

void
ContextfreeGrammar_l::removeSimpleRules()
{
  grammar->removeSimpleRules();
}

bool
ContextfreeGrammar_l::isSelf()
{
  return grammar->isSelf();
}

void
ContextfreeGrammar_l::toSelf()
{
  NonterminalSymbol old_start_symbol = grammar->toSelf();
  NonterminalSymbol start_symbol = grammar->getStartSymbol();
  if (old_start_symbol != start_symbol) {
    string new_name = nonterminal_symbols[old_start_symbol].getName() + "'";
    nonterminal_symbols[start_symbol] = NonterminalSymbol_l(new_name);
  }
  map<NonterminalSymbol, NonterminalSymbol_l> new_nonterminal_symbols;
  map<TerminalSymbol, TerminalSymbol_l> new_terminal_symbols;
  set<NonterminalSymbol> ntss = grammar->getNonterminalSymbols();
  for (set<NonterminalSymbol>::const_iterator iter_sy = ntss.begin();
       iter_sy != ntss.end(); ++iter_sy)
    new_nonterminal_symbols[*iter_sy] = nonterminal_symbols[*iter_sy];
  set<TerminalSymbol> tss = grammar->getTerminalSymbols();
  for (set<TerminalSymbol>::const_iterator iter_sy = tss.begin();
       iter_sy != tss.end(); ++iter_sy)
    new_terminal_symbols[*iter_sy] = terminal_symbols[*iter_sy];
  nonterminal_symbols = new_nonterminal_symbols;
  terminal_symbols = new_terminal_symbols;
}

bool
ContextfreeGrammar_l::isinChomskyNormalForm()
{
  return grammar->isinChomskyNormalForm();
}

void
ContextfreeGrammar_l::toChomskyNormalForm()
{
  map<NonterminalSymbol, vector<Symbol> > 
    symbol_map = grammar->toChomskyNormalForm();
  if (symbol_map.empty()) return;
  vector<Symbol> symbls;
  string name;
  for (map<NonterminalSymbol, vector<Symbol> >::const_iterator
	 iter_map = symbol_map.begin(); iter_map != symbol_map.end();
       ++iter_map) {
    // build nonterminal symbols names from old symbols names
    symbls = iter_map->second;
    if (symbls.size() == 1) {
      name = terminal_symbols[symbls.front()].getName() + "'";
      nonterminal_symbols[iter_map->first] = NonterminalSymbol_l(name);
    }
    else {
      name = "<";
      for (vector<Symbol>::const_iterator iter_sy = symbls.begin();
	   iter_sy != symbls.end(); ++iter_sy)
	if (grammar->isSymbolNonterminal(*iter_sy))
	  name += nonterminal_symbols[*iter_sy].getName();
	else name += terminal_symbols[*iter_sy].getName();
      name += ">";
      nonterminal_symbols[iter_map->first] = NonterminalSymbol_l(name);
    }
  }
}

string
ContextfreeGrammar_l::getRecursiveSymbols()
{
  string result;
  result += string_lbracket; result += "nonterminal_symbols";
  set<NonterminalSymbol> r_symbols = grammar->getRecursiveSymbols();
  for (set<NonterminalSymbol>::const_iterator iter_sy = r_symbols.begin();
       iter_sy != r_symbols.end(); ++iter_sy)
    result += string_space +
      nonterminal_symbols[*iter_sy].getLook
      (*iter_sy, getNonterminalSymbolFlags(*iter_sy));
  result += string_rbracket;
  return result;
}

bool
ContextfreeGrammar_l::isRecursive()
{
  return grammar->isRecursive();
}

void
ContextfreeGrammar_l::eliminateLeftRecursion()
{
  map<NonterminalSymbol, NonterminalSymbol> 
    symbol_map = grammar->eliminateLeftRecursion();
  if (symbol_map.empty()) return;
  string name;
  for (map<NonterminalSymbol, NonterminalSymbol>::const_iterator
	 iter_map = symbol_map.begin(); iter_map != symbol_map.end();
       ++iter_map) {
    // build nonterminal symbols names from old symbols names
    name = nonterminal_symbols[iter_map->second].getName() + "'";
    nonterminal_symbols[iter_map->first] = NonterminalSymbol_l(name);
  }
}

bool
ContextfreeGrammar_l::isinGreibachNormalForm()
{
  return grammar->isinGreibachNormalForm();
}

void
ContextfreeGrammar_l::toGreibachNormalForm()
{
  map<NonterminalSymbol, TerminalSymbol>
    symbol_map = grammar->toGreibachNormalForm();
  if (symbol_map.empty()) return;
  string name;
  for (map<NonterminalSymbol, TerminalSymbol>::const_iterator
	 iter_map = symbol_map.begin(); iter_map != symbol_map.end();
       ++iter_map) {
    // build nonterminal symbols names from old symbols names
    name = terminal_symbols[iter_map->second].getName() + "'";
    nonterminal_symbols[iter_map->first] = NonterminalSymbol_l(name);
  }
}

const ContextfreeGrammar_l &
ContextfreeGrammar_l::operator=(const ContextfreeGrammar_l &g)
{
  if (this != &g) {
    nonterminal_symbols = g.nonterminal_symbols;
    terminal_symbols = g.terminal_symbols;
    *grammar = *(g.grammar);
    *(Grammar_l::grammar) = *(g.grammar);
  }
  return *this;
}
