/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the contextfree grammar look

#ifndef __CONTEXTFREE_GRAMMAR_L_H__
#define __CONTEXTFREE_GRAMMAR_L_H__

#include "grammar_l.h"
#include "contextfree_grammar.h"

class ContextfreeGrammar_l : public Grammar_l {
 protected:
  ContextfreeGrammar* grammar;
 public:
  ContextfreeGrammar_l();
  ~ContextfreeGrammar_l();
  string getRedundantSymbols();
  // returns: nonterminal redundant symbols looks
  // note: this method returns only nonterminal symbols, which the string of
  // terminal symbols cannot be generated from
  // returns: (nonterminal_symbols _NonterminalSymbol_look_ 
  // _NonterminalSymbol_look_ ...)
  string getInaccessibleSymbols();
  // returns: inaccessible symbols looks
  // returns: (symbols _Symbol_look_ _Symbol_look_ ...) 
  void removeRedundantSymbols();
  // removes all redundant and inaccessible symbols
  string getERules();
  // returns: e-rules looks
  // returns: (rules 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...)) 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...)))
  bool isERulesfree();
  // returns: if grammar is e-rules-free
  void removeERules();
  // removes e-rules
  string getSimpleRules();
  // returns: simple rules looks
  // returns: (rules 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...)) 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...)))
  void removeSimpleRules();
  // removes all simple rules
  // errors: E_GC_NOT_ERULES_FREE
  bool isSelf();
  // returns: if grammar is self
  void toSelf();
  // converts grammar to equivalent self one
  bool isinChomskyNormalForm();
  // returns: if the grammar is in Chomsky normal form
  void toChomskyNormalForm();
  // converts grammar to equivalent one in Chomsky normal form
  // errors: E_GC_NOT_SELF, E_GC_HAS_SIMPLE_RULES
  string getRecursiveSymbols();
  // returns: all recursive symbols looks
  // returns: (nonterminal_symbols _NonterminalSymbol_look_ 
  // _NonterminalSymbol_look_ ...)
  bool isRecursive();
  // returns: if grammar is recursive
  void eliminateLeftRecursion();
  // eliminates left recursion from grammar
  // errors: E_GC_NOT_SELF
  bool isinGreibachNormalForm();
  // returns: if the grammar is in Greibach normal form
  void toGreibachNormalForm();
  // converts grammar to equivalent one in Greibach normal form
  // errors: E_GC_NOT_SELF, E_GC_IS_LEFT_RECURSIVE
 protected:
  string getRulesLook(set<Rule> rules);
 public:
  const ContextfreeGrammar_l &operator=(const ContextfreeGrammar_l &g);
  // for conversion functions only
  ContextfreeGrammar **
    getGrammar_h() { return &grammar; }
};

#endif
