/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "conversions.h"

FiniteAutomata
RegularGrammar_to_FiniteAutomata(RegularGrammar *grammar,
				 map<NonterminalSymbol, State> &nts_st_map,
				 State &new_state,
				 map<TerminalSymbol, EntrySymbol> &ts_es_map)
{
  FiniteAutomata *automata = new FiniteAutomata;
  set<NonterminalSymbol> ntss = grammar->getNonterminalSymbols();
  NonterminalSymbol start_nts = grammar->getStartSymbol();
  State st;
  for (set<NonterminalSymbol>::const_iterator iter_sy = ntss.begin();
       iter_sy != ntss.end(); ++iter_sy) {
    st = automata->addState();
    nts_st_map[*iter_sy] = st;
    if (*iter_sy == start_nts) automata->setStartState(st);
  }
  new_state = automata->addState();
  set<TerminalSymbol> tss = grammar->getTerminalSymbols();
  for (set<TerminalSymbol>::const_iterator iter_sy = tss.begin();
       iter_sy != tss.end(); ++iter_sy)
    ts_es_map[*iter_sy] = automata->addEntrySymbol();
  set<Rule> rules = grammar->getRules();
  for (set<Rule>::const_iterator iter_r = rules.begin();
       iter_r != rules.end(); ++iter_r) {
    if ((iter_r->left.front() == start_nts) && (iter_r->right.empty()))
      automata->setEndState(nts_st_map[start_nts]);
    if (iter_r->right.size() == 1) st = new_state;
    else st = nts_st_map[iter_r->right.back()];
    automata->addTransition
      (nts_st_map[iter_r->left.front()], ts_es_map[iter_r->right.front()], st);
  }
  automata->setEndState(new_state);
  return *automata;
}

RegularGrammar
FiniteAutomata_to_RegularGrammar(FiniteAutomata *automata,
				 map<State, NonterminalSymbol> &st_nts_map, 
				 map<EntrySymbol, TerminalSymbol> &es_ts_map)
{
  RegularGrammar *grammar = new RegularGrammar;
  // automata is already deterministic
  set<State> sts = automata->getStates();
  State start_st = automata->getStartState();
  NonterminalSymbol nts;
  for (set<State>::const_iterator iter_st = sts.begin(); iter_st != sts.end();
       ++iter_st) {
    nts = grammar->addNonterminalSymbol();
    st_nts_map[*iter_st] = nts;
    if (*iter_st == start_st) grammar->setStartSymbol(nts);
  }
  set<EntrySymbol> ess = automata->getEntrySymbols();
  for (set<EntrySymbol>::const_iterator iter_sy = ess.begin();
       iter_sy != ess.end(); ++iter_sy)
    es_ts_map[*iter_sy] = grammar->addTerminalSymbol();
  set<State> end_states = automata->getEndStates();
  set<State>::iterator iter_st_f = end_states.find(start_st);
  if (iter_st_f != end_states.end())
    grammar->addRule
      (vector<Symbol>(1, st_nts_map[start_st]), vector<Symbol>());
  set<Transition> transitions = automata->getTransitions();
  vector<Symbol> vect;
  for (set<Transition>::const_iterator iter_t = transitions.begin();
       iter_t != transitions.end(); ++iter_t) {
    vect.clear(); vect.push_back(iter_t->entry_symbol);
    iter_st_f = end_states.find(iter_t->to_state);
    if (iter_st_f != end_states.end()) 
      grammar->addRule(vector<Symbol>(1, iter_t->from_state), vect);
    vect.push_back(iter_t->to_state);
    grammar->addRule(vector<Symbol>(1, iter_t->from_state), vect);
  }
  return *grammar;
}

StackAutomata
ContextfreeGrammar_to_StackAutomata
(ContextfreeGrammar *grammar,vector<State> &new_states,
 map<TerminalSymbol, EntrySymbol> &ts_es_map,
 map<NonterminalSymbol, StackSymbol> &nts_ss_map,
 map<TerminalSymbol, StackSymbol> &ts_ss_map, StackSymbol &new_stack_symbol,
 bool top_down)
{
  StackAutomata *automata = new StackAutomata;
  map<Symbol, StackSymbol> s_ss_map;
  int c = 2; if (top_down) c = 3;
  for (int i = 0; i < c; ++i) new_states.push_back(automata->addState());
  automata->setStartState(new_states.front());
  automata->setEndState(new_states.back());
  set<NonterminalSymbol> ntss = grammar->getNonterminalSymbols();
  for (set<NonterminalSymbol>::const_iterator iter_sy = ntss.begin();
       iter_sy != ntss.end(); ++iter_sy)
    s_ss_map[*iter_sy] = nts_ss_map[*iter_sy] = automata->addStackSymbol();
  set<TerminalSymbol> tss = grammar->getTerminalSymbols();
  for (set<TerminalSymbol>::const_iterator iter_sy = tss.begin();
       iter_sy != tss.end(); ++iter_sy) {
    ts_es_map[*iter_sy] = automata->addEntrySymbol();
    s_ss_map[*iter_sy] = ts_ss_map[*iter_sy] = automata->addStackSymbol();
    if (top_down)
      automata->addTransition
	(new_states[1], ts_es_map[*iter_sy], new_states[1],
	 vector<StackSymbol>(1, ts_ss_map[*iter_sy]), vector<StackSymbol>());
    else
      automata->addTransition
	(new_states.front(), ts_es_map[*iter_sy], new_states.front(),
	 vector<StackSymbol>(), vector<StackSymbol>(1, ts_ss_map[*iter_sy]));
  }
  new_stack_symbol = automata->addStackSymbol();
  automata->setStartStackSymbol(new_stack_symbol);
  if (top_down)
    automata->addTransition
      (new_states.front(), EntrySymbol_NULL, new_states[1],
       vector<StackSymbol>(), 
       vector<StackSymbol>(1, nts_ss_map[grammar->getStartSymbol()]));
  set<Rule> rules = grammar->getRules();
  vector<StackSymbol> stack;
  for (set<Rule>::const_iterator iter_r = rules.begin();
       iter_r != rules.end(); ++iter_r) {
    stack.clear();
    for (vector<Symbol>::const_iterator iter_sy = iter_r->right.begin();
	 iter_sy != iter_r->right.end(); ++iter_sy)
      stack.push_back(s_ss_map[*iter_sy]);
    if (top_down) {
      reverse(stack.begin(), stack.end());
      automata->addTransition
	(new_states[1], EntrySymbol_NULL, new_states[1], 
	 vector<StackSymbol>(1, nts_ss_map[iter_r->left.front()]), stack);
    } else
      automata->addTransition
	(new_states.front(), EntrySymbol_NULL, new_states.front(), 
	 stack, vector<StackSymbol>(1, nts_ss_map[iter_r->left.front()]));
  }
  if (top_down)
    automata->addTransition
      (new_states[1], EntrySymbol_NULL, new_states.back(),
       vector<StackSymbol>(1, new_stack_symbol), vector<StackSymbol>());
  else {
    stack.clear(); stack.push_back(new_stack_symbol);
    stack.push_back(nts_ss_map[grammar->getStartSymbol()]);
    automata->addTransition
      (new_states.front(), EntrySymbol_NULL, new_states.back(), stack,
       vector<StackSymbol>());
  }
  return *automata;
}

NonterminalSymbol
find_nts(vector<State> sts_v, StackSymbol ss,
	 map<NonterminalSymbol, vector<State> > nts_sts_map, 
	 map<NonterminalSymbol, StackSymbol> nts_ss_map)
{
  map<NonterminalSymbol, vector<State> >::const_iterator
    iter_map1 = nts_sts_map.begin();
  map<NonterminalSymbol, StackSymbol>::const_iterator
    iter_map2 = nts_ss_map.begin();
  for (; iter_map2 != nts_ss_map.end(); ++iter_map1, ++iter_map2)
    if ((iter_map1->second == sts_v) && (iter_map2->second == ss))
      return iter_map2->first;
  return NonterminalSymbol_NULL;
}

set<vector<Symbol> >
build_right_parts_of_rule(State first_st, vector<StackSymbol> stack,
			  State last_st,
			  map<NonterminalSymbol, vector<State> > &nts_sts_map, 
			  map<NonterminalSymbol, StackSymbol> &nts_ss_map,
			  set<State> sts, ContextfreeGrammar *grammar,
			  vector<Symbol> nts_v)
{
  set<vector<Symbol> > result;
  NonterminalSymbol nts;
  vector<State> sts_v; sts_v.push_back(first_st);
  if (stack.size() == 1) { 
    sts_v.push_back(last_st);
    nts = find_nts(sts_v, stack.front(), nts_sts_map, nts_ss_map);
    if (nts == NonterminalSymbol_NULL) {
      nts = grammar->addNonterminalSymbol();
      nts_sts_map[nts] = sts_v; nts_ss_map[nts] = stack.front();
    }
    nts_v.push_back(nts);
    result.insert(nts_v);
  }
  else {
    vector<StackSymbol> stk;
    set<vector<Symbol> > res;
    for (set<State>::const_iterator iter_st = sts.begin();
	 iter_st != sts.end(); ++iter_st) {
      sts_v.push_back(*iter_st);
      nts = find_nts(sts_v, stack.front(), nts_sts_map, nts_ss_map);
      if (nts == NonterminalSymbol_NULL) {
	nts = grammar->addNonterminalSymbol();
	nts_sts_map[nts] = sts_v; nts_ss_map[nts] = stack.front();
      }
      nts_v.push_back(nts);
      stk = stack; stk.erase(stk.begin());
      res = 
	build_right_parts_of_rule(*iter_st, stk, last_st, nts_sts_map,
				 nts_ss_map, sts, grammar, nts_v);
      result.insert(res.begin(), res.end());
      nts_v.pop_back(); sts_v.pop_back();
    }
  }
  return result;
}

ContextfreeGrammar
StackAutomata_to_ContextfreeGrammar
(StackAutomata *automata, map<NonterminalSymbol, vector<State> > &nts_sts_map,
 map<NonterminalSymbol, StackSymbol> &nts_ss_map, 
 map<EntrySymbol, TerminalSymbol> &es_ts_map)
{
  ContextfreeGrammar *grammar = new ContextfreeGrammar;
  // automata is already top stack working and empty stack checking
  set<EntrySymbol> ess = automata->getEntrySymbols();
  for (set<EntrySymbol>::const_iterator iter_sy = ess.begin();
       iter_sy != ess.end(); ++iter_sy)
    es_ts_map[*iter_sy] = grammar->addTerminalSymbol();
  set<State> sts = automata->getStates();
  State start_st = automata->getStartState();
  StackSymbol start_ss = automata->getStartStackSymbol();
  NonterminalSymbol nts, snts = grammar->addNonterminalSymbol();
  grammar->setStartSymbol(snts);
  vector<State> sts_v; sts_v.push_back(start_st);
  for (set<State>::const_iterator iter_st = sts.begin(); iter_st != sts.end();
       ++iter_st) {
    nts = grammar->addNonterminalSymbol();
    sts_v.push_back(*iter_st);
    nts_sts_map[nts] = sts_v; nts_ss_map[nts] = start_ss;
    sts_v.pop_back();
    grammar->addRule(vector<Symbol>(1, snts), vector<Symbol>(1, nts));
  }
  set<vector<Symbol> > rights;
  vector<StackSymbol> stk;
  set<StackTransition> transitions = automata->getTransitions();
  for (set<StackTransition>::const_iterator iter_t = transitions.begin();
       iter_t != transitions.end(); ++iter_t) {
    sts_v.clear(); sts_v.push_back(iter_t->from_state);
    if (iter_t->to_stack.empty()) {
      sts_v.push_back(iter_t->to_state);
      nts = find_nts
	(sts_v, iter_t->from_stack.front(), nts_sts_map, nts_ss_map);
      if (nts == NonterminalSymbol_NULL) {
	nts = grammar->addNonterminalSymbol();
	nts_sts_map[nts] = sts_v; nts_ss_map[nts] = iter_t->from_stack.front();
      }
      if (iter_t->entry_symbol != EntrySymbol_NULL)
	grammar->addRule
	  (vector<Symbol>(1, nts), vector<Symbol>(1, iter_t->entry_symbol));
      else grammar->addRule(vector<Symbol>(1, nts), vector<Symbol>());
    }
    else {
      for (set<State>::const_iterator iter_st = sts.begin();
	   iter_st != sts.end(); ++iter_st) {
	sts_v.push_back(*iter_st);
	nts = find_nts
	  (sts_v, iter_t->from_stack.front(), nts_sts_map, nts_ss_map);
	if (nts == NonterminalSymbol_NULL) {
	  nts = grammar->addNonterminalSymbol();
	  nts_sts_map[nts] = sts_v;
	  nts_ss_map[nts] = iter_t->from_stack.front();
	}
	stk = iter_t->to_stack; reverse(stk.begin(), stk.end());
	if (iter_t->entry_symbol != EntrySymbol_NULL)
	  rights = 
	    build_right_parts_of_rule(iter_t->to_state, stk, *iter_st,
				      nts_sts_map, nts_ss_map, sts, grammar, 
				      vector<Symbol>(1, iter_t->entry_symbol));
	else
	  rights = 
	    build_right_parts_of_rule(iter_t->to_state, stk, *iter_st,
				      nts_sts_map, nts_ss_map, sts, grammar, 
				      vector<Symbol>());
	for (set<vector<Symbol> >::const_iterator iter_v = rights.begin();
	     iter_v != rights.end(); ++iter_v)
	  grammar->addRule(vector<Symbol>(1, nts), *iter_v);
	sts_v.pop_back();
      }
    }
  }
  // there can be redundant or inaccessible nonterminal symbols
  grammar->removeRedundantSymbols();
  set<NonterminalSymbol> ntss = grammar->getNonterminalSymbols();
  map<NonterminalSymbol, vector<State> > _nts_sts_map;
  map<NonterminalSymbol, StackSymbol> _nts_ss_map;
  for (set<NonterminalSymbol>::const_iterator iter_sy = ntss.begin();
       iter_sy != ntss.end(); ++iter_sy) {
    nts = *iter_sy;
    if (nts != snts) {
      _nts_sts_map[nts] = nts_sts_map[nts];
      _nts_ss_map[nts] = nts_ss_map[nts];
    }
  }
  nts_sts_map = _nts_sts_map;
  nts_ss_map = _nts_ss_map;
  return *grammar;
}
