/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CONVERSIONS_H__
#define __CONVERSIONS_H__

#include "finite_automata.h"
#include "regular_grammar.h"
#include "stack_automata.h"
#include "contextfree_grammar.h"

FiniteAutomata
RegularGrammar_to_FiniteAutomata(RegularGrammar *grammar,
				 map<NonterminalSymbol, State> &nts_st_map,
				 State &new_state, map<TerminalSymbol,
				 EntrySymbol> &ts_es_map);
// converts regular grammar grammar to equivalent finite automata
// returns: new finite automata, map of nonterminal symbol to state in 
// nts_st_map, new end state in new_state and map of terminal symbol to entry
// symbol in ts_es_map

RegularGrammar
FiniteAutomata_to_RegularGrammar(FiniteAutomata *automata, map<State,
				 NonterminalSymbol> &st_nts_map,
				 map<EntrySymbol, TerminalSymbol> &es_ts_map);
// converts finite automata automata to equivalent regular grammar
// returns: new regular grammar, map of state to nonterminal symbol in 
// st_nts_map and map of entry symbol to terminal symbol in ts_es_map

StackAutomata
ContextfreeGrammar_to_StackAutomata
(ContextfreeGrammar *grammar, vector<State> &new_states,
 map<TerminalSymbol, EntrySymbol> &ts_es_map,
 map<NonterminalSymbol, StackSymbol> &nts_ss_map,
 map<TerminalSymbol, StackSymbol> &ts_ss_map, StackSymbol &new_stack_symbol,
 bool top_down);
// converts contextfree grammar grammar to equivalent stack automata top-down
// or bottom-up method specified in top_down
// returns: new stack automata, states in nes_states, map of terminal
// symbol to entry symbol in ts_es_map, map of nonterminal symbol to stack
// symbol in nts_ss_map, map of terminal symbol to stack symbol in
// ts_ss_map and start stack symbol in new_stack_symbol

ContextfreeGrammar
StackAutomata_to_ContextfreeGrammar
(StackAutomata *automata, map<NonterminalSymbol, vector<State> > &nts_sts_map,
 map<NonterminalSymbol, StackSymbol> &nts_ss_map, 
 map<EntrySymbol, TerminalSymbol> &es_ts_map);
// converts stack automata automata to equivalent contextfree grammar
// returns: new contextfree grammar, map of two states to nonterminal symbol in
// nts_sts_map, map of stack symbol to nonterminal symbol in nts_ss_map, map
// of entry symbol to terminal symbol in es_ts_map

#endif
