/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "conversions_l.h"
#include "conversions.h"

FiniteAutomata_l
RegularGrammar_l_to_FiniteAutomata_l(RegularGrammar_l *grammar)
{
  FiniteAutomata_l *automata = new FiniteAutomata_l;
  map<State, State_l> *states = automata->getStates_p();
  map<EntrySymbol, EntrySymbol_l> *entry_symbols = 
    automata->getEntrySymbols_p();
  map<NonterminalSymbol, NonterminalSymbol_l> *nonterminal_symbols = 
    grammar->getNonterminalSymbols_p();
  map<TerminalSymbol, TerminalSymbol_l> *terminal_symbols = 
    grammar->getTerminalSymbols_p();
  map<NonterminalSymbol, State> nts_st_map;
  State new_state;
  map<TerminalSymbol, EntrySymbol> ts_es_map;
  **(automata->getAutomata_h()) = 
    RegularGrammar_to_FiniteAutomata
    (*(grammar->getGrammar_h()), nts_st_map, new_state, ts_es_map);
  for (map<NonterminalSymbol, State>::const_iterator
	 iter_map = nts_st_map.begin(); iter_map != nts_st_map.end();
       ++iter_map)
    (*states)[iter_map->second] = 
      State_l((*nonterminal_symbols)[iter_map->first].getName());
  (*states)[new_state] = State_l("A");
  for (map<TerminalSymbol, EntrySymbol>::const_iterator
	 iter_map = ts_es_map.begin(); iter_map != ts_es_map.end(); ++iter_map)
    (*entry_symbols)[iter_map->second] = 
      EntrySymbol_l((*terminal_symbols)[iter_map->first].getName());
  return *automata;
}

RegularGrammar_l
FiniteAutomata_l_to_RegularGrammar_l(FiniteAutomata_l *automata)
{
  RegularGrammar_l *grammar = new RegularGrammar_l;
  // automata must be deterministic
  FiniteAutomata_l d_automata = *automata;
  if (!d_automata.isDeterministic()) d_automata.toDeterministic();
  map<NonterminalSymbol, NonterminalSymbol_l> *nonterminal_symbols = 
    grammar->getNonterminalSymbols_p();
  map<TerminalSymbol, TerminalSymbol_l> *terminal_symbols = 
    grammar->getTerminalSymbols_p();
  map<State, State_l> *states = d_automata.getStates_p();
  map<EntrySymbol, EntrySymbol_l> *entry_symbols = 
    d_automata.getEntrySymbols_p();
  map<State, NonterminalSymbol> st_nts_map;
  map<EntrySymbol, TerminalSymbol> es_ts_map;
  **(grammar->getGrammar_h()) = 
    FiniteAutomata_to_RegularGrammar
    (*(d_automata.getAutomata_h()), st_nts_map, es_ts_map);
  for (map<State, NonterminalSymbol>::const_iterator
	 iter_map = st_nts_map.begin(); iter_map != st_nts_map.end();
       ++iter_map)
    (*nonterminal_symbols)[iter_map->second] = 
      NonterminalSymbol_l((*states)[iter_map->first].getName());
  for (map<EntrySymbol, TerminalSymbol>::const_iterator
	 iter_map = es_ts_map.begin(); iter_map != es_ts_map.end(); ++iter_map)
    (*terminal_symbols)[iter_map->second] = 
      TerminalSymbol_l((*entry_symbols)[iter_map->first].getName());
  return *grammar;
}

StackAutomata_l
ContextfreeGrammar_l_to_StackAutomata_l(ContextfreeGrammar_l *grammar,
					bool top_down)
{
  StackAutomata_l *automata = new StackAutomata_l;
  map<State, State_l> *states = automata->getStates_p();
  map<EntrySymbol, EntrySymbol_l> *entry_symbols = 
    automata->getEntrySymbols_p();
  map<StackSymbol, StackSymbol_l> *stack_symbols = 
    automata->getStackSymbols_p();
  map<NonterminalSymbol, NonterminalSymbol_l> *nonterminal_symbols = 
    grammar->getNonterminalSymbols_p();
  map<TerminalSymbol, TerminalSymbol_l> *terminal_symbols = 
    grammar->getTerminalSymbols_p();
  vector<State> new_states;
  map<TerminalSymbol, EntrySymbol> ts_es_map;
  map<NonterminalSymbol, StackSymbol> nts_ss_map;
  map<TerminalSymbol, StackSymbol> ts_ss_map;
  StackSymbol new_stack_symbol;
  **(automata->getAutomata_h()) = 
    ContextfreeGrammar_to_StackAutomata
    (*(grammar->getGrammar_h()), new_states, ts_es_map, nts_ss_map, ts_ss_map,
     new_stack_symbol, top_down);
  if (top_down) {
    (*states)[new_states.front()] = State_l("p");
    (*states)[new_states[1]] = State_l("q");
  }
  else (*states)[new_states.front()] = State_l("q");
  (*states)[new_states.back()] = State_l("r");
  for (map<TerminalSymbol, EntrySymbol>::const_iterator
	 iter_map = ts_es_map.begin(); iter_map != ts_es_map.end(); ++iter_map)
    (*entry_symbols)[iter_map->second] = 
      EntrySymbol_l((*terminal_symbols)[iter_map->first].getName());
  for (map<NonterminalSymbol, StackSymbol>::const_iterator
	 iter_map = nts_ss_map.begin(); iter_map != nts_ss_map.end();
       ++iter_map)
    (*stack_symbols)[iter_map->second] = 
      StackSymbol_l((*nonterminal_symbols)[iter_map->first].getName());
  for (map<TerminalSymbol, StackSymbol>::const_iterator
	 iter_map = ts_ss_map.begin(); iter_map != ts_ss_map.end(); ++iter_map)
    (*stack_symbols)[iter_map->second] = 
      StackSymbol_l((*terminal_symbols)[iter_map->first].getName());
  (*stack_symbols)[new_stack_symbol] = StackSymbol_l("#");
  return *automata;
}

ContextfreeGrammar_l
StackAutomata_l_to_ContextfreeGrammar_l(StackAutomata_l *automata)
{
  ContextfreeGrammar_l *grammar = new ContextfreeGrammar_l;
  // automata must be top stack working and empty stack checking
  StackAutomata_l t_automata = *automata;
  if (!t_automata.isTopStackWorking()) t_automata.toTopStackWorking();
  if (!t_automata.isEmptyStackChecking()) t_automata.toEmptyStackChecking();
  map<NonterminalSymbol, NonterminalSymbol_l> *nonterminal_symbols = 
    grammar->getNonterminalSymbols_p();
  map<TerminalSymbol, TerminalSymbol_l> *terminal_symbols = 
    grammar->getTerminalSymbols_p();
  map<State, State_l> *states = t_automata.getStates_p();
  map<EntrySymbol, EntrySymbol_l> *entry_symbols = 
    t_automata.getEntrySymbols_p();
  map<StackSymbol, StackSymbol_l> *stack_symbols = 
    t_automata.getStackSymbols_p();
  map<NonterminalSymbol, vector<State> > nts_sts_map;
  map<NonterminalSymbol, StackSymbol> nts_ss_map;
  map<EntrySymbol, TerminalSymbol> es_ts_map;
  **(grammar->getGrammar_h()) = 
    StackAutomata_to_ContextfreeGrammar
    (*(t_automata.getAutomata_h()), nts_sts_map, nts_ss_map, es_ts_map);
  for (map<EntrySymbol, TerminalSymbol>::const_iterator
	 iter_map = es_ts_map.begin(); iter_map != es_ts_map.end(); ++iter_map)
    (*terminal_symbols)[iter_map->second] = 
      TerminalSymbol_l((*entry_symbols)[iter_map->first].getName());
  (*nonterminal_symbols)[(**(grammar->getGrammar_h())).getStartSymbol()] 
    = NonterminalSymbol_l("S");
  string name;
  map<NonterminalSymbol, vector<State> >::const_iterator
    iter_map1 = nts_sts_map.begin();
  map<NonterminalSymbol, StackSymbol>::const_iterator
    iter_map2 = nts_ss_map.begin();
  for (; iter_map2 != nts_ss_map.end(); ++iter_map1, ++iter_map2) {
    name = "["; name += (*states)[iter_map1->second.front()].getName();
    name += (*stack_symbols)[iter_map2->second].getName();
    name += (*states)[iter_map1->second.back()].getName() + "]";
    (*nonterminal_symbols)[iter_map2->first] = NonterminalSymbol_l(name);
  }
  return *grammar;
}
