/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the finite automata

#ifndef __FINITE_AUTOMATA_H__
#define __FINITE_AUTOMATA_H__

#include "automata.h"
#include <map>

class FiniteAutomata : public Automata {
 public:
  FiniteAutomata();
  ~FiniteAutomata();
  bool addTransition(Transition& transition, State from_state,
		     EntrySymbol symbol, State to_state);
  // adds new transition consisting of from_state, symbol and to_state
  // returns: if successful, in transition new transition
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD
  bool addTransition(State from_state, EntrySymbol symbol, State to_state);
  // adds new transition formed from from_state, symbol and to_state
  // returns: if successful
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD
  bool removeTransition(State from_state, EntrySymbol symbol, State to_state);
  // removes transition consisting of from_state, symbol and to_state
  // returns: if successful
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD, E_A_TRANSITION_BAD
  bool setConfiguration(State state, vector<EntrySymbol> sentence);
  // sets configuration consisting of state and sentence
  // returns: if successful
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD
  void getConfiguration(State& state, vector<EntrySymbol>& sentence);
  // returns: configuration state in state and sentence in sentence
  set<State> getTransitionDestinations();
  // returns: states, into which automata can do one transition from actual
  // configuration
  bool doTransition(State state);
  // does one transition to state state from actual configuration
  // returns: if successful
  // errors: E_AF_SENTENCE_EMPTY, E_AF_STATE_CANT_REACH
  bool checkSentence(vector<EntrySymbol> sentence);
  // returns: if sentence sentence is accepted by automata
  bool isDeterministic();
  // returns: if automata is deterministic
  map<State, set<State> > toDeterministic();
  // converts automata to equivalent deterministic one
  // returns: map of new state to set of old states pairs
  set<State> getInaccessibleStates();
  // returns: all inaccessible states
  map<State, set<State> > toReduced();
  // converts automata to equivalent reduced one
  // returns: map of new_state to set of old states pairs
  // errors: E_AF_NOT_DETERMINISTIC
 protected:
  set<State> _getTransitionDestinations(State state, EntrySymbol symbol);
  bool _doTransition(State& from_state, State to_state,
		     vector<EntrySymbol>& sentence);
  bool _checkSentence(State state, vector<EntrySymbol> sentence);
  set<set<State> > getEquivalentionRelation();
  bool areStatesinOneSet(State st1, State st2, set<set<State> > sets);
};

#endif
