/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the finite automata look

#ifndef __FINITE_AUTOMATA_L_H__
#define __FINITE_AUTOMATA_L_H__

#include "automata_l.h"
#include "finite_automata.h"

class FiniteAutomata_l : public Automata_l {
 protected:
  FiniteAutomata* automata;
 public:
  FiniteAutomata_l();
  ~FiniteAutomata_l();
  bool addTransition(string from_state_id, string symbol_id,
		     string to_state_id);
  // adds new transition consisting of from_state_id, symbol_id and to_state_id
  // returns: if successful
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD
  bool removeTransition(string from_state_id, string symbol_id,
			string to_state_id);
  // removes transition consisting of from_state_id, symbol_id and to_state_id
  // returns: if successful
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD, E_A_TRANSITION_BAD
  bool setConfiguration(string state_id, string sentence);
  // sets configuration consisting of state_id and sentence
  // returns: if successful
  // sentence: _EntrySymbol_id_ _EntrySymbol_id_ ...
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD
  void getConfiguration(string& state, string& sentence);
  // returns: configuration state in state and sentence in sentence
  // state: _State_look_
  // sentence: (entry_symbols _EntrySymbol_look_ _EntrySymbol_look_ ...)
  string getLook();
  // returns: the look of the whole automata
  // returns: (finite_automata (states _State_look_ _State_look_ ...) 
  // (entry_symbols _EntrySymbol_look_ _EntrySymbol_look_ ...) 
  // (transitions 
  // (transition _from_state_look_ _entry_symbol_look_ _to_state_look_) 
  // (transition _from_state_look_ _entry_symbol_look_ _to_state_look_) 
  // ...))
  string getTransitionDestinations();
  // returns: state looks of states, into which automata can do one transition
  // from actual configuration
  // returns: (states _State_look_ _State_look_ ...)
  bool doTransition(string state_id);
  // does one transition to state state_id from actual configuration
  // returns: if successful
  // errors: E_AF_SENTENCE_EMPTY, E_AF_STATE_CANT_REACH
  bool checkSentence(string sentence);
  // returns: if sentence sentence is accepted by automata
  // sentence: _EntrySymbol_id_ _EntrySymbol_id_ ...
  bool isDeterministic();
  // returns: if automata is deterministic
  void toDeterministic();
  // converts automata to equivalent deterministic one
  string getInaccessibleStates();
  // returns: all inaccessible states looks
  // returns: (states _State_look_ _State_look_ ...)
  void toReduced();
  // converts automata to equivalent reduced one
  // errors: E_AF_NOT_DETERMINISTIC
 public:
  const FiniteAutomata_l &operator=(const FiniteAutomata_l &a);
  // for conversion functions only
  FiniteAutomata **
    getAutomata_h() { return &automata; }
};

#endif
