/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of a grammar (any)

#ifndef __GRAMMAR_H__
#define __GRAMMAR_H__

#include "grammar_elements.h"
#include <set>

class Grammar {
 protected:
  set<NonterminalSymbol> nonterminal_symbols;
  set<TerminalSymbol> terminal_symbols;
  set<Rule> rules;
  NonterminalSymbol start_symbol;
 public:
  Grammar();
  virtual ~Grammar();
  NonterminalSymbol addNonterminalSymbol();
  // adds new nonterminal symbol
  // returns: new nonterminal symbol
  // errors: E_G_NONTERMINAL_SYMBOL_MAX
  bool removeNonterminalSymbol(NonterminalSymbol symbol);
  // removes nonterminal symbol symbol
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD
  set<NonterminalSymbol> getNonterminalSymbols();
  // returns: all nonterminal symbols
  TerminalSymbol addTerminalSymbol();
  // adds new terminal symbol
  // returns: new terminal symbol
  // errors: E_G_TERMINAL_SYMBOL_MAX
  bool removeTerminalSymbol(TerminalSymbol symbol);
  // removes terminal symbol symbol
  // returns: if successful
  // errors: E_G_TERMINAL_SYMBOL_BAD
  set<TerminalSymbol> getTerminalSymbols();
  // returns: all terminal symbols
  virtual bool addRule(Rule& rule, vector<Symbol> left, vector<Symbol> right);
  // adds new rule consisting of left and right part
  // returns: if successful, in rule new rule
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD
  virtual bool addRule(vector<Symbol> left, vector<Symbol> right);
  // adds new rule consisting of left and right part
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD
  bool removeRule(Rule rule);
  // removes rule rule
  // returns: if successful 
  // errors: E_G_RULE_BAD
  bool removeRule(vector<Symbol> left, vector<Symbol> right);
  // removes rule consisting of left and right part
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD, E_G_RULE_BAD
  set<Rule> getRules();
  // returns: all rules
  bool setStartSymbol(NonterminalSymbol symbol);
  // sets start nonterminal symbol to symbol
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD
  NonterminalSymbol getStartSymbol();
  // returns: start nonterminal symbol
  enum t_classification 
  { C_BOUNDLESS = 1, C_CONTEXT = 2, C_CONTEXTFREE = 3, C_REGULAR = 4 };
  t_classification getClassification();
  // returns: grammar classification
  NonterminalSymbol toExtended();
  // converts grammar to equivalent extended one
  // returns: old start symbol
 protected:
  set<Rule> find_rules_with_nonterminal_symbol(NonterminalSymbol symbol);
  set<Rule> find_rules_with_terminal_symbol(TerminalSymbol symbol);
  bool isRegular();
  bool isContextfree();
  bool isContext();
 public:
  // for the Grammar_l class only
  bool isSymbolNonterminal(Symbol symbol);
  bool isSymbolTerminal(Symbol symbol);
};

#endif
