/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the (any) grammar look

#ifndef __GRAMMAR_L_H__
#define __GRAMMAR_L_H__

#include "grammar_elements_l.h"
#include "grammar.h"
#include <map>

class Grammar_l {
 protected:
  map<NonterminalSymbol, NonterminalSymbol_l> nonterminal_symbols;
  map<TerminalSymbol, TerminalSymbol_l> terminal_symbols;
  Grammar* grammar;
 public:
  Grammar_l();
  virtual ~Grammar_l();
  string addNonterminalSymbol(string name);
  // adds new nonterminal symbol with name
  // returns: _NonterminalSymbol_look_
  // errors: E_G_NONTERMINAL_SYMBOL_NAME_EXISTS, E_G_NONTERMINAL_SYMBOL_MAX
  bool removeNonterminalSymbol(string symbol_id);
  // removes nonterminal symbol symbol_id
  // returns: if successful
  // errors: E_G_NONTERMINAL_SYMBOL_BAD
  string editNonterminalSymbol(string symbol_id, string new_name);
  // changes the name of nonterminal symbol symbol_id to new_name
  // returns: _NonterminalSymbol_look_
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_NONTERMINAL_SYMBOL_NAME_EXISTS
  string addTerminalSymbol(string name);
  // adds new terminal symbol with name
  // returns: _TerminalSymbol_look_
  // errors: E_G_TERMINAL_SYMBOL_NAME_EXISTS, E_G_TERMINAL_SYMBOL_MAX
  bool removeTerminalSymbol(string symbol_id);
  // removes terminal symbol symbol_id
  // returns: if successful
  // errors: E_G_TERMINAL_SYMBOL_BAD
  string editTerminalSymbol(string symbol_id, string new_name);
  // changes the name of terminal symbol symbol_id to new_name
  // returns: _TerminalSymbol_look_
  // errors: E_G_TERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_NAME_EXISTS
  bool setStartSymbol(string symbol_id);
  // sets start nonterminal symbol to symbol_id
  // errors: E_G_NONTERMINAL_SYMBOL_BAD
  bool addRule(string left, string right);
  // adds new rule consisting of left and right part
  // returns: if successful
  // left: _symbol_id_ _symbol_id_ ...
  // right: _symbol_id_ _symbol_id_ ...
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD, E_GR_RULE_BAD
  // or E_GC_RULE_BAD
  bool removeRule(string left, string right);
  // removes rule consisting of left and right part
  // returns: if successful
  // left: _symbol_id_ _symbol_id_ ...
  // right: _symbol_id_ _symbol_id_ ...
  // errors: E_G_NONTERMINAL_SYMBOL_BAD, E_G_TERMINAL_SYMBOL_BAD, E_G_RULE_BAD
  string getLook();
  // returns: the look of the whole grammar
  // returns: (grammar (nonterminal_symbols _NonterminalSymbol_look_ 
  // _NonterminalSymbol_look_ ...) 
  // (terminal_symbols _TerminalSymbol_look_ _TerminalSymbol_look_ ...) 
  // (rules 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...)) 
  // (rule (left _symbol_look_ _symbol_look_ ...) (right _symbol_look_ 
  // _symbol_look_ ...))))
  string getClassification();
  // returns: grammar classification
  // returns: "BOUNDLESS" or "CONTEXT" or "CONTEXTFREE" or "REGULAR"
  void toExtended();
  // converts grammar to equivalent extended one
 protected:
  NonterminalSymbol string_to_Symbol(string symbol_id);
  NonterminalSymbol_l::t_flags getNonterminalSymbolFlags
    (NonterminalSymbol symbol);
 public:
  virtual const Grammar_l &operator=(const Grammar_l &g);
  // for conversion functions only
  map<NonterminalSymbol, NonterminalSymbol_l> *
    getNonterminalSymbols_p() { return &nonterminal_symbols; }
  map<TerminalSymbol, TerminalSymbol_l> *
    getTerminalSymbols_p() { return &terminal_symbols; }
};

#endif  
