/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "error.h"
#include "regular_grammar.h"

RegularGrammar::RegularGrammar()
{

}

RegularGrammar::~RegularGrammar()
{

}

bool
RegularGrammar::addRule(Rule &rule, vector<Symbol> left, vector<Symbol> right)
{
  // check if all symbols in both parts are in the [non]terminal_symbols set
  // CHOICE: if not, add it - now NO
  set<Symbol>::iterator iter_sy_f;
  for (vector<Symbol>::const_iterator iter_sy = left.begin();
       iter_sy != left.end(); ++iter_sy)
    if (isSymbolNonterminal(*iter_sy)) {
      iter_sy_f = nonterminal_symbols.find(*iter_sy);
      if (iter_sy_f == nonterminal_symbols.end()) {
	setError(E_G_NONTERMINAL_SYMBOL_BAD); return false;
      }
    } else {
      iter_sy_f = terminal_symbols.find(*iter_sy);
      if (iter_sy_f == terminal_symbols.end()) {
	setError(E_G_TERMINAL_SYMBOL_BAD); return false;
      }
    }
  for (vector<Symbol>::const_iterator iter_sy = right.begin();
       iter_sy != right.end(); ++iter_sy)
    if (isSymbolNonterminal(*iter_sy)) {
      iter_sy_f = nonterminal_symbols.find(*iter_sy);
      if (iter_sy_f == nonterminal_symbols.end()) {
	setError(E_G_NONTERMINAL_SYMBOL_BAD); return false;
      }
    } else {
      iter_sy_f = terminal_symbols.find(*iter_sy);
      if (iter_sy_f == terminal_symbols.end()) {
	setError(E_G_TERMINAL_SYMBOL_BAD); return false;
      }
    }
  // check if the rule suits criterions for the regular grammar rule
  if (left.size() != 1) { setError(E_GR_RULE_BAD); return false; }
  if (!isSymbolNonterminal(left.front())) { 
    setError(E_GR_RULE_BAD); return false;
  }
  if ((right.size() != 1) && (right.size() != 2)) { 
    setError(E_GR_RULE_BAD); return false;
  }
  if (!isSymbolTerminal(right.front())) { 
    setError(E_GR_RULE_BAD); return false;
  }
  if (right.size() == 2) if (!isSymbolNonterminal(right[1])) { 
    setError(E_GR_RULE_BAD); return false;
  }
  rule.left = left;
  rule.right = right;
  rules.insert(rule);
  return true;
}

bool
RegularGrammar::addRule(vector<Symbol> left, vector<Symbol> right)
{
  Rule rule;
  return addRule(rule, left, right);
}
