/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the stack automata look

#ifndef __STACK_AUTOMATA_L_H__
#define __STACK_AUTOMATA_L_H__

#include "automata_l.h"
#include "stack_automata_elements_l.h"
#include "stack_automata.h"

class StackAutomata_l : public Automata_l {
 protected:
  map<StackSymbol, StackSymbol_l> stack_symbols;
  StackAutomata* automata;
 public:
  StackAutomata_l();
  ~StackAutomata_l();
  string addStackSymbol(string name);
  // adds new stack symbol with name name
  // returns: _StackSymbol_look_
  // errors: E_AS_STACK_SYMBOL_NAME_EXISTS, E_AS_STACK_SYMBOL_MAX
  bool removeStackSymbol(string symbol_id);
  // removes stack symbol symbol_id
  // returns: if successful
  // errors: E_AS_STACK_SYMBOL_BAD
  string editStackSymbol(string symbol_id, string new_name);
  // changes the name of stack symbol symbol_id to new_name
  // returns: _StackSymbol_look_
  // errors: E_AS_STACK_SYMBOL_BAD, E_AS_STACK_SYMBOL_NAME_EXISTS
  bool setStartStackSymbol(string symbol_id);
  // sets start stack symbol to symbol_id
  // returns: if successful
  // errors: E_AS_STACK_SYMBOL_BAD
  bool addTransition(string from_state_id, string symbol_id,
		     string to_state_id, string from_stack, string to_stack);
  // adds new transition consisting of from_state_id, symbol_id, to_state_id,
  // from_stack and to_stack
  // returns: if successful
  // from_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // to_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // no entry symbol => symbol_id = "0"
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD, E_AS_STACK_SYMBOL_BAD
  bool removeTransition(string from_state_id, string symbol_id,
			string to_state_id, string from_stack,
			string to_stack);
  // removes transition consisting of from_state_id, symbol_id, to_state_id,
  // from_stack and to_stack
  // returns: if successful
  // from_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // to_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // no entry symbol => symbol_id = "0"
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD, E_AS_STACK_SYMBOL_BAD,
  // E_A_TRANSITION_BAD
  bool setConfiguration(string state_id, string sentence, string stack);
  // sets configuration consisting of state_id, sentence and stack
  // returns: if successful
  // sentence: _EntrySymbol_id_ _EntrySymbol_id_ ...
  // stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // errors: E_A_STATE_BAD, E_A_ENTRY_SYMBOL_BAD, E_AS_STACK_SYMBOL_BAD
  void getConfiguration(string& state, string& sentence, string& stack);
  // returns: configuration state in state, sentence in sentence and stack in
  // stack
  // state: _State_look_
  // sentence: (entry_symbols _EntrySymbol_look_ _EntrySymbol_look_ ...)
  // stack: (stack _StackSymbol_look_ _StackSymbol_look_ ...)
  string getLook();
  // returns: the look of the whole automata
  // returns: (stack_automata (states _State_look_ _State_look_ ...) 
  // (entry_symbols _EntrySymbol_look_ _EntrySymbol_look_ ...) 
  // (stack_symbols _StackSymbol_look_ _StackSymbol_look_ ...) 
  // (transitions 
  // (transition _from_state_look_ _entry_symbol_look_ _to_state_look_ 
  // (from_stack _from_stack_symbol_look_ _from_stack_symbol_look_) 
  // (to_stack _to_stack_symbol_look_ _to_stack_symbol_look_)) 
  // (transition _from_state_look_ _entry_symbol_look_ _to_state_look_ 
  // (from_stack _from_stack_symbol_look_ _from_stack_symbol_look_) 
  // (to_stack _to_stack_symbol_look_ _to_stack_symbol_look_)) 
  // ...))
  // transition with no entry symbol => _entry_symbol_name_ = "-"
  string getFromStacks();
  // returns: looks of stacks, which can be popped from stack (from all
  // transitions)
  // returns: (stacks (stack _StackSymbol_look_ _StackSymbol_look_ ...) 
  // (stack _StackSymbol_look_ _StackSymbol_look_ ...) 
  // ...)
  string getTransitionDestinations(string from_stack);
  // returns: state and stack pairs looks, into which automata can do one
  // transition from actual configuration when popping from_stack from stack
  // from_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // returns: (state-stack-pairs (_State_look_ 
  // (stack _StackSymbol_look_ _StackSymbol_look_ ...)) 
  // (_State_look_ 
  // (stack _StackSymbol_look_ _StackSymbol_look_ ...)) 
  // ...)
  bool doTransition(string state_id, string from_stack, string to_stack);
  // does one transition to state state_id from actual configuration when
  // popping from_stack and pushing to_stack to stack
  // returns: if successful
  // from_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // to_stack: _StackSymbol_id_ _StackSymbol_id_ ...
  // errors: E_AS_STATE_STACK_CANT_REACH, E_AS_STACK_CANT_POP
  bool checkSentence(string sentence);
  // returns: if sentence sentence is accepted by automata
  // sentence: _EntrySymbol_id_ _EntrySymbol_id_ ...
  bool isDeterministic();
  // returns: if automata is deterministic
  bool isTopStackWorking();
  // returns: if automata is top stack working
  void toTopStackWorking();
  // converts automata to equivalent top stack working one
  // _new_state_name_ = "_original_state_name__number"
  bool isEmptyStackChecking();
  // returns: if automata is empty stack checking the sentence
  void toEmptyStackChecking();
  // converts automata to equivalent empty stack checking one
  // _new_state_name_ = "F"
  void toEndStatesChecking();
  // converts automata to equivalent end states checking one
  // _end_state_name_ = "F"
 protected:
  StackSymbol string_to_StackSymbol(string symbol_id);
  StackSymbol_l::t_flags getStackSymbolFlags(StackSymbol symbol);
  string int_to_string(int i);
 public:
  const StackAutomata_l &operator=(const StackAutomata_l &a);
  // for conversion functions only
  map<StackSymbol, StackSymbol_l> *
    getStackSymbols_p() { return &stack_symbols; }
  StackAutomata **
    getAutomata_h() { return &automata; }
};

#endif
