/* GAAE - GAAE Ain't an Automata Editor
 * Copyright (C) 2001 Jan Outrata <outrataj@upcase.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// template of class of undo-redo

#ifndef __UNDOREDO_H__
#define __UNDOREDO_H__

template<class T> class UndoRedo {
 protected:
  struct Item {
    T data;
    Item *prev;
    Item *next;
  };
  Item *tail;
  Item *actual;
 public:
  UndoRedo();
  ~UndoRedo();
  void save(T *d);
  // saves T at the end
  void removeLast();
  // removes the last T
  T *undo(T *d);
  // returns: previous T (to actual) or d
  T *redo(T *d);
  // returns: next T (to actual) or d
 protected:
  void insertLast(T *d);
};

template<class T>
UndoRedo<T>::UndoRedo()
{
  tail = actual = NULL;
}

template<class T>
UndoRedo<T>::~UndoRedo()
{
  Item *p;
  while (tail != NULL) { p = tail; tail = tail->prev; delete p; }
}

template<class T> void
UndoRedo<T>::save(T *d)
{
  Item *p;
  if (actual != NULL)
    while (tail != actual->prev) { p = tail; tail = tail->prev; delete p; }
  if (tail != NULL) tail->next = NULL;
  insertLast(d);
  actual = NULL;
}

template<class T> void
UndoRedo<T>::insertLast(T *d)
{
  Item *p = new Item;
  p->data = *d;
  p->next = NULL; p->prev = tail;
  if (tail != NULL) tail->next = p;
  tail = p;
}

template<class T> void
UndoRedo<T>::removeLast()
{
  if ((actual == NULL) && (tail != NULL)) {
    Item *p = tail;
    tail = p->prev;
    if (p->prev != NULL) p->prev->next = p->next;
    delete p;
  }
}

template<class T> T *
UndoRedo<T>::undo(T *d)
{
  T *result = new T; *result = *d;
  if (tail != NULL) {
    if (actual == NULL) { insertLast(d); actual = tail; }
    if (actual->prev != NULL) { 
      actual = actual->prev; *result = actual->data;
    }
  }
  return result;
}

template<class T> T *
UndoRedo<T>::redo(T *d)
{
  T *result = new T; *result = *d;
  if ((actual != NULL) && (actual != tail)) {
    actual = actual->next;
    *result = actual->data;
    if (actual == tail) { actual = NULL; removeLast(); }
  }
  return result;
}

#endif
